@@ -1,5 +1,4 @@
-//go:build !ci && !ios && !js && !wasm && !test_web_driver
-// +build !ci,!ios,!js,!wasm,!test_web_driver
+//go:build !ci && !ios && !wasm && !test_web_driver && !mobile
 
 package app
 
@@ -10,19 +9,30 @@ package app
 #include <AppKit/AppKit.h>
 
 bool isBundled();
-bool isDarkMode();
 void watchTheme();
 */
 import "C"
 import (
 	"net/url"
 	"os"
+	"os/exec"
 	"path/filepath"
 
 	"fyne.io/fyne/v2"
-	"fyne.io/fyne/v2/theme"
 )
 
+func (a *fyneApp) OpenURL(url *url.URL) error {
+	cmd := exec.Command("open", url.String())
+	cmd.Stdin, cmd.Stdout, cmd.Stderr = os.Stdin, os.Stdout, os.Stderr
+	return cmd.Run()
+}
+
+// SetSystemTrayIcon sets a custom image for the system tray icon.
+// You should have previously called `SetSystemTrayMenu` to initialise the menu icon.
+func (a *fyneApp) SetSystemTrayIcon(icon fyne.Resource) {
+	a.Driver().(systrayDriver).SetSystemTrayIcon(icon)
+}
+
 // SetSystemTrayMenu creates a system tray item and attaches the specified menu.
 // By default this will use the application icon.
 func (a *fyneApp) SetSystemTrayMenu(menu *fyne.Menu) {
@@ -31,19 +41,6 @@ func (a *fyneApp) SetSystemTrayMenu(menu *fyne.Menu) {
 	}
 }
 
-// SetSystemTrayIcon sets a custom image for the system tray icon.
-// You should have previously called `SetSystemTrayMenu` to initialise the menu icon.
-func (a *fyneApp) SetSystemTrayIcon(icon fyne.Resource) {
-	a.Driver().(systrayDriver).SetSystemTrayIcon(icon)
-}
-
-func defaultVariant() fyne.ThemeVariant {
-	if C.isDarkMode() {
-		return theme.VariantDark
-	}
-	return theme.VariantLight
-}
-
 func rootConfigDir() string {
 	homeDir, _ := os.UserHomeDir()
 
@@ -51,12 +48,6 @@ func rootConfigDir() string {
 	return filepath.Join(desktopConfig, "fyne")
 }
 
-func (a *fyneApp) OpenURL(url *url.URL) error {
-	cmd := a.exec("open", url.String())
-	cmd.Stdin, cmd.Stdout, cmd.Stderr = os.Stdin, os.Stdout, os.Stderr
-	return cmd.Run()
-}
-
 //export themeChanged
 func themeChanged() {
 	fyne.CurrentApp().Settings().(*settings).setupTheme()
