@@ -1,5 +1,4 @@
 //go:build !ci && android
-// +build !ci,android
 
 package app
 
@@ -46,10 +45,6 @@ func (a *fyneApp) SendNotification(n *fyne.Notification) {
 	})
 }
 
-func defaultVariant() fyne.ThemeVariant {
-	return systemTheme
-}
-
 func rootConfigDir() string {
 	filesDir := os.Getenv("FILESDIR")
 	if filesDir == "" {
