@@ -1,5 +1,4 @@
-//go:build !ci && ios
-// +build !ci,ios
+//go:build !ci && ios && !mobile
 
 package app
 
@@ -18,8 +17,6 @@ import (
 	"net/url"
 	"path/filepath"
 	"unsafe"
-
-	"fyne.io/fyne/v2"
 )
 
 func rootConfigDir() string {
@@ -34,7 +31,3 @@ func (a *fyneApp) OpenURL(url *url.URL) error {
 
 	return nil
 }
-
-func defaultVariant() fyne.ThemeVariant {
-	return systemTheme
-}
