@@ -1,5 +1,4 @@
 //go:build !ci && wasm
-// +build !ci,wasm
 
 package app
 
@@ -9,7 +8,7 @@ import (
 	"syscall/js"
 )
 
-func (app *fyneApp) OpenURL(url *url.URL) error {
+func (a *fyneApp) OpenURL(url *url.URL) error {
 	window := js.Global().Call("open", url.String(), "_blank", "")
 	if window.Equal(js.Null()) {
 		return fmt.Errorf("Unable to open a new window/tab for URL: %v.", url)
