@@ -4,14 +4,12 @@ import (
 	"testing"
 
 	"github.com/stretchr/testify/assert"
-
-	"fyne.io/fyne/v2/internal"
 )
 
 func TestStore_RootURI(t *testing.T) {
 	id := "io.fyne.test"
 	a := &fyneApp{uniqueID: id}
-	d := makeStoreDocs(id, &internal.InMemoryPreferences{}, &store{a: a})
+	d := makeStoreDocs(id, &store{a: a})
 
 	w, err := d.Create("test")
 	assert.Nil(t, err)
