@@ -4,11 +4,12 @@ import "fyne.io/fyne/v2"
 
 // Refresh instructs the containing canvas to refresh the specified obj.
 func Refresh(obj fyne.CanvasObject) {
-	if fyne.CurrentApp() == nil || fyne.CurrentApp().Driver() == nil {
+	app := fyne.CurrentApp()
+	if app == nil || app.Driver() == nil {
 		return
 	}
 
-	c := fyne.CurrentApp().Driver().CanvasForObject(obj)
+	c := app.Driver().CanvasForObject(obj)
 	if c != nil {
 		c.Refresh(obj)
 	}
@@ -16,11 +17,12 @@ func Refresh(obj fyne.CanvasObject) {
 
 // repaint instructs the containing canvas to redraw, even if nothing changed.
 func repaint(obj fyne.CanvasObject) {
-	if fyne.CurrentApp() == nil || fyne.CurrentApp().Driver() == nil {
+	app := fyne.CurrentApp()
+	if app == nil || app.Driver() == nil {
 		return
 	}
 
-	c := fyne.CurrentApp().Driver().CanvasForObject(obj)
+	c := app.Driver().CanvasForObject(obj)
 	if c != nil {
 		if paint, ok := c.(interface{ SetDirty() }); ok {
 			paint.SetDirty()
