@@ -1,13 +1,13 @@
 package commands
 
-import "fyne.io/fyne/v2/cmd/fyne/internal/metadata"
+import "fyne.io/fyne/v2/internal/metadata"
 
 type appData struct {
 	icon, Name        string
 	AppID, AppVersion string
 	AppBuild          int
 	ResGoString       string
-	Release           bool
+	Release, rawIcon  bool
 	CustomMetadata    map[string]string
 	VersionAtLeast2_3 bool
 }
