@@ -28,8 +28,7 @@ var macAppStoreCategories = []string{
 
 // Release returns the cli command for bundling release builds of fyne applications
 func Release() *cli.Command {
-	r := &Releaser{}
-	r.appData = &appData{}
+	r := NewReleaser()
 
 	return &cli.Command{
 		Name:  "release",
@@ -121,6 +120,12 @@ func Release() *cli.Command {
 				Value:       "",
 				Destination: &r.icon,
 			},
+			&cli.BoolFlag{
+				Name:        "use-raw-icon",
+				Usage:       "Skip any OS-specific icon pre-processing",
+				Value:       false,
+				Destination: &r.rawIcon,
+			},
 			&cli.GenericFlag{
 				Name:  "metadata",
 				Usage: "Specify custom metadata key value pair that you do not want to store in your FyneApp.toml (key=value)",
@@ -143,6 +148,13 @@ type Releaser struct {
 	password     string
 }
 
+// NewReleaser returns a command that can handle the packaging a GUI apps for release from local Fyne source code.
+func NewReleaser() *Releaser {
+	r := &Releaser{}
+	r.appData = &appData{}
+	return r
+}
+
 // AddFlags adds the flags for interacting with the release command.
 //
 // Deprecated: Access to the individual cli commands are being removed.
@@ -176,14 +188,14 @@ func (r *Releaser) PrintHelp(indent string) {
 //
 // Deprecated: A better version will be exposed in the future.
 func (r *Releaser) Run(params []string) {
+	r.Packager.distribution = true
+	r.Packager.release = true
+
 	if err := r.validate(); err != nil {
 		fmt.Fprintf(os.Stderr, "%s\n", err.Error())
 		return
 	}
 
-	r.Packager.distribution = true
-	r.Packager.release = true
-
 	if err := r.beforePackage(); err != nil {
 		fmt.Fprintf(os.Stderr, "%s\n", err.Error())
 		return
@@ -195,13 +207,13 @@ func (r *Releaser) Run(params []string) {
 }
 
 func (r *Releaser) releaseAction(_ *cli.Context) error {
+	r.Packager.distribution = true
+	r.Packager.release = true
+
 	if err := r.validate(); err != nil {
 		return err
 	}
 
-	r.Packager.distribution = true
-	r.Packager.release = true
-
 	if err := r.beforePackage(); err != nil {
 		return err
 	}
@@ -487,7 +499,7 @@ func (r *Releaser) validate() error {
 	return nil
 }
 
-func (r *Releaser) writeEntitlements(tmpl *template.Template, entitlementData interface{}) (cleanup func(), err error) {
+func (r *Releaser) writeEntitlements(tmpl *template.Template, entitlementData any) (cleanup func(), err error) {
 	entitlementPath := filepath.Join(r.dir, "entitlements.plist")
 	entitlements, err := os.Create(entitlementPath)
 	if err != nil {
