@@ -129,12 +129,12 @@ func (c *Container) Refresh() {
 // This method is not intended to be used inside a loop, to remove all the elements.
 // It is much more efficient to call RemoveAll() instead.
 func (c *Container) Remove(rem CanvasObject) {
+	c.lock.Lock()
+	defer c.lock.Unlock()
 	if len(c.Objects) == 0 {
 		return
 	}
 
-	c.lock.Lock()
-	defer c.lock.Unlock()
 	for i, o := range c.Objects {
 		if o != rem {
 			continue
@@ -198,11 +198,12 @@ func (c *Container) layout() {
 // repaint instructs the containing canvas to redraw, even if nothing changed.
 // This method is a duplicate of what is in `canvas/canvas.go` to avoid a dependency loop or public API.
 func repaint(obj *Container) {
-	if CurrentApp() == nil || CurrentApp().Driver() == nil {
+	app := CurrentApp()
+	if app == nil || app.Driver() == nil {
 		return
 	}
 
-	c := CurrentApp().Driver().CanvasForObject(obj)
+	c := app.Driver().CanvasForObject(obj)
 	if c != nil {
 		if paint, ok := c.(interface{ SetDirty() }); ok {
 			paint.SetDirty()
