@@ -16,6 +16,9 @@ type listBase struct {
 
 // GetItem returns the DataItem at the specified index.
 func (b *listBase) GetItem(i int) (DataItem, error) {
+	b.lock.RLock()
+	defer b.lock.RUnlock()
+
 	if i < 0 || i >= len(b.items) {
 		return nil, errOutOfBounds
 	}
@@ -25,6 +28,9 @@ func (b *listBase) GetItem(i int) (DataItem, error) {
 
 // Length returns the number of items in this data list.
 func (b *listBase) Length() int {
+	b.lock.RLock()
+	defer b.lock.RUnlock()
+
 	return len(b.items)
 }
 
