@@ -11,7 +11,7 @@ func TestEntryDialog_Confirm(t *testing.T) {
 	value := ""
 	ed := NewEntryDialog("Test", "message", func(v string) {
 		value = v
-	}, test.NewWindow(nil))
+	}, test.NewTempWindow(t, nil))
 	ed.Show()
 	test.Type(ed.entry, "123")
 	test.Tap(ed.confirm)
@@ -23,7 +23,7 @@ func TestEntryDialog_Dismiss(t *testing.T) {
 	value := "123"
 	ed := NewEntryDialog("Test", "message", func(v string) {
 		value = v
-	}, test.NewWindow(nil))
+	}, test.NewTempWindow(t, nil))
 	ed.Show()
 	test.Type(ed.entry, "XYZ")
 	test.Tap(ed.cancel)
