@@ -7,22 +7,31 @@ import (
 	"path/filepath"
 	"runtime"
 	"strings"
+	"sync"
 
 	"fyne.io/fyne/v2"
 	"fyne.io/fyne/v2/container"
+	"fyne.io/fyne/v2/lang"
 	"fyne.io/fyne/v2/storage"
 	"fyne.io/fyne/v2/storage/repository"
 	"fyne.io/fyne/v2/theme"
 	"fyne.io/fyne/v2/widget"
 )
 
-type viewLayout int
+// ViewLayout can be passed to SetView() to set the view of
+// a FileDialog
+//
+// Since: 2.5
+type ViewLayout int
 
 const (
-	gridView viewLayout = iota
-	listView
+	defaultView ViewLayout = iota
+	ListView
+	GridView
 )
 
+const viewLayoutKey = "fyne:fileDialogViewLayout"
+
 type textWidget interface {
 	fyne.Widget
 	SetText(string)
@@ -34,6 +43,12 @@ type favoriteItem struct {
 	loc     fyne.URI
 }
 
+type fileDialogPanel interface {
+	fyne.Widget
+
+	Unselect(int)
+}
+
 type fileDialog struct {
 	file             *FileDialog
 	fileName         textWidget
@@ -41,24 +56,30 @@ type fileDialog struct {
 	open             *widget.Button
 	breadcrumb       *fyne.Container
 	breadcrumbScroll *container.Scroll
-	files            *fyne.Container
+	files            fileDialogPanel
 	filesScroll      *container.Scroll
 	favorites        []favoriteItem
 	favoritesList    *widget.List
 	showHidden       bool
 
-	view viewLayout
+	view ViewLayout
 
-	win      *widget.PopUp
-	selected *fileDialogItem
-	dir      fyne.ListableURI
+	data     []fyne.URI
+	dataLock sync.RWMutex
+
+	win        *widget.PopUp
+	selected   fyne.URI
+	selectedID int
+	dir        fyne.ListableURI
 	// this will be the initial filename in a FileDialog in save mode
 	initialFileName string
+
+	toggleViewButton *widget.Button
 }
 
 // FileDialog is a dialog containing a file picker for use in opening or saving files.
 type FileDialog struct {
-	callback         interface{}
+	callback         any
 	onClosedCallback func(bool)
 	parent           fyne.Window
 	dialog           *fileDialog
@@ -71,6 +92,8 @@ type FileDialog struct {
 	startingLocation fyne.ListableURI
 	// this will be the initial filename in a FileDialog in save mode
 	initialFileName string
+	// this will be the initial view in a FileDialog
+	initialView ViewLayout
 }
 
 // Declare conformity to Dialog interface
@@ -86,20 +109,149 @@ func (f *fileDialog) makeUI() fyne.CanvasObject {
 				f.open.Enable()
 			}
 		}
-		saveName.SetPlaceHolder("Enter filename")
+		saveName.SetPlaceHolder(lang.L("Enter filename"))
+		saveName.OnSubmitted = func(s string) {
+			f.open.OnTapped()
+		}
 		f.fileName = saveName
 	} else {
 		f.fileName = widget.NewLabel("")
 	}
 
-	label := "Open"
+	label := lang.L("Open")
 	if f.file.save {
-		label = "Save"
+		label = lang.L("Save")
 	}
 	if f.file.confirmText != "" {
 		label = f.file.confirmText
 	}
-	f.open = widget.NewButton(label, func() {
+	f.open = f.makeOpenButton(label)
+
+	if f.file.save {
+		f.fileName.SetText(f.initialFileName)
+	}
+
+	dismissLabel := lang.L("Cancel")
+	if f.file.dismissText != "" {
+		dismissLabel = f.file.dismissText
+	}
+	f.dismiss = f.makeDismissButton(dismissLabel)
+
+	buttons := container.NewGridWithRows(1, f.dismiss, f.open)
+
+	f.filesScroll = container.NewScroll(nil) // filesScroll's content will be set by setView function.
+	verticalExtra := float32(float64(fileIconSize) * 0.25)
+	itemMin := f.newFileItem(storage.NewFileURI("filename.txt"), false, false).MinSize()
+	f.filesScroll.SetMinSize(itemMin.AddWidthHeight(itemMin.Width+theme.Padding()*3, verticalExtra))
+
+	f.breadcrumb = container.NewHBox()
+	f.breadcrumbScroll = container.NewHScroll(container.NewPadded(f.breadcrumb))
+	title := label + " " + lang.L("File")
+	if f.file.isDirectory() {
+		title = label + " " + lang.L("Folder")
+	}
+
+	view := ViewLayout(fyne.CurrentApp().Preferences().Int(viewLayoutKey))
+
+	// handle invalid values
+	if view != GridView && view != ListView {
+		view = defaultView
+	}
+
+	if view == defaultView {
+		// set GridView as default
+		view = GridView
+
+		if f.file.initialView != defaultView {
+			view = f.file.initialView
+		}
+	}
+
+	// icon of button is set in subsequent setView() call
+	f.toggleViewButton = widget.NewButtonWithIcon("", nil, func() {
+		if f.view == GridView {
+			f.setView(ListView)
+		} else {
+			f.setView(GridView)
+		}
+	})
+	f.setView(view)
+
+	f.loadFavorites()
+
+	f.favoritesList = widget.NewList(
+		func() int {
+			return len(f.favorites)
+		},
+		func() fyne.CanvasObject {
+			return container.NewHBox(container.New(&iconPaddingLayout{}, widget.NewIcon(theme.DocumentIcon())), widget.NewLabel("Template Object"))
+		},
+		func(id widget.ListItemID, item fyne.CanvasObject) {
+			item.(*fyne.Container).Objects[0].(*fyne.Container).Objects[0].(*widget.Icon).SetResource(f.favorites[id].locIcon)
+			item.(*fyne.Container).Objects[1].(*widget.Label).SetText(f.favorites[id].locName)
+		},
+	)
+	f.favoritesList.OnSelected = func(id widget.ListItemID) {
+		f.setLocation(f.favorites[id].loc)
+	}
+
+	var optionsButton *widget.Button
+	optionsButton = widget.NewButtonWithIcon("", theme.SettingsIcon(), func() {
+		f.optionsMenu(fyne.CurrentApp().Driver().AbsolutePositionForObject(optionsButton), optionsButton.Size())
+	})
+
+	newFolderButton := widget.NewButtonWithIcon("", theme.FolderNewIcon(), func() {
+		newFolderEntry := widget.NewEntry()
+		ShowForm(lang.L("New Folder"), lang.L("Create Folder"), lang.L("Cancel"), []*widget.FormItem{
+			{
+				Text:   lang.X("file.name", "Name"),
+				Widget: newFolderEntry,
+			},
+		}, func(s bool) {
+			if !s || newFolderEntry.Text == "" {
+				return
+			}
+
+			newFolderPath := filepath.Join(f.dir.Path(), newFolderEntry.Text)
+			createFolderErr := os.MkdirAll(newFolderPath, 0750)
+			if createFolderErr != nil {
+				fyne.LogError(
+					fmt.Sprintf("Failed to create folder with path %s", newFolderPath),
+					createFolderErr,
+				)
+				ShowError(errors.New("folder cannot be created"), f.file.parent)
+			}
+			f.refreshDir(f.dir)
+		}, f.file.parent)
+	})
+
+	optionsbuttons := container.NewHBox(
+		newFolderButton,
+		f.toggleViewButton,
+		optionsButton,
+	)
+
+	header := container.NewBorder(nil, nil, nil, optionsbuttons,
+		optionsbuttons, widget.NewLabelWithStyle(title, fyne.TextAlignLeading, fyne.TextStyle{Bold: true}),
+	)
+
+	footer := container.NewBorder(nil, nil, nil, buttons,
+		buttons, container.NewHScroll(f.fileName),
+	)
+
+	body := container.NewHSplit(
+		f.favoritesList,
+		container.NewBorder(f.breadcrumbScroll, nil, nil, nil,
+			f.breadcrumbScroll, f.filesScroll,
+		),
+	)
+	body.SetOffset(0) // Set the minimum offset so that the favoritesList takes only it's minimal width
+
+	return container.NewBorder(header, footer, nil, nil, body)
+}
+
+func (f *fileDialog) makeOpenButton(label string) *widget.Button {
+	btn := widget.NewButton(label, func() {
 		if f.file.callback == nil {
 			f.win.Hide()
 			if f.file.onClosedCallback != nil {
@@ -150,7 +302,7 @@ func (f *fileDialog) makeUI() fyne.CanvasObject {
 			if f.file.onClosedCallback != nil {
 				f.file.onClosedCallback(true)
 			}
-			callback(storage.Reader(f.selected.location))
+			callback(storage.Reader(f.selected))
 		} else if f.file.isDirectory() {
 			callback := f.file.callback.(func(fyne.ListableURI, error))
 			f.win.Hide()
@@ -160,16 +312,15 @@ func (f *fileDialog) makeUI() fyne.CanvasObject {
 			callback(f.dir, nil)
 		}
 	})
-	f.open.Importance = widget.HighImportance
-	f.open.Disable()
-	if f.file.save {
-		f.fileName.SetText(f.initialFileName)
-	}
-	dismissLabel := "Cancel"
-	if f.file.dismissText != "" {
-		dismissLabel = f.file.dismissText
-	}
-	f.dismiss = widget.NewButton(dismissLabel, func() {
+
+	btn.Importance = widget.HighImportance
+	btn.Disable()
+
+	return btn
+}
+
+func (f *fileDialog) makeDismissButton(label string) *widget.Button {
+	btn := widget.NewButton(label, func() {
 		f.win.Hide()
 		if f.file.onClosedCallback != nil {
 			f.file.onClosedCallback(false)
@@ -184,107 +335,12 @@ func (f *fileDialog) makeUI() fyne.CanvasObject {
 			}
 		}
 	})
-	buttons := container.NewGridWithRows(1, f.dismiss, f.open)
 
-	f.filesScroll = container.NewScroll(nil) // filesScroll's content will be set by setView function.
-	verticalExtra := float32(float64(fileIconSize) * 0.25)
-	f.filesScroll.SetMinSize(fyne.NewSize(fileIconCellWidth*2+theme.Padding(),
-		(fileIconSize+fileTextSize)+theme.Padding()*2+verticalExtra))
-
-	f.breadcrumb = container.NewHBox()
-	f.breadcrumbScroll = container.NewHScroll(container.NewPadded(f.breadcrumb))
-	title := label + " File"
-	if f.file.isDirectory() {
-		title = label + " Folder"
-	}
-
-	f.setView(gridView)
-	f.loadFavorites()
-
-	f.favoritesList = widget.NewList(
-		func() int {
-			return len(f.favorites)
-		},
-		func() fyne.CanvasObject {
-			return container.NewHBox(widget.NewIcon(theme.DocumentIcon()), widget.NewLabel("Template Object"))
-		},
-		func(id widget.ListItemID, item fyne.CanvasObject) {
-			item.(*fyne.Container).Objects[0].(*widget.Icon).SetResource(f.favorites[id].locIcon)
-			item.(*fyne.Container).Objects[1].(*widget.Label).SetText(f.favorites[id].locName)
-		},
-	)
-	f.favoritesList.OnSelected = func(id widget.ListItemID) {
-		f.setLocation(f.favorites[id].loc)
-	}
-
-	var optionsButton *widget.Button
-	optionsButton = widget.NewButtonWithIcon("", theme.SettingsIcon(), func() {
-		f.optionsMenu(fyne.CurrentApp().Driver().AbsolutePositionForObject(optionsButton), optionsButton.Size())
-	})
-
-	var toggleViewButton *widget.Button
-	toggleViewButton = widget.NewButtonWithIcon("", theme.ListIcon(), func() {
-		if f.view == gridView {
-			f.setView(listView)
-			toggleViewButton.SetIcon(theme.GridIcon())
-		} else {
-			f.setView(gridView)
-			toggleViewButton.SetIcon(theme.ListIcon())
-		}
-	})
-
-	newFolderButton := widget.NewButtonWithIcon("", theme.FolderNewIcon(), func() {
-		newFolderEntry := widget.NewEntry()
-		ShowForm("New Folder", "Create Folder", "Cancel", []*widget.FormItem{
-			{
-				Text:   "Name",
-				Widget: newFolderEntry,
-			},
-		}, func(s bool) {
-			if !s || newFolderEntry.Text == "" {
-				return
-			}
-
-			newFolderPath := filepath.Join(f.dir.Path(), newFolderEntry.Text)
-			createFolderErr := os.MkdirAll(newFolderPath, 0750)
-			if createFolderErr != nil {
-				fyne.LogError(
-					fmt.Sprintf("Failed to create folder with path %s", newFolderPath),
-					createFolderErr,
-				)
-				ShowError(errors.New("folder cannot be created"), f.file.parent)
-			}
-			f.refreshDir(f.dir)
-		}, f.file.parent)
-	})
-
-	optionsbuttons := container.NewHBox(
-		newFolderButton,
-		toggleViewButton,
-		optionsButton,
-	)
-
-	header := container.NewBorder(nil, nil, nil, optionsbuttons,
-		optionsbuttons, widget.NewLabelWithStyle(title, fyne.TextAlignLeading, fyne.TextStyle{Bold: true}),
-	)
-
-	footer := container.NewBorder(nil, nil, nil, buttons,
-		buttons, container.NewHScroll(f.fileName),
-	)
-
-	body := container.NewHSplit(
-		f.favoritesList,
-		container.NewBorder(f.breadcrumbScroll, nil, nil, nil,
-			f.breadcrumbScroll, f.filesScroll,
-		),
-	)
-	body.SetOffset(0) // Set the minimum offset so that the favoritesList takes only it's minimal width
-
-	return container.NewBorder(header, footer, nil, nil, body)
+	return btn
 }
 
 func (f *fileDialog) optionsMenu(position fyne.Position, buttonSize fyne.Size) {
-	hiddenFiles := widget.NewCheck("Show Hidden Files", func(changed bool) {
+	hiddenFiles := widget.NewCheck(lang.L("Show Hidden Files"), func(changed bool) {
 		f.showHidden = changed
 		f.refreshDir(f.dir)
 	})
@@ -326,7 +382,9 @@ func (f *fileDialog) loadFavorites() {
 }
 
 func (f *fileDialog) refreshDir(dir fyne.ListableURI) {
-	f.files.Objects = nil
+	f.dataLock.Lock()
+	f.data = nil
+	f.dataLock.Unlock()
 
 	files, err := dir.List()
 	if err != nil {
@@ -334,16 +392,14 @@ func (f *fileDialog) refreshDir(dir fyne.ListableURI) {
 		return
 	}
 
-	var icons []fyne.CanvasObject
+	var icons []fyne.URI
 	parent, err := storage.Parent(dir)
 	if err != nil && err != repository.ErrURIRoot {
 		fyne.LogError("Unable to get parent of "+dir.String(), err)
 		return
 	}
 	if parent != nil && parent.String() != dir.String() {
-		fi := &fileDialogItem{picker: f, name: "(Parent)", location: parent, dir: true}
-		fi.ExtendBaseWidget(fi)
-		icons = append(icons, fi)
+		icons = append(icons, parent)
 	}
 
 	for _, file := range files {
@@ -351,25 +407,31 @@ func (f *fileDialog) refreshDir(dir fyne.ListableURI) {
 			continue
 		}
 
-		listable, err := storage.CanList(file)
+		listable, err := storage.ListerForURI(file)
 		if f.file.isDirectory() && err != nil {
 			continue
-		} else if err == nil && listable { // URI points to a directory
-			icons = append(icons, f.newFileItem(file, true)) // Pass the listable URI to avoid doing the same check in FileIcon
+		} else if err == nil { // URI points to a directory
+			icons = append(icons, listable)
 		} else if f.file.filter == nil || f.file.filter.Matches(file) {
-			icons = append(icons, f.newFileItem(file, false))
+			icons = append(icons, file)
 		}
 	}
 
-	f.files.Objects = icons
+	f.dataLock.Lock()
+	f.data = icons
+	f.dataLock.Unlock()
+
 	f.files.Refresh()
 	f.filesScroll.Offset = fyne.NewPos(0, 0)
 	f.filesScroll.Refresh()
 }
 
 func (f *fileDialog) setLocation(dir fyne.URI) error {
+	if f.selectedID > -1 {
+		f.files.Unselect(f.selectedID)
+	}
 	if dir == nil {
-		return fmt.Errorf("failed to open nil directory")
+		return errors.New("failed to open nil directory")
 	}
 	list, err := storage.ListerForURI(dir)
 	if err != nil {
@@ -391,7 +453,7 @@ func (f *fileDialog) setLocation(dir fyne.URI) error {
 		f.favoritesList.UnselectAll()
 	}
 
-	f.setSelected(nil)
+	f.setSelected(nil, -1)
 	f.dir = list
 
 	f.breadcrumb.Objects = nil
@@ -445,22 +507,17 @@ func (f *fileDialog) setLocation(dir fyne.URI) error {
 	return nil
 }
 
-func (f *fileDialog) setSelected(file *fileDialogItem) {
-	if f.selected != nil {
-		f.selected.isCurrent = false
-		f.selected.Refresh()
-	}
-	if file != nil && file.isDirectory() {
-		listable, err := storage.CanList(file.location)
-		if err != nil || !listable {
-			fyne.LogError("Failed to create lister for URI"+file.location.String(), err)
+func (f *fileDialog) setSelected(file fyne.URI, id int) {
+	if file != nil {
+		if listable, err := storage.CanList(file); err == nil && listable {
+			f.setLocation(file)
+			return
 		}
-		f.setLocation(file.location)
-		return
 	}
 	f.selected = file
+	f.selectedID = id
 
-	if file == nil || file.location.String()[len(file.location.Scheme())+3:] == "" {
+	if file == nil || file.String()[len(file.Scheme())+3:] == "" {
 		// keep user input while navigating
 		// in a FileSave dialog
 		if !f.file.save {
@@ -468,22 +525,57 @@ func (f *fileDialog) setSelected(file *fileDialogItem) {
 			f.open.Disable()
 		}
 	} else {
-		file.isCurrent = true
-		f.fileName.SetText(file.location.Name())
+		f.fileName.SetText(file.Name())
 		f.open.Enable()
 	}
 }
 
-func (f *fileDialog) setView(view viewLayout) {
+func (f *fileDialog) setView(view ViewLayout) {
 	f.view = view
-	if f.view == gridView {
-		padding := fyne.NewSize(fileIconCellWidth-fileIconSize, theme.Padding())
-		f.files = container.NewGridWrap(
-			fyne.NewSize(fileIconSize, fileIconSize+fileTextSize).Add(padding),
-		)
-	} else {
-		f.files = container.NewVBox()
+	fyne.CurrentApp().Preferences().SetInt(viewLayoutKey, int(view))
+	var selectF func(id int)
+	choose := func(id int) {
+		selectF(id)
+		if file, ok := f.getDataItem(id); ok {
+			f.selectedID = id
+			f.setSelected(file, id)
+		}
 	}
+	count := func() int {
+		f.dataLock.RLock()
+		defer f.dataLock.RUnlock()
+
+		return len(f.data)
+	}
+	template := func() fyne.CanvasObject {
+		return f.newFileItem(storage.NewFileURI("./tempfile"), true, false)
+	}
+	update := func(id widget.GridWrapItemID, o fyne.CanvasObject) {
+		if dir, ok := f.getDataItem(id); ok {
+			parent := id == 0 && len(dir.Path()) < len(f.dir.Path())
+			_, isDir := dir.(fyne.ListableURI)
+			o.(*fileDialogItem).setLocation(dir, isDir || parent, parent)
+			o.(*fileDialogItem).choose = choose
+			o.(*fileDialogItem).id = id
+			o.(*fileDialogItem).open = f.open.OnTapped
+		}
+	}
+	// Actually, during the real interaction, the OnSelected won't be called.
+	// It will be called only when we directly calls container.select(i)
+	if f.view == GridView {
+		grid := widget.NewGridWrap(count, template, update)
+		grid.OnSelected = choose
+		f.files = grid
+		f.toggleViewButton.SetIcon(theme.ListIcon())
+		selectF = grid.Select
+	} else {
+		list := widget.NewList(count, template, update)
+		list.OnSelected = choose
+		f.files = list
+		f.toggleViewButton.SetIcon(theme.GridIcon())
+		selectF = list.Select
+	}
+
 	if f.dir != nil {
 		f.refreshDir(f.dir)
 	}
@@ -491,6 +583,17 @@ func (f *fileDialog) setView(view viewLayout) {
 	f.filesScroll.Refresh()
 }
 
+func (f *fileDialog) getDataItem(id int) (fyne.URI, bool) {
+	f.dataLock.RLock()
+	defer f.dataLock.RUnlock()
+
+	if id >= len(f.data) {
+		return nil, false
+	}
+
+	return f.data[id], true
+}
+
 // effectiveStartingDir calculates the directory at which the file dialog should
 // open, based on the values of startingDirectory, CWD, home, and any error
 // conditions which occur.
@@ -554,16 +657,20 @@ func (f *FileDialog) effectiveStartingDir() fyne.ListableURI {
 }
 
 func showFile(file *FileDialog) *fileDialog {
-	d := &fileDialog{file: file, initialFileName: file.initialFileName}
+	d := &fileDialog{file: file, initialFileName: file.initialFileName, view: GridView}
 	ui := d.makeUI()
-	size := ui.MinSize().Add(fyne.NewSize(fileIconCellWidth*2+theme.Padding()*6+theme.Padding(),
-		(fileIconSize+fileTextSize)+theme.Padding()*6))
+	pad := theme.Padding()
+	itemMin := d.newFileItem(storage.NewFileURI("filename.txt"), false, false).MinSize()
+	size := ui.MinSize().Add(itemMin.AddWidthHeight(itemMin.Width+pad*4, pad*2))
 
 	d.win = widget.NewModalPopUp(ui, file.parent.Canvas())
 	d.win.Resize(size)
 
 	d.setLocation(file.effectiveStartingDir())
 	d.win.Show()
+	if file.save {
+		d.win.Canvas.Focus(d.fileName.(*widget.Entry))
+	}
 	return d
 }
 
@@ -643,7 +750,7 @@ func (f *FileDialog) SetDismissText(label string) {
 	f.dialog.win.Refresh()
 }
 
-// SetLocation tells this FileDirectory which location to display.
+// SetLocation tells this FileDialog which location to display.
 // This is normally called before the dialog is shown.
 //
 // Since: 1.4
@@ -657,13 +764,13 @@ func (f *FileDialog) SetLocation(u fyne.ListableURI) {
 // SetOnClosed sets a callback function that is called when
 // the dialog is closed.
 func (f *FileDialog) SetOnClosed(closed func()) {
-	if f.dialog == nil {
-		return
-	}
 	// If there is already a callback set, remember it and call both.
 	originalCallback := f.onClosedCallback
 
 	f.onClosedCallback = func(response bool) {
+		if f.dialog == nil {
+			return
+		}
 		closed()
 		if originalCallback != nil {
 			originalCallback(response)
@@ -695,6 +802,17 @@ func (f *FileDialog) SetFileName(fileName string) {
 	}
 }
 
+// SetView changes the default display view of the FileDialog
+// This is normally called before the dialog is shown.
+//
+// Since: 2.5
+func (f *FileDialog) SetView(v ViewLayout) {
+	f.initialView = v
+	if f.dialog != nil {
+		f.dialog.setView(v)
+	}
+}
+
 // NewFileOpen creates a file dialog allowing the user to choose a file to open.
 // The callback function will run when the dialog closes. The URI will be nil
 // when the user cancels or when nothing is selected.
@@ -749,6 +867,7 @@ func getFavoriteIcons() map[string]fyne.Resource {
 	if runtime.GOOS == "darwin" {
 		return map[string]fyne.Resource{
 			"Documents": theme.DocumentIcon(),
+			"Desktop":   theme.DesktopIcon(),
 			"Downloads": theme.DownloadIcon(),
 			"Music":     theme.MediaMusicIcon(),
 			"Pictures":  theme.MediaPhotoIcon(),
@@ -758,6 +877,7 @@ func getFavoriteIcons() map[string]fyne.Resource {
 
 	return map[string]fyne.Resource{
 		"Documents": theme.DocumentIcon(),
+		"Desktop":   theme.DesktopIcon(),
 		"Downloads": theme.DownloadIcon(),
 		"Music":     theme.MediaMusicIcon(),
 		"Pictures":  theme.MediaPhotoIcon(),
@@ -767,6 +887,7 @@ func getFavoriteIcons() map[string]fyne.Resource {
 
 func getFavoriteOrder() []string {
 	order := []string{
+		"Desktop",
 		"Documents",
 		"Downloads",
 		"Music",
@@ -789,3 +910,18 @@ func storageURI(a fyne.App) fyne.URI {
 	dir, _ := storage.Child(a.Storage().RootURI(), "Documents")
 	return dir
 }
+
+// iconPaddingLayout adds padding to the left of a widget.Icon().
+// NOTE: It assumes that the slice only contains one item.
+type iconPaddingLayout struct {
+}
+
+func (i *iconPaddingLayout) Layout(objects []fyne.CanvasObject, size fyne.Size) {
+	padding := theme.Padding() * 2
+	objects[0].Move(fyne.NewPos(padding, 0))
+	objects[0].Resize(size.SubtractWidthHeight(padding, 0))
+}
+
+func (i *iconPaddingLayout) MinSize(objects []fyne.CanvasObject) fyne.Size {
+	return objects[0].MinSize().AddWidthHeight(theme.Padding()*2, 0)
+}
