@@ -11,7 +11,6 @@ import (
 
 	"fyne.io/fyne/v2"
 	"fyne.io/fyne/v2/container"
-	"fyne.io/fyne/v2/layout"
 	"fyne.io/fyne/v2/storage"
 	"fyne.io/fyne/v2/test"
 	"fyne.io/fyne/v2/theme"
@@ -62,8 +61,6 @@ func TestEffectiveStartingDir(t *testing.T) {
 	}
 
 	parent, err := storage.ListerForURI(parentURI)
-	t.Log(parentURI)
-	t.Log(parent)
 	if err != nil {
 		t.Skipf("Could not get lister for parent of working directory: %s", err)
 	}
@@ -111,7 +108,7 @@ func TestEffectiveStartingDir(t *testing.T) {
 }
 
 func TestFileDialogResize(t *testing.T) {
-	win := test.NewWindow(widget.NewLabel("Content"))
+	win := test.NewTempWindow(t, widget.NewLabel("Content"))
 	win.Resize(fyne.NewSize(600, 400))
 	file := NewFileOpen(func(file fyne.URIReadCloser, err error) {}, win)
 	file.SetFilter(storage.NewExtensionFileFilter([]string{".png"}))
@@ -120,8 +117,9 @@ func TestFileDialogResize(t *testing.T) {
 	d := &fileDialog{file: file}
 	open := widget.NewButton("open", func() {})
 	ui := container.NewBorder(nil, nil, nil, open)
-	originalSize := ui.MinSize().Add(fyne.NewSize(fileIconCellWidth*2+theme.Padding()*4,
-		(fileIconSize+fileTextSize)+theme.Padding()*4))
+	pad := theme.Padding()
+	itemMin := d.newFileItem(storage.NewFileURI("filename.txt"), false, false).MinSize()
+	originalSize := ui.MinSize().Add(itemMin.AddWidthHeight(itemMin.Width+pad*6, pad*3))
 	d.win = widget.NewModalPopUp(ui, file.parent.Canvas())
 	d.win.Resize(originalSize)
 	file.dialog = d
@@ -163,7 +161,7 @@ func TestFileDialogResize(t *testing.T) {
 func TestShowFileOpen(t *testing.T) {
 	var chosen fyne.URIReadCloser
 	var openErr error
-	win := test.NewWindow(widget.NewLabel("Content"))
+	win := test.NewTempWindow(t, widget.NewLabel("Content"))
 	d := NewFileOpen(func(file fyne.URIReadCloser, err error) {
 		chosen = file
 		openErr = err
@@ -188,13 +186,13 @@ func TestShowFileOpen(t *testing.T) {
 	//optionsbuttons
 	createNewFolderButton := ui.Objects[1].(*fyne.Container).Objects[0].(*fyne.Container).Objects[0].(*widget.Button)
 	assert.Equal(t, "", createNewFolderButton.Text)
-	assert.Equal(t, theme.FolderNewIcon(), createNewFolderButton.Icon)
+	assert.Equal(t, theme.FolderNewIcon().Name(), createNewFolderButton.Icon.Name())
 	toggleViewButton := ui.Objects[1].(*fyne.Container).Objects[0].(*fyne.Container).Objects[1].(*widget.Button)
 	assert.Equal(t, "", toggleViewButton.Text)
-	assert.Equal(t, theme.ListIcon(), toggleViewButton.Icon)
+	assert.Equal(t, theme.ListIcon().Name(), toggleViewButton.Icon.Name())
 	optionsButton := ui.Objects[1].(*fyne.Container).Objects[0].(*fyne.Container).Objects[2].(*widget.Button)
 	assert.Equal(t, "", optionsButton.Text)
-	assert.Equal(t, theme.SettingsIcon(), optionsButton.Icon)
+	assert.Equal(t, theme.SettingsIcon().Name(), optionsButton.Icon.Name())
 	//footer
 	nameLabel := ui.Objects[2].(*fyne.Container).Objects[1].(*container.Scroll).Content.(*widget.Label)
 	buttons := ui.Objects[2].(*fyne.Container).Objects[0].(*fyne.Container)
@@ -215,21 +213,26 @@ func TestShowFileOpen(t *testing.T) {
 		}
 	}
 
-	files := ui.Objects[0].(*container.Split).Trailing.(*fyne.Container).Objects[1].(*container.Scroll).Content.(*fyne.Container).Objects[0].(*fyne.Container)
-	assert.Greater(t, len(files.Objects), 0)
+	files := ui.Objects[0].(*container.Split).Trailing.(*fyne.Container).Objects[1].(*container.Scroll).Content.(*fyne.Container).Objects[0].(*widget.GridWrap)
+	objects := test.TempWidgetRenderer(t, files).Objects()[0].(*container.Scroll).Content.(*fyne.Container).Objects
+	assert.Greater(t, len(objects), 0)
 
-	fileName := files.Objects[0].(*fileDialogItem).name
+	fileName := test.TempWidgetRenderer(t, objects[0].(fyne.Widget)).Objects()[1].(*fileDialogItem).name
 	assert.Equal(t, "(Parent)", fileName)
 	assert.True(t, open.Disabled())
 
 	var target *fileDialogItem
-	for _, icon := range files.Objects {
-		if icon.(*fileDialogItem).dir == false {
-			target = icon.(*fileDialogItem)
+	id := 0
+	for i, icon := range objects {
+		item := test.TempWidgetRenderer(t, icon.(fyne.Widget)).Objects()[1].(*fileDialogItem)
+		if item.dir == false {
+			target = item
+			id = i
+			break
 		}
 	}
 	assert.NotNil(t, target, "Failed to find file in testdata")
-	test.Tap(target)
+	d.dialog.files.(*widget.GridWrap).Select(id)
 	assert.Equal(t, target.location.Name(), nameLabel.Text)
 	assert.False(t, open.Disabled())
 
@@ -257,7 +260,7 @@ func TestHiddenFiles(t *testing.T) {
 		t.Error("Failed to hide .hidden", err)
 	}
 
-	win := test.NewWindow(widget.NewLabel("Content"))
+	win := test.NewTempWindow(t, widget.NewLabel("Content"))
 	d := NewFileOpen(func(file fyne.URIReadCloser, err error) {
 	}, win)
 	d.SetLocation(dir)
@@ -271,21 +274,23 @@ func TestHiddenFiles(t *testing.T) {
 
 	createNewFolderButton := ui.Objects[1].(*fyne.Container).Objects[0].(*fyne.Container).Objects[0].(*widget.Button)
 	assert.Equal(t, "", createNewFolderButton.Text)
-	assert.Equal(t, theme.FolderNewIcon(), createNewFolderButton.Icon)
+	assert.Equal(t, theme.FolderNewIcon().Name(), createNewFolderButton.Icon.Name())
 	toggleViewButton := ui.Objects[1].(*fyne.Container).Objects[0].(*fyne.Container).Objects[1].(*widget.Button)
 	assert.Equal(t, "", toggleViewButton.Text)
-	assert.Equal(t, theme.ListIcon(), toggleViewButton.Icon)
+	assert.Equal(t, theme.ListIcon().Name(), toggleViewButton.Icon.Name())
 	optionsButton := ui.Objects[1].(*fyne.Container).Objects[0].(*fyne.Container).Objects[2].(*widget.Button)
 	assert.Equal(t, "", optionsButton.Text)
-	assert.Equal(t, theme.SettingsIcon(), optionsButton.Icon)
+	assert.Equal(t, theme.SettingsIcon().Name(), optionsButton.Icon.Name())
 
-	files := ui.Objects[0].(*container.Split).Trailing.(*fyne.Container).Objects[1].(*container.Scroll).Content.(*fyne.Container).Objects[0].(*fyne.Container)
-	assert.Greater(t, len(files.Objects), 0)
+	files := ui.Objects[0].(*container.Split).Trailing.(*fyne.Container).Objects[1].(*container.Scroll).Content.(*fyne.Container).Objects[0].(*widget.GridWrap)
+	objects := test.TempWidgetRenderer(t, files).Objects()[0].(*container.Scroll).Content.(*fyne.Container).Objects
+	assert.Greater(t, len(objects), 0)
 
 	var target *fileDialogItem
-	for _, icon := range files.Objects {
-		if icon.(*fileDialogItem).name == ".hidden" {
-			target = icon.(*fileDialogItem)
+	for _, icon := range objects {
+		item := test.TempWidgetRenderer(t, icon.(fyne.Widget)).Objects()[1].(*fileDialogItem)
+		if item.name == ".hidden" {
+			target = item
 		}
 	}
 	assert.Nil(t, target, "Failed, .hidden found in testdata")
@@ -293,9 +298,10 @@ func TestHiddenFiles(t *testing.T) {
 	d.dialog.showHidden = true
 	d.dialog.refreshDir(d.dialog.dir)
 
-	for _, icon := range files.Objects {
-		if icon.(*fileDialogItem).name == ".hidden" {
-			target = icon.(*fileDialogItem)
+	for _, icon := range objects {
+		item := test.TempWidgetRenderer(t, icon.(fyne.Widget)).Objects()[1].(*fileDialogItem)
+		if item.name == ".hidden" {
+			target = item
 		}
 	}
 	assert.NotNil(t, target, "Failed,.hidden not found in testdata")
@@ -304,11 +310,12 @@ func TestHiddenFiles(t *testing.T) {
 func TestShowFileSave(t *testing.T) {
 	var chosen fyne.URIWriteCloser
 	var saveErr error
-	win := test.NewWindow(widget.NewLabel("Content"))
-	ShowFileSave(func(file fyne.URIWriteCloser, err error) {
+	win := test.NewTempWindow(t, widget.NewLabel("Content"))
+	saver := NewFileSave(func(file fyne.URIWriteCloser, err error) {
 		chosen = file
 		saveErr = err
 	}, win)
+	saver.Show()
 
 	popup := win.Canvas().Overlays().Top().(*widget.PopUp)
 	defer win.Canvas().Overlays().Remove(popup)
@@ -322,17 +329,22 @@ func TestShowFileSave(t *testing.T) {
 	buttons := ui.Objects[2].(*fyne.Container).Objects[0].(*fyne.Container)
 	save := buttons.Objects[1].(*widget.Button)
 
-	files := ui.Objects[0].(*container.Split).Trailing.(*fyne.Container).Objects[1].(*container.Scroll).Content.(*fyne.Container).Objects[0].(*fyne.Container)
-	assert.Greater(t, len(files.Objects), 0)
+	files := ui.Objects[0].(*container.Split).Trailing.(*fyne.Container).Objects[1].(*container.Scroll).Content.(*fyne.Container).Objects[0].(*widget.GridWrap)
+	objects := test.TempWidgetRenderer(t, files).Objects()[0].(*container.Scroll).Content.(*fyne.Container).Objects
+	assert.Greater(t, len(objects), 0)
 
-	fileName := files.Objects[0].(*fileDialogItem).name
-	assert.Equal(t, "(Parent)", fileName)
+	item := test.TempWidgetRenderer(t, objects[0].(fyne.Widget)).Objects()[1].(*fileDialogItem)
+	assert.Equal(t, "(Parent)", item.name)
 	assert.True(t, save.Disabled())
 
 	var target *fileDialogItem
-	for _, icon := range files.Objects {
-		if icon.(*fileDialogItem).dir == false {
-			target = icon.(*fileDialogItem)
+	id := -1
+	for i, icon := range objects {
+		item := test.TempWidgetRenderer(t, icon.(fyne.Widget)).Objects()[1].(*fileDialogItem)
+		if item.dir == false {
+			target = item
+			id = i
+			break
 		}
 	}
 
@@ -341,9 +353,10 @@ func TestShowFileSave(t *testing.T) {
 		return
 	}
 
-	// This will only execute if we have a file in the home path.
-	// Until we have a way to set the directory of an open file dialog.
-	test.Tap(target)
+	abs, _ := filepath.Abs("./testdata/")
+	dir, _ := storage.ListerForURI(storage.NewFileURI(abs))
+	saver.SetLocation(dir)
+	saver.dialog.files.(*widget.GridWrap).Select(id)
 	assert.Equal(t, target.location.Name(), nameEntry.Text)
 	assert.False(t, save.Disabled())
 
@@ -373,7 +386,7 @@ func TestShowFileSave(t *testing.T) {
 }
 
 func TestFileFilters(t *testing.T) {
-	win := test.NewWindow(widget.NewLabel("Content"))
+	win := test.NewTempWindow(t, widget.NewLabel("Content"))
 	f := NewFileOpen(func(file fyne.URIReadCloser, err error) {
 	}, win)
 
@@ -394,43 +407,49 @@ func TestFileFilters(t *testing.T) {
 	f.dialog.setLocation(testDataLister)
 
 	count := 0
-	for _, icon := range f.dialog.files.Objects {
-		if icon.(*fileDialogItem).dir == false {
-			uri := icon.(*fileDialogItem).location
+	for _, uri := range f.dialog.data {
+		ok, _ := storage.CanList(uri)
+		if !ok {
 			assert.Equal(t, uri.Extension(), ".png")
 			count++
 		}
 	}
-	assert.Equal(t, 5, count)
+
+	// NOTE: This count needs to be updated when more test images are added.
+	assert.Equal(t, 8, count)
 
 	f.SetFilter(storage.NewMimeTypeFileFilter([]string{"image/jpeg"}))
 
 	count = 0
-	for _, icon := range f.dialog.files.Objects {
-		if icon.(*fileDialogItem).dir == false {
-			uri := icon.(*fileDialogItem).location
+	for _, uri := range f.dialog.data {
+		ok, _ := storage.CanList(uri)
+		if !ok {
 			assert.Equal(t, uri.MimeType(), "image/jpeg")
 			count++
 		}
 	}
+
+	// NOTE: This count needs to be updated when more test images are added.
 	assert.Equal(t, 1, count)
 
 	f.SetFilter(storage.NewMimeTypeFileFilter([]string{"image/*"}))
 
 	count = 0
-	for _, icon := range f.dialog.files.Objects {
-		if icon.(*fileDialogItem).dir == false {
-			uri := icon.(*fileDialogItem).location
+	for _, uri := range f.dialog.data {
+		ok, _ := storage.CanList(uri)
+		if !ok {
 			mimeType := strings.Split(uri.MimeType(), "/")[0]
 			assert.Equal(t, mimeType, "image")
 			count++
 		}
 	}
-	assert.Equal(t, 6, count)
+
+	// NOTE: This count needs to be updated when more test images are added.
+	assert.Equal(t, 9, count)
 }
 
 func TestView(t *testing.T) {
-	win := test.NewWindow(widget.NewLabel("Content"))
+	win := test.NewTempWindow(t, widget.NewLabel("Content"))
 
 	dlg := NewFileOpen(func(reader fyne.URIReadCloser, err error) {
 		assert.Nil(t, err)
@@ -447,37 +466,38 @@ func TestView(t *testing.T) {
 
 	ui := popup.Content.(*fyne.Container)
 	toggleViewButton := ui.Objects[1].(*fyne.Container).Objects[0].(*fyne.Container).Objects[1].(*widget.Button)
-	files := ui.Objects[0].(*container.Split).Trailing.(*fyne.Container).Objects[1].(*container.Scroll).Content.(*fyne.Container).Objects[0].(*fyne.Container)
-
-	listLayout := layout.NewVBoxLayout()
+	panel := ui.Objects[0].(*container.Split).Trailing.(*fyne.Container).Objects[1].(*container.Scroll).Content.(*fyne.Container).Objects[0]
 
 	// view should be a grid
-	assert.NotEqual(t, listLayout, files.Layout)
+	_, isGrid := panel.(*widget.GridWrap)
+	assert.True(t, isGrid)
 	// toggleViewButton should reflect to what it will do (change to a list view).
 	assert.Equal(t, "", toggleViewButton.Text)
-	assert.Equal(t, theme.ListIcon(), toggleViewButton.Icon)
+	assert.Equal(t, theme.ListIcon().Name(), toggleViewButton.Icon.Name())
 
 	// toggle view
 	test.Tap(toggleViewButton)
 	// reload files container
-	files = ui.Objects[0].(*container.Split).Trailing.(*fyne.Container).Objects[1].(*container.Scroll).Content.(*fyne.Container).Objects[0].(*fyne.Container)
+	panel = ui.Objects[0].(*container.Split).Trailing.(*fyne.Container).Objects[1].(*container.Scroll).Content.(*fyne.Container).Objects[0]
 
 	// view should be a list
-	assert.Equal(t, listLayout, files.Layout)
+	_, isList := panel.(*widget.List)
+	assert.True(t, isList)
 	// toggleViewButton should reflect to what it will do (change to a grid view).
 	assert.Equal(t, "", toggleViewButton.Text)
-	assert.Equal(t, theme.GridIcon(), toggleViewButton.Icon)
+	assert.Equal(t, theme.GridIcon().Name(), toggleViewButton.Icon.Name())
 
 	// toggle view
 	test.Tap(toggleViewButton)
 	// reload files container
-	files = ui.Objects[0].(*container.Split).Trailing.(*fyne.Container).Objects[1].(*container.Scroll).Content.(*fyne.Container).Objects[0].(*fyne.Container)
+	panel = ui.Objects[0].(*container.Split).Trailing.(*fyne.Container).Objects[1].(*container.Scroll).Content.(*fyne.Container).Objects[0]
 
 	// view should be a grid again
-	assert.NotEqual(t, listLayout, files.Layout)
+	_, isGrid = panel.(*widget.GridWrap)
+	assert.True(t, isGrid)
 	// toggleViewButton should reflect to what it will do (change to a list view).
 	assert.Equal(t, "", toggleViewButton.Text)
-	assert.Equal(t, theme.ListIcon(), toggleViewButton.Icon)
+	assert.Equal(t, theme.ListIcon().Name(), toggleViewButton.Icon.Name())
 
 	confirm := ui.Objects[2].(*fyne.Container).Objects[0].(*fyne.Container).Objects[1].(*widget.Button)
 	assert.Equal(t, "Yes", confirm.Text)
@@ -485,8 +505,133 @@ func TestView(t *testing.T) {
 	assert.Equal(t, "Dismiss", dismiss.Text)
 }
 
+func TestSetView(t *testing.T) {
+	win := test.NewTempWindow(t, widget.NewLabel("Content"))
+
+	fyne.CurrentApp().Preferences().SetInt(viewLayoutKey, int(defaultView))
+
+	dlg := NewFileOpen(func(reader fyne.URIReadCloser, err error) {
+		assert.Nil(t, err)
+		assert.Nil(t, reader)
+	}, win)
+
+	dlg.SetConfirmText("Yes")
+	dlg.SetDismissText("Dismiss")
+
+	// set view to list
+	dlg.SetView(ListView)
+
+	dlg.Show()
+
+	popup := win.Canvas().Overlays().Top().(*widget.PopUp)
+	defer win.Canvas().Overlays().Remove(popup)
+	assert.NotNil(t, popup)
+
+	ui := popup.Content.(*fyne.Container)
+	toggleViewButton := ui.Objects[1].(*fyne.Container).Objects[0].(*fyne.Container).Objects[1].(*widget.Button)
+	panel := ui.Objects[0].(*container.Split).Trailing.(*fyne.Container).Objects[1].(*container.Scroll).Content.(*fyne.Container).Objects[0]
+
+	// view should be a list
+	_, isList := panel.(*widget.List)
+	assert.True(t, isList)
+	// toggleViewButton should reflect to what it will do (change to a grid view).
+	assert.Equal(t, "", toggleViewButton.Text)
+	assert.Equal(t, theme.GridIcon(), toggleViewButton.Icon)
+
+	confirm := ui.Objects[2].(*fyne.Container).Objects[0].(*fyne.Container).Objects[1].(*widget.Button)
+	assert.Equal(t, "Yes", confirm.Text)
+	dismiss := ui.Objects[2].(*fyne.Container).Objects[0].(*fyne.Container).Objects[0].(*widget.Button)
+	assert.Equal(t, "Dismiss", dismiss.Text)
+
+	// set view to grid on already opened dialog - should be updated automatically
+	dlg.SetView(GridView)
+
+	// view should be a grid again
+	panel = ui.Objects[0].(*container.Split).Trailing.(*fyne.Container).Objects[1].(*container.Scroll).Content.(*fyne.Container).Objects[0]
+	_, isGrid := panel.(*widget.GridWrap)
+	assert.True(t, isGrid)
+	// toggleViewButton should reflect to what it will do (change to a list view).
+	assert.Equal(t, "", toggleViewButton.Text)
+	assert.Equal(t, theme.ListIcon(), toggleViewButton.Icon)
+}
+
+func TestSetViewPreferences(t *testing.T) {
+	win := test.NewTempWindow(t, widget.NewLabel("Content"))
+
+	prefs := fyne.CurrentApp().Preferences()
+
+	// set user-saved viewLayout to GridView
+	prefs.SetInt(viewLayoutKey, int(GridView))
+
+	dlg := NewFileOpen(func(reader fyne.URIReadCloser, err error) {
+		assert.Nil(t, err)
+		assert.Nil(t, reader)
+	}, win)
+
+	// set default view to be ListView
+	dlg.SetView(ListView)
+
+	dlg.Show()
+
+	popup := win.Canvas().Overlays().Top().(*widget.PopUp)
+	defer win.Canvas().Overlays().Remove(popup)
+	assert.NotNil(t, popup)
+
+	ui := popup.Content.(*fyne.Container)
+	toggleViewButton := ui.Objects[1].(*fyne.Container).Objects[0].(*fyne.Container).Objects[1].(*widget.Button)
+	panel := ui.Objects[0].(*container.Split).Trailing.(*fyne.Container).Objects[1].(*container.Scroll).Content.(*fyne.Container).Objects[0]
+
+	// check that preference setting overrules configured default view
+	_, isGrid := panel.(*widget.GridWrap)
+	assert.True(t, isGrid)
+	// toggleViewButton should reflect to what it will do (change to a list view).
+	assert.Equal(t, "", toggleViewButton.Text)
+	assert.Equal(t, theme.ListIcon(), toggleViewButton.Icon)
+}
+
+func TestViewPreferences(t *testing.T) {
+	win := test.NewTempWindow(t, widget.NewLabel("Content"))
+
+	prefs := fyne.CurrentApp().Preferences()
+
+	// set viewLayout to an invalid value to verify that this situation is handled properly
+	prefs.SetInt(viewLayoutKey, -1)
+
+	dlg := NewFileOpen(func(reader fyne.URIReadCloser, err error) {
+		assert.Nil(t, err)
+		assert.Nil(t, reader)
+	}, win)
+
+	dlg.Show()
+
+	popup := win.Canvas().Overlays().Top().(*widget.PopUp)
+	defer win.Canvas().Overlays().Remove(popup)
+	assert.NotNil(t, popup)
+
+	ui := popup.Content.(*fyne.Container)
+	toggleViewButton := ui.Objects[1].(*fyne.Container).Objects[0].(*fyne.Container).Objects[1].(*widget.Button)
+
+	// default viewLayout preference should be 'grid'
+	view := ViewLayout(prefs.Int(viewLayoutKey))
+	assert.Equal(t, GridView, view)
+
+	// toggle view
+	test.Tap(toggleViewButton)
+
+	// viewLayout preference should be 'list'
+	view = ViewLayout(prefs.Int(viewLayoutKey))
+	assert.Equal(t, ListView, view)
+
+	// toggle view
+	test.Tap(toggleViewButton)
+
+	// viewLayout preference should be 'grid' again
+	view = ViewLayout(prefs.Int(viewLayoutKey))
+	assert.Equal(t, GridView, view)
+}
+
 func TestFileFavorites(t *testing.T) {
-	win := test.NewWindow(widget.NewLabel("Content"))
+	win := test.NewTempWindow(t, widget.NewLabel("Content"))
 
 	dlg := NewFileOpen(func(reader fyne.URIReadCloser, err error) {
 		assert.Nil(t, err)
@@ -531,7 +676,7 @@ func TestFileFavorites(t *testing.T) {
 }
 
 func TestSetFileNameBeforeShow(t *testing.T) {
-	win := test.NewWindow(widget.NewLabel("Content"))
+	win := test.NewTempWindow(t, widget.NewLabel("Content"))
 	dSave := NewFileSave(func(fyne.URIWriteCloser, error) {}, win)
 	dSave.SetFileName("testfile.zip")
 	dSave.Show()
@@ -549,7 +694,7 @@ func TestSetFileNameBeforeShow(t *testing.T) {
 
 func TestSetFileNameAfterShow(t *testing.T) {
 
-	win := test.NewWindow(widget.NewLabel("Content"))
+	win := test.NewTempWindow(t, widget.NewLabel("Content"))
 	dSave := NewFileSave(func(fyne.URIWriteCloser, error) {}, win)
 	dSave.Show()
 	dSave.SetFileName("testfile.zip")
@@ -566,7 +711,7 @@ func TestSetFileNameAfterShow(t *testing.T) {
 }
 
 func TestCreateNewFolderInDir(t *testing.T) {
-	win := test.NewWindow(widget.NewLabel("Content"))
+	win := test.NewTempWindow(t, widget.NewLabel("Content"))
 
 	folderDialog := NewFolderOpen(func(lu fyne.ListableURI, err error) {
 		assert.Nil(t, err)
@@ -579,11 +724,11 @@ func TestCreateNewFolderInDir(t *testing.T) {
 	defer win.Canvas().Overlays().Remove(folderDialogPopup)
 	assert.NotNil(t, folderDialogPopup)
 
-	folderDialogUi := folderDialogPopup.Content.(*fyne.Container)
+	folderDialogUI := folderDialogPopup.Content.(*fyne.Container)
 
-	createNewFolderButton := folderDialogUi.Objects[1].(*fyne.Container).Objects[0].(*fyne.Container).Objects[0].(*widget.Button)
+	createNewFolderButton := folderDialogUI.Objects[1].(*fyne.Container).Objects[0].(*fyne.Container).Objects[0].(*widget.Button)
 	assert.Equal(t, "", createNewFolderButton.Text)
-	assert.Equal(t, theme.FolderNewIcon(), createNewFolderButton.Icon)
+	assert.Equal(t, theme.FolderNewIcon().Name(), createNewFolderButton.Icon.Name())
 
 	// open folder name input dialog
 	test.Tap(createNewFolderButton)
@@ -592,21 +737,31 @@ func TestCreateNewFolderInDir(t *testing.T) {
 	defer win.Canvas().Overlays().Remove(inputPopup)
 	assert.NotNil(t, inputPopup)
 
-	folderNameInputUi := inputPopup.Content.(*fyne.Container)
+	folderNameInputUI := inputPopup.Content.(*fyne.Container)
 
-	folderNameInputTitle := folderNameInputUi.Objects[4].(*widget.Label)
+	folderNameInputTitle := folderNameInputUI.Objects[4].(*widget.Label)
 	assert.Equal(t, "New Folder", folderNameInputTitle.Text)
 
-	folderNameInputLabel := folderNameInputUi.Objects[2].(*widget.Form).Items[0].Text
+	folderNameInputLabel := folderNameInputUI.Objects[2].(*widget.Form).Items[0].Text
 	assert.Equal(t, "Name", folderNameInputLabel)
 
-	folderNameInputEntry := folderNameInputUi.Objects[2].(*widget.Form).Items[0].Widget.(*widget.Entry)
+	folderNameInputEntry := folderNameInputUI.Objects[2].(*widget.Form).Items[0].Widget.(*widget.Entry)
 	assert.Equal(t, "", folderNameInputEntry.Text)
 
-	folderNameInputCancel := folderNameInputUi.Objects[3].(*fyne.Container).Objects[1].(*widget.Button)
+	folderNameInputCancel := folderNameInputUI.Objects[3].(*fyne.Container).Objects[0].(*widget.Button)
 	assert.Equal(t, "Cancel", folderNameInputCancel.Text)
 	assert.Equal(t, theme.CancelIcon(), folderNameInputCancel.Icon)
 
-	folderNameInputCreate := folderNameInputUi.Objects[3].(*fyne.Container).Objects[2].(*widget.Button)
+	folderNameInputCreate := folderNameInputUI.Objects[3].(*fyne.Container).Objects[1].(*widget.Button)
 	assert.Equal(t, theme.ConfirmIcon(), folderNameInputCreate.Icon)
 }
+
+func TestSetOnClosedBeforeShow(t *testing.T) {
+	win := test.NewTempWindow(t, widget.NewLabel("Content"))
+	d := NewFileSave(func(fyne.URIWriteCloser, error) {}, win)
+	onClosedCalled := false
+	d.SetOnClosed(func() { onClosedCalled = true })
+	d.Show()
+	d.Hide()
+	assert.True(t, onClosedCalled)
+}
