@@ -1,5 +1,4 @@
 //go:build !windows && !android && !ios && !wasm && !js
-// +build !windows,!android,!ios,!wasm,!js
 
 package dialog
 
@@ -37,11 +36,3 @@ func isHidden(file fyne.URI) bool {
 func hideFile(filename string) error {
 	return nil
 }
-
-func fileOpenOSOverride(*FileDialog) bool {
-	return false
-}
-
-func fileSaveOSOverride(*FileDialog) bool {
-	return false
-}
