@@ -1,5 +1,4 @@
 //go:build android
-// +build android
 
 package driver
 
@@ -9,7 +8,7 @@ import "fyne.io/fyne/v2/internal/driver/mobile/app"
 // On Android this provides the JVM pointers required to execute various NDK calls or use JNI APIs.
 //
 // Since: 2.3
-func RunNative(fn func(interface{}) error) error {
+func RunNative(fn func(any) error) error {
 	return app.RunOnJVM(func(vm, env, ctx uintptr) error {
 		data := &AndroidContext{VM: vm, Env: env, Ctx: ctx}
 		return fn(data)
