@@ -11,14 +11,14 @@ func TestTextCacheGet(t *testing.T) {
 	ResetThemeCaches()
 	assert.Equal(t, 0, len(fontSizeCache))
 
-	bound, base := GetFontMetrics("hi", 10, fyne.TextStyle{})
+	bound, base := GetFontMetrics("hi", 10, fyne.TextStyle{}, nil)
 	assert.True(t, bound.IsZero())
 	assert.Equal(t, float32(0), base)
 
-	SetFontMetrics("hi", 10, fyne.TextStyle{}, fyne.NewSize(10, 10), 8)
+	SetFontMetrics("hi", 10, fyne.TextStyle{}, nil, fyne.NewSize(10, 10), 8)
 	assert.Equal(t, 1, len(fontSizeCache))
 
-	bound, base = GetFontMetrics("hi", 10, fyne.TextStyle{})
+	bound, base = GetFontMetrics("hi", 10, fyne.TextStyle{}, nil)
 	assert.Equal(t, fyne.NewSize(10, 10), bound)
 	assert.Equal(t, float32(8), base)
 }
