@@ -3,13 +3,15 @@ package glfw
 import (
 	"runtime"
 	"sync"
+	"sync/atomic"
 	"time"
 
 	"fyne.io/fyne/v2"
-	"fyne.io/fyne/v2/internal"
 	"fyne.io/fyne/v2/internal/app"
 	"fyne.io/fyne/v2/internal/cache"
+	"fyne.io/fyne/v2/internal/driver/common"
 	"fyne.io/fyne/v2/internal/painter"
+	"fyne.io/fyne/v2/internal/scale"
 )
 
 type funcData struct {
@@ -23,53 +25,33 @@ type drawData struct {
 	done chan struct{} // Zero allocation signalling channel
 }
 
-type runFlag struct {
-	sync.Mutex
-	flag bool
-	cond *sync.Cond
-}
-
 // channel for queuing functions on the main thread
 var funcQueue = make(chan funcData)
 var drawFuncQueue = make(chan drawData)
-var run *runFlag
+var running atomic.Bool
 var initOnce = &sync.Once{}
-var donePool = &sync.Pool{New: func() interface{} {
-	return make(chan struct{})
-}}
-
-func newRun() *runFlag {
-	r := runFlag{}
-	r.cond = sync.NewCond(&r)
-	return &r
-}
 
 // Arrange that main.main runs on main thread.
 func init() {
 	runtime.LockOSThread()
 	mainGoroutineID = goroutineID()
-
-	run = newRun()
 }
 
 // force a function f to run on the main thread
 func runOnMain(f func()) {
 	// If we are on main just execute - otherwise add it to the main queue and wait.
 	// The "running" variable is normally false when we are on the main thread.
-	run.Lock()
-	if !run.flag {
+	if !running.Load() {
 		f()
-		run.Unlock()
-	} else {
-		run.Unlock()
-
-		done := donePool.Get().(chan struct{})
-		defer donePool.Put(done)
-
-		funcQueue <- funcData{f: f, done: done}
-
-		<-done
+		return
 	}
+
+	done := common.DonePool.Get().(chan struct{})
+	defer common.DonePool.Put(done)
+
+	funcQueue <- funcData{f: f, done: done}
+
+	<-done
 }
 
 // force a function f to run on the draw thread
@@ -78,15 +60,22 @@ func runOnDraw(w *window, f func()) {
 		runOnMain(func() { w.RunWithContext(f) })
 		return
 	}
-	done := donePool.Get().(chan struct{})
-	defer donePool.Put(done)
+	done := common.DonePool.Get().(chan struct{})
+	defer common.DonePool.Put(done)
 
 	drawFuncQueue <- drawData{f: f, win: w, done: done}
 	<-done
 }
 
+// Preallocate to avoid allocations on every drawSingleFrame.
+// Note that the capacity of this slice can only grow,
+// but its length will never be longer than the total number of
+// window canvases that are dirty on a single frame.
+// So its memory impact should be negligible and does not
+// need periodic shrinking.
+var refreshingCanvases []fyne.Canvas
+
 func (d *gLDriver) drawSingleFrame() {
-	refreshingCanvases := make([]fyne.Canvas, 0)
 	for _, win := range d.windowList() {
 		w := win.(*window)
 		w.viewLock.RLock()
@@ -107,37 +96,45 @@ func (d *gLDriver) drawSingleFrame() {
 		refreshingCanvases = append(refreshingCanvases, canvas)
 	}
 	cache.CleanCanvases(refreshingCanvases)
+
+	// cleanup refreshingCanvases slice
+	for i := 0; i < len(refreshingCanvases); i++ {
+		refreshingCanvases[i] = nil
+	}
+	refreshingCanvases = refreshingCanvases[:0]
 }
 
 func (d *gLDriver) runGL() {
-	eventTick := time.NewTicker(time.Second / 60)
-	run.Lock()
-	run.flag = true
-	run.Unlock()
-	run.cond.Broadcast()
+	if !running.CompareAndSwap(false, true) {
+		return // Run was called twice.
+	}
+	close(d.waitForStart) // Signal that execution can continue.
 
 	d.initGLFW()
 	if d.trayStart != nil {
 		d.trayStart()
 	}
-	fyne.CurrentApp().Lifecycle().(*app.Lifecycle).TriggerStarted()
+	if f := fyne.CurrentApp().Lifecycle().(*app.Lifecycle).OnStarted(); f != nil {
+		go f() // don't block main, we don't have window event queue
+	}
+	eventTick := time.NewTicker(time.Second / 60)
 	for {
 		select {
 		case <-d.done:
 			eventTick.Stop()
-			d.drawDone <- nil // wait for draw thread to stop
+			d.drawDone <- struct{}{} // wait for draw thread to stop
 			d.Terminate()
-			fyne.CurrentApp().Lifecycle().(*app.Lifecycle).TriggerStopped()
+			l := fyne.CurrentApp().Lifecycle().(*app.Lifecycle)
+			if f := l.OnStopped(); f != nil {
+				l.QueueEvent(f)
+			}
 			return
 		case f := <-funcQueue:
 			f.f()
-			if f.done != nil {
-				f.done <- struct{}{}
-			}
+			f.done <- struct{}{}
 		case <-eventTick.C:
-			d.tryPollEvents()
-			newWindows := []fyne.Window{}
-			reassign := false
+			d.pollEvents()
+			windowsToRemove := 0
 			for _, win := range d.windowList() {
 				w := win.(*window)
 				if w.viewport == nil {
@@ -145,15 +142,7 @@ func (d *gLDriver) runGL() {
 				}
 
 				if w.viewport.ShouldClose() {
-					reassign = true
-					w.viewLock.Lock()
-					w.visible = false
-					v := w.viewport
-					w.viewLock.Unlock()
-
-					// remove window from window list
-					v.Destroy()
-					w.destroy(d)
+					windowsToRemove++
 					continue
 				}
 
@@ -169,18 +158,40 @@ func (d *gLDriver) runGL() {
 					w.shouldExpand = false
 					view := w.viewport
 					w.viewLock.Unlock()
-					if shouldExpand {
+					if shouldExpand && runtime.GOOS != "js" {
 						view.SetSize(w.shouldWidth, w.shouldHeight)
 					}
 				}
 
-				newWindows = append(newWindows, win)
-
 				if drawOnMainThread {
 					d.drawSingleFrame()
 				}
 			}
-			if reassign {
+			if windowsToRemove > 0 {
+				oldWindows := d.windowList()
+				newWindows := make([]fyne.Window, 0, len(oldWindows)-windowsToRemove)
+
+				for _, win := range oldWindows {
+					w := win.(*window)
+					if w.viewport == nil {
+						continue
+					}
+
+					if w.viewport.ShouldClose() {
+						w.viewLock.Lock()
+						w.visible = false
+						v := w.viewport
+						w.viewLock.Unlock()
+
+						// remove window from window list
+						v.Destroy()
+						w.destroy(d)
+						continue
+					}
+
+					newWindows = append(newWindows, win)
+				}
+
 				d.windowLock.Lock()
 				d.windows = newWindows
 				d.windowLock.Unlock()
@@ -196,7 +207,7 @@ func (d *gLDriver) runGL() {
 func (d *gLDriver) repaintWindow(w *window) {
 	canvas := w.canvas
 	w.RunWithContext(func() {
-		if w.canvas.EnsureMinSize() {
+		if canvas.EnsureMinSize() {
 			w.viewLock.Lock()
 			w.shouldExpand = true
 			w.viewLock.Unlock()
@@ -236,9 +247,7 @@ func (d *gLDriver) startDrawThread() {
 				return
 			case f := <-drawFuncQueue:
 				f.win.RunWithContext(f.f)
-				if f.done != nil {
-					f.done <- struct{}{}
-				}
+				f.done <- struct{}{}
 			case set := <-settingsChange:
 				painter.ClearFontCache()
 				cache.ResetThemeCaches()
@@ -263,12 +272,12 @@ func refreshWindow(w *window) {
 }
 
 func updateGLContext(w *window) {
-	canvas := w.Canvas().(*glCanvas)
+	canvas := w.canvas
 	size := canvas.Size()
 
 	// w.width and w.height are not correct if we are maximised, so figure from canvas
-	winWidth := float32(internal.ScaleInt(canvas, size.Width)) * canvas.texScale
-	winHeight := float32(internal.ScaleInt(canvas, size.Height)) * canvas.texScale
+	winWidth := float32(scale.ToScreenCoordinate(canvas, size.Width)) * canvas.texScale
+	winHeight := float32(scale.ToScreenCoordinate(canvas, size.Height)) * canvas.texScale
 
 	canvas.Painter().SetFrameBufferScale(canvas.texScale)
 	w.canvas.Painter().SetOutputSize(int(winWidth), int(winHeight))
