@@ -1,5 +1,4 @@
-//go:build !js && !wasm && !test_web_driver
-// +build !js,!wasm,!test_web_driver
+//go:build !wasm && !test_web_driver
 
 package glfw
 
@@ -13,11 +12,15 @@ import (
 
 	"fyne.io/fyne/v2"
 	"fyne.io/fyne/v2/canvas"
+	"fyne.io/fyne/v2/container"
 	"fyne.io/fyne/v2/driver/desktop"
-	"fyne.io/fyne/v2/internal"
+	"fyne.io/fyne/v2/internal/build"
 	"fyne.io/fyne/v2/internal/driver/common"
 	"fyne.io/fyne/v2/internal/painter"
 	"fyne.io/fyne/v2/internal/painter/gl"
+	"fyne.io/fyne/v2/internal/scale"
+	"fyne.io/fyne/v2/internal/svg"
+	"fyne.io/fyne/v2/storage"
 
 	"github.com/go-gl/glfw/v3.3/glfw"
 )
@@ -75,8 +78,6 @@ type window struct {
 	icon         fyne.Resource
 	mainmenu     *fyne.MainMenu
 
-	clipboard fyne.Clipboard
-
 	master     bool
 	fullScreen bool
 	centered   bool
@@ -133,6 +134,10 @@ func (w *window) SetFullScreen(full bool) {
 }
 
 func (w *window) CenterOnScreen() {
+	if build.IsWayland {
+		return
+	}
+
 	w.centered = true
 
 	if w.view() != nil {
@@ -140,6 +145,25 @@ func (w *window) CenterOnScreen() {
 	}
 }
 
+func (w *window) SetOnDropped(dropped func(pos fyne.Position, items []fyne.URI)) {
+	w.runOnMainWhenCreated(func() {
+		w.viewport.SetDropCallback(func(win *glfw.Window, names []string) {
+			if dropped == nil {
+				return
+			}
+
+			uris := make([]fyne.URI, len(names))
+			for i, name := range names {
+				uris[i] = storage.NewFileURI(name)
+			}
+
+			w.QueueEvent(func() {
+				dropped(w.mousePos, uris)
+			})
+		})
+	})
+}
+
 func (w *window) doCenterOnScreen() {
 	viewWidth, viewHeight := w.screenSize(w.canvas.size)
 	if w.width > viewWidth { // in case our window has not called back to canvas size yet
@@ -165,7 +189,7 @@ func (w *window) doCenterOnScreen() {
 }
 
 func (w *window) RequestFocus() {
-	if isWayland || w.view() == nil {
+	if build.IsWayland || w.view() == nil {
 		return
 	}
 
@@ -174,6 +198,10 @@ func (w *window) RequestFocus() {
 
 func (w *window) SetIcon(icon fyne.Resource) {
 	w.icon = icon
+	if build.IsWayland {
+		return
+	}
+
 	if icon == nil {
 		appIcon := fyne.CurrentApp().Icon()
 		if appIcon != nil {
@@ -189,7 +217,7 @@ func (w *window) SetIcon(icon fyne.Resource) {
 		}
 
 		var img image.Image
-		if painter.IsResourceSVG(w.icon) {
+		if svg.IsResourceSVG(w.icon) {
 			img = painter.PaintImage(&canvas.Image{Resource: w.icon}, nil, windowIconSize, windowIconSize)
 		} else {
 			pix, _, err := image.Decode(bytes.NewReader(w.icon.Content()))
@@ -247,24 +275,26 @@ func (w *window) fitContent() {
 }
 
 func (w *window) getMonitorForWindow() *glfw.Monitor {
-	x, y := w.xpos, w.ypos
-	if w.fullScreen {
-		x, y = w.viewport.GetPos()
-	}
-	xOff := x + (w.width / 2)
-	yOff := y + (w.height / 2)
-
-	for _, monitor := range glfw.GetMonitors() {
-		x, y := monitor.GetPos()
-
-		if x > xOff || y > yOff {
-			continue
-		}
-		if x+monitor.GetVideoMode().Width <= xOff || y+monitor.GetVideoMode().Height <= yOff {
-			continue
+	if !build.IsWayland {
+		x, y := w.xpos, w.ypos
+		if w.fullScreen {
+			x, y = w.viewport.GetPos()
 		}
+		xOff := x + (w.width / 2)
+		yOff := y + (w.height / 2)
 
-		return monitor
+		for _, monitor := range glfw.GetMonitors() {
+			x, y := monitor.GetPos()
+
+			if x > xOff || y > yOff {
+				continue
+			}
+			if videoMode := monitor.GetVideoMode(); x+videoMode.Width <= xOff || y+videoMode.Height <= yOff {
+				continue
+			}
+
+			return monitor
+		}
 	}
 
 	// try built-in function to detect monitor if above logic didn't succeed
@@ -277,15 +307,18 @@ func (w *window) getMonitorForWindow() *glfw.Monitor {
 }
 
 func (w *window) detectScale() float32 {
-	if isWayland { // Wayland controls scale through content scaling
-		return 1.0
+	if build.IsWayland { // Wayland controls scale through content scaling
+		return 1
 	}
 	monitor := w.getMonitorForWindow()
 	if monitor == nil {
-		return 1.0
+		return 1
 	}
 
-	widthMm, _ := monitor.GetPhysicalSize()
+	widthMm, heightMm := monitor.GetPhysicalSize()
+	if runtime.GOOS == "linux" && widthMm == 60 && heightMm == 60 { // Steam Deck incorrectly reports 6cm square!
+		return 1
+	}
 	widthPx := monitor.GetVideoMode().Width
 
 	return calculateDetectedScale(widthMm, widthPx)
@@ -300,7 +333,7 @@ func (w *window) resized(_ *glfw.Window, width, height int) {
 }
 
 func (w *window) scaled(_ *glfw.Window, x float32, y float32) {
-	if !isWayland { // other platforms handle this using older APIs
+	if !build.IsWayland { // other platforms handle this using older APIs
 		return
 	}
 
@@ -564,10 +597,6 @@ func keyCodeToKeyName(code string) fyne.KeyName {
 }
 
 func keyToName(code glfw.Key, scancode int) fyne.KeyName {
-	if runtime.GOOS == "darwin" && scancode == 0x69 { // TODO remove once fixed upstream glfw/glfw#1786
-		code = glfw.KeyPrintScreen
-	}
-
 	ret := glfwKeyToKeyName(code)
 	if ret != fyne.KeyUnknown {
 		return ret
@@ -600,6 +629,7 @@ func convertASCII(key glfw.Key) fyne.KeyName {
 func (w *window) keyPressed(_ *glfw.Window, key glfw.Key, scancode int, action glfw.Action, mods glfw.ModifierKey) {
 	keyName := keyToName(key, scancode)
 	keyDesktopModifier := desktopModifier(mods)
+	w.driver.currentKeyModifiers = desktopModifierCorrected(mods, key, action)
 	keyAction := convertAction(action)
 	keyASCII := convertASCII(key)
 
@@ -623,6 +653,32 @@ func desktopModifier(mods glfw.ModifierKey) fyne.KeyModifier {
 	return m
 }
 
+func desktopModifierCorrected(mods glfw.ModifierKey, key glfw.Key, action glfw.Action) fyne.KeyModifier {
+	// On X11, pressing/releasing modifier keys does not include newly pressed/released keys in 'mod' mask.
+	// https://github.com/glfw/glfw/issues/1630
+	if action == glfw.Press {
+		mods |= glfwKeyToModifier(key)
+	} else {
+		mods &= ^glfwKeyToModifier(key)
+	}
+	return desktopModifier(mods)
+}
+
+func glfwKeyToModifier(key glfw.Key) glfw.ModifierKey {
+	var m glfw.ModifierKey
+	switch key {
+	case glfw.KeyLeftControl, glfw.KeyRightControl:
+		m = glfw.ModControl
+	case glfw.KeyLeftAlt, glfw.KeyRightAlt:
+		m = glfw.ModAlt
+	case glfw.KeyLeftShift, glfw.KeyRightShift:
+		m = glfw.ModShift
+	case glfw.KeyLeftSuper, glfw.KeyRightSuper:
+		m = glfw.ModSuper
+	}
+	return m
+}
+
 // charInput defines the character with modifiers callback which is called when a
 // Unicode character is input.
 //
@@ -648,8 +704,8 @@ func (w *window) rescaleOnMain() {
 	if w.fullScreen {
 		w.width, w.height = w.viewport.GetSize()
 		scaledFull := fyne.NewSize(
-			internal.UnscaleInt(w.canvas, w.width),
-			internal.UnscaleInt(w.canvas, w.height))
+			scale.ToFyneCoordinate(w.canvas, w.width),
+			scale.ToFyneCoordinate(w.canvas, w.height))
 		w.canvas.Resize(scaledFull)
 		return
 	}
@@ -661,7 +717,7 @@ func (w *window) rescaleOnMain() {
 
 func (w *window) create() {
 	runOnMain(func() {
-		if !isWayland {
+		if !build.IsWayland {
 			// make the window hidden, we will set it up and then show it later
 			glfw.WindowHint(glfw.Visible, glfw.False)
 		}
@@ -737,7 +793,7 @@ func (w *window) create() {
 
 		if w.FixedSize() && (w.requestedWidth == 0 || w.requestedHeight == 0) {
 			bigEnough := w.canvas.canvasSize(w.canvas.Content().MinSize())
-			w.width, w.height = internal.ScaleInt(w.canvas, bigEnough.Width), internal.ScaleInt(w.canvas, bigEnough.Height)
+			w.width, w.height = scale.ToScreenCoordinate(w.canvas, bigEnough.Width), scale.ToScreenCoordinate(w.canvas, bigEnough.Height)
 			w.shouldWidth, w.shouldHeight = w.width, w.height
 		}
 
@@ -756,3 +812,8 @@ func (w *window) view() *glfw.Window {
 	}
 	return w.viewport
 }
+
+// wrapInnerWindow is a no-op to match what the web driver provides
+func wrapInnerWindow(*container.InnerWindow, fyne.Window, *gLDriver) fyne.Window {
+	return nil
+}
