@@ -6,7 +6,9 @@ import (
 	"unsafe"
 
 	"fyne.io/fyne/v2"
-	"fyne.io/fyne/v2/internal"
+	"fyne.io/fyne/v2/driver"
+	"fyne.io/fyne/v2/internal/scale"
+
 	"golang.org/x/sys/windows/registry"
 )
 
@@ -45,7 +47,22 @@ func isDark() bool {
 
 func (w *window) computeCanvasSize(width, height int) fyne.Size {
 	if w.fixedSize {
-		return fyne.NewSize(internal.UnscaleInt(w.canvas, w.width), internal.UnscaleInt(w.canvas, w.height))
+		return fyne.NewSize(scale.ToFyneCoordinate(w.canvas, w.width), scale.ToFyneCoordinate(w.canvas, w.height))
 	}
-	return fyne.NewSize(internal.UnscaleInt(w.canvas, width), internal.UnscaleInt(w.canvas, height))
+	return fyne.NewSize(scale.ToFyneCoordinate(w.canvas, width), scale.ToFyneCoordinate(w.canvas, height))
+}
+
+// assert we are implementing driver.NativeWindow
+var _ driver.NativeWindow = (*window)(nil)
+
+func (w *window) RunNative(f func(any)) {
+	var hwnd uintptr
+	if v := w.view(); v != nil {
+		hwnd = uintptr(unsafe.Pointer(v.GetWin32Window()))
+	}
+	runOnMain(func() {
+		f(driver.WindowsWindowContext{
+			HWND: hwnd,
+		})
+	})
 }
