@@ -3,7 +3,6 @@
 // license that can be found in the LICENSE file.
 
 //go:build android
-// +build android
 
 /*
 Android Apps are built with -buildmode=c-shared. They are loaded by a
@@ -48,6 +47,7 @@ void showKeyboard(JNIEnv* env, int keyboardType);
 void hideKeyboard(JNIEnv* env);
 void showFileOpen(JNIEnv* env, char* mimes);
 void showFileSave(JNIEnv* env, char* mimes, char* filename);
+void finish(JNIEnv* env, jobject ctx);
 
 void Java_org_golang_app_GoNativeActivity_filePickerReturned(JNIEnv *env, jclass clazz, jstring str);
 */
@@ -57,8 +57,6 @@ import (
 	"log"
 	"mime"
 	"os"
-	"runtime"
-	"runtime/debug"
 	"strings"
 	"time"
 	"unsafe"
@@ -77,6 +75,18 @@ var mimeMap = map[string]string{
 	".txt": "text/plain",
 }
 
+// GoBack asks the OS to go to the previous app / activity
+func GoBack() {
+	err := RunOnJVM(func(_, jniEnv, ctx uintptr) error {
+		env := (*C.JNIEnv)(unsafe.Pointer(jniEnv))
+		C.finish(env, C.jobject(ctx))
+		return nil
+	})
+	if err != nil {
+		log.Fatalf("app: %v", err)
+	}
+}
+
 // RunOnJVM runs fn on a new goroutine locked to an OS thread with a JNIEnv.
 //
 // RunOnJVM blocks until the call to fn is complete. Any Java
@@ -118,25 +128,21 @@ func callMain(mainPC uintptr) {
 	go callfn.CallFn(mainPC)
 }
 
-//export onStart
-func onStart(activity *C.ANativeActivity) {
-}
-
-//export onResume
-func onResume(activity *C.ANativeActivity) {
-}
-
 //export onSaveInstanceState
 func onSaveInstanceState(activity *C.ANativeActivity, outSize *C.size_t) unsafe.Pointer {
 	return nil
 }
 
-//export onPause
-func onPause(activity *C.ANativeActivity) {
-}
+//export onBackPressed
+func onBackPressed() {
+	k := key.Event{
+		Code:      key.CodeBackButton,
+		Direction: key.DirPress,
+	}
+	theApp.events.In() <- k
 
-//export onStop
-func onStop(activity *C.ANativeActivity) {
+	k.Direction = key.DirRelease
+	theApp.events.In() <- k
 }
 
 //export onCreate
@@ -270,8 +276,7 @@ func onConfigurationChanged(activity *C.ANativeActivity) {
 
 //export onLowMemory
 func onLowMemory(activity *C.ANativeActivity) {
-	runtime.GC()
-	debug.FreeOSMemory()
+	cleanCaches()
 }
 
 var (
@@ -283,8 +288,8 @@ var (
 	windowConfigChange = make(chan windowConfig)
 	activityDestroyed  = make(chan struct{})
 
-	screenInsetTop, screenInsetBottom, screenInsetLeft, screenInsetRight int
-	darkMode                                                             bool
+	currentSize size.Event
+	darkMode    bool
 )
 
 func init() {
@@ -346,7 +351,12 @@ func filePickerReturned(str *C.char) {
 
 //export insetsChanged
 func insetsChanged(top, bottom, left, right int) {
-	screenInsetTop, screenInsetBottom, screenInsetLeft, screenInsetRight = top, bottom, left, right
+	currentSize.InsetTopPx = top
+	currentSize.InsetBottomPx = bottom
+	currentSize.InsetLeftPx = left
+	currentSize.InsetRightPx = right
+
+	theApp.events.In() <- currentSize
 }
 
 func mimeStringFromFilter(filter *FileFilter) string {
@@ -449,20 +459,21 @@ func mainUI(vm, jniEnv, ctx uintptr) error {
 			theApp.sendLifecycle(lifecycle.StageFocused)
 			widthPx := int(C.ANativeWindow_getWidth(w))
 			heightPx := int(C.ANativeWindow_getHeight(w))
-			theApp.events.In() <- size.Event{
+			currentSize = size.Event{
 				WidthPx:       widthPx,
 				HeightPx:      heightPx,
 				WidthPt:       float32(widthPx) / pixelsPerPt,
 				HeightPt:      float32(heightPx) / pixelsPerPt,
-				InsetTopPx:    screenInsetTop,
-				InsetBottomPx: screenInsetBottom,
-				InsetLeftPx:   screenInsetLeft,
-				InsetRightPx:  screenInsetRight,
+				InsetTopPx:    currentSize.InsetTopPx,
+				InsetBottomPx: currentSize.InsetBottomPx,
+				InsetLeftPx:   currentSize.InsetLeftPx,
+				InsetRightPx:  currentSize.InsetRightPx,
 				PixelsPerPt:   pixelsPerPt,
 				Orientation:   screenOrientation(widthPx, heightPx), // we are guessing orientation here as it was not always working
 				DarkMode:      darkMode,
 			}
-			theApp.events.In() <- paint.Event{External: true}
+			theApp.events.In() <- currentSize
+			theApp.events.In() <- paint.Event{External: true, Window: uintptr(unsafe.Pointer(w))}
 		case <-windowDestroyed:
 			if C.surface != nil {
 				if errStr := C.destroyEGLSurface(); errStr != nil {
@@ -508,7 +519,7 @@ func runInputQueue(vm, jniEnv, ctx uintptr) error {
 
 	var q *C.AInputQueue
 	for {
-		if C.ALooper_pollAll(-1, nil, nil, nil) == C.ALOOPER_POLL_WAKE {
+		if C.ALooper_pollOnce(-1, nil, nil, nil) == C.ALOOPER_POLL_WAKE {
 			select {
 			default:
 			case p := <-pending:
