@@ -1,14 +1,14 @@
 package mobile
 
 import (
+	"fyne.io/fyne/v2"
 	"fyne.io/fyne/v2/driver/mobile"
 	"fyne.io/fyne/v2/internal/driver/mobile/event/size"
-
-	"fyne.io/fyne/v2"
+	"fyne.io/fyne/v2/lang"
 )
 
 type device struct {
-	safeTop, safeLeft, safeWidth, safeHeight int
+	safeTop, safeLeft, safeBottom, safeRight int
 }
 
 //lint:file-ignore U1000 Var currentDPI is used in other files, but not here
@@ -20,6 +20,10 @@ var (
 // Declare conformity with Device
 var _ fyne.Device = (*device)(nil)
 
+func (*device) Locale() fyne.Locale {
+	return lang.SystemLocale()
+}
+
 func (*device) Orientation() fyne.DeviceOrientation {
 	switch currentOrientation {
 	case size.OrientationLandscape:
