@@ -1,5 +1,4 @@
 //go:build wayland
-// +build wayland
 
 package mobile
 
@@ -10,3 +9,7 @@ const tapYOffset = -4.0 // to compensate for how we hold our fingers on the devi
 func (*device) SystemScaleForWindow(_ fyne.Window) float32 {
 	return 1 // PinePhone simplification, our only wayland mobile currently
 }
+
+func setDisableScreenBlank(_ bool) {
+	// ignore in mobile simulation mode
+}
