@@ -3,15 +3,18 @@ package mobile
 import (
 	"runtime"
 	"strconv"
+	"sync/atomic"
 	"time"
 
 	"fyne.io/fyne/v2"
-	"fyne.io/fyne/v2/canvas"
+	fynecanvas "fyne.io/fyne/v2/canvas"
+	"fyne.io/fyne/v2/driver/mobile"
 	"fyne.io/fyne/v2/internal"
 	"fyne.io/fyne/v2/internal/animation"
 	intapp "fyne.io/fyne/v2/internal/app"
+	"fyne.io/fyne/v2/internal/build"
 	"fyne.io/fyne/v2/internal/cache"
-	"fyne.io/fyne/v2/internal/driver"
+	intdriver "fyne.io/fyne/v2/internal/driver"
 	"fyne.io/fyne/v2/internal/driver/common"
 	"fyne.io/fyne/v2/internal/driver/mobile/app"
 	"fyne.io/fyne/v2/internal/driver/mobile/event/key"
@@ -22,12 +25,14 @@ import (
 	"fyne.io/fyne/v2/internal/driver/mobile/gl"
 	"fyne.io/fyne/v2/internal/painter"
 	pgl "fyne.io/fyne/v2/internal/painter/gl"
+	"fyne.io/fyne/v2/internal/scale"
 	"fyne.io/fyne/v2/theme"
 )
 
 const (
 	tapMoveThreshold  = 4.0                    // how far can we move before it is a drag
 	tapSecondaryDelay = 300 * time.Millisecond // how long before secondary tap
+	tapDoubleDelay    = 500 * time.Millisecond // max duration between taps for a DoubleTap event
 )
 
 // Configuration is the system information about the current device
@@ -40,45 +45,46 @@ type ConfiguredDriver interface {
 	SetOnConfigurationChanged(func(*Configuration))
 }
 
-type mobileDriver struct {
+type driver struct {
 	app   app.App
 	glctx gl.Context
 
 	windows     []fyne.Window
-	device      *device
-	animation   *animation.Runner
+	device      device
+	animation   animation.Runner
 	currentSize size.Event
 
 	theme           fyne.ThemeVariant
 	onConfigChanged func(*Configuration)
 	painting        bool
+	running         atomic.Bool
 }
 
 // Declare conformity with Driver
-var _ fyne.Driver = (*mobileDriver)(nil)
-var _ ConfiguredDriver = (*mobileDriver)(nil)
+var _ fyne.Driver = (*driver)(nil)
+var _ ConfiguredDriver = (*driver)(nil)
 
 func init() {
 	runtime.LockOSThread()
 }
 
-func (d *mobileDriver) CreateWindow(title string) fyne.Window {
-	c := NewCanvas().(*mobileCanvas) // silence lint
+func (d *driver) CreateWindow(title string) fyne.Window {
+	c := newCanvas(fyne.CurrentDevice()).(*canvas) // silence lint
 	ret := &window{title: title, canvas: c, isChild: len(d.windows) > 0}
 	ret.InitEventQueue()
 	go ret.RunEventQueue()
-	c.setContent(&canvas.Rectangle{FillColor: theme.BackgroundColor()})
+	c.setContent(&fynecanvas.Rectangle{FillColor: theme.Color(theme.ColorNameBackground)})
 	c.SetPainter(pgl.NewPainter(c, ret))
 	d.windows = append(d.windows, ret)
 	return ret
 }
 
-func (d *mobileDriver) AllWindows() []fyne.Window {
+func (d *driver) AllWindows() []fyne.Window {
 	return d.windows
 }
 
 // currentWindow returns the most recently opened window - we can only show one at a time.
-func (d *mobileDriver) currentWindow() *window {
+func (d *driver) currentWindow() *window {
 	if len(d.windows) == 0 {
 		return nil
 	}
@@ -94,11 +100,11 @@ func (d *mobileDriver) currentWindow() *window {
 	return last
 }
 
-func (d *mobileDriver) RenderedTextSize(text string, textSize float32, style fyne.TextStyle) (size fyne.Size, baseline float32) {
-	return painter.RenderedTextSize(text, textSize, style)
+func (d *driver) RenderedTextSize(text string, textSize float32, style fyne.TextStyle, source fyne.Resource) (size fyne.Size, baseline float32) {
+	return painter.RenderedTextSize(text, textSize, style, source)
 }
 
-func (d *mobileDriver) CanvasForObject(obj fyne.CanvasObject) fyne.Canvas {
+func (d *driver) CanvasForObject(obj fyne.CanvasObject) fyne.Canvas {
 	if len(d.windows) == 0 {
 		return nil
 	}
@@ -107,35 +113,41 @@ func (d *mobileDriver) CanvasForObject(obj fyne.CanvasObject) fyne.Canvas {
 	return d.currentWindow().Canvas()
 }
 
-func (d *mobileDriver) AbsolutePositionForObject(co fyne.CanvasObject) fyne.Position {
+func (d *driver) AbsolutePositionForObject(co fyne.CanvasObject) fyne.Position {
 	c := d.CanvasForObject(co)
 	if c == nil {
 		return fyne.NewPos(0, 0)
 	}
 
-	mc := c.(*mobileCanvas)
-	pos := driver.AbsolutePositionForObject(co, mc.ObjectTrees())
+	mc := c.(*canvas)
+	pos := intdriver.AbsolutePositionForObject(co, mc.ObjectTrees())
 	inset, _ := c.InteractiveArea()
-
-	if mc.windowHead != nil {
-		if len(mc.windowHead.(*fyne.Container).Objects) > 1 {
-			topHeight := mc.windowHead.MinSize().Height
-			pos = pos.Subtract(fyne.NewSize(0, topHeight))
-		}
-	}
 	return pos.Subtract(inset)
 }
 
-func (d *mobileDriver) Quit() {
+func (d *driver) GoBack() {
+	app.GoBack()
+}
+
+func (d *driver) Quit() {
 	// Android and iOS guidelines say this should not be allowed!
 }
 
-func (d *mobileDriver) Run() {
+func (d *driver) Run() {
+	if !d.running.CompareAndSwap(false, true) {
+		return // Run was called twice.
+	}
+
 	app.Main(func(a app.App) {
 		d.app = a
 		settingsChange := make(chan fyne.Settings)
 		fyne.CurrentApp().Settings().AddChangeListener(settingsChange)
 		draw := time.NewTicker(time.Second / 60)
+		defer func() {
+			l := fyne.CurrentApp().Lifecycle().(*intapp.Lifecycle)
+			l.WaitForEvents()
+			l.DestroyEventQueue()
+		}()
 
 		for {
 			select {
@@ -145,7 +157,7 @@ func (d *mobileDriver) Run() {
 				painter.ClearFontCache()
 				cache.ResetThemeCaches()
 				intapp.ApplySettingsWithCallback(set, fyne.CurrentApp(), func(w fyne.Window) {
-					c, ok := w.Canvas().(*mobileCanvas)
+					c, ok := w.Canvas().(*canvas)
 					if !ok {
 						return
 					}
@@ -159,7 +171,7 @@ func (d *mobileDriver) Run() {
 				if current == nil {
 					continue
 				}
-				c := current.Canvas().(*mobileCanvas)
+				c := current.Canvas().(*canvas)
 
 				switch e := a.Filter(e).(type) {
 				case lifecycle.Event:
@@ -173,14 +185,19 @@ func (d *mobileDriver) Run() {
 					currentDPI = e.PixelsPerPt * 72
 					d.setTheme(e.DarkMode)
 
-					dev := d.device
+					dev := &d.device
+					insetChange := dev.safeTop != e.InsetTopPx || dev.safeBottom != e.InsetBottomPx ||
+						dev.safeLeft != e.InsetLeftPx || dev.safeRight != e.InsetRightPx
 					dev.safeTop = e.InsetTopPx
 					dev.safeLeft = e.InsetLeftPx
-					dev.safeHeight = e.HeightPx - e.InsetTopPx - e.InsetBottomPx
-					dev.safeWidth = e.WidthPx - e.InsetLeftPx - e.InsetRightPx
+					dev.safeBottom = e.InsetBottomPx
+					dev.safeRight = e.InsetRightPx
 					c.scale = fyne.CurrentDevice().SystemScaleForWindow(nil)
 					c.Painter().SetFrameBufferScale(1.0)
 
+					if insetChange {
+						current.canvas.sizeContent(current.canvas.size) // even if size didn't change we invalidate
+					}
 					// make sure that we paint on the next frame
 					c.Content().Refresh()
 				case paint.Event:
@@ -206,8 +223,12 @@ func (d *mobileDriver) Run() {
 	})
 }
 
-func (d *mobileDriver) handleLifecycle(e lifecycle.Event, w fyne.Window) {
-	c := w.Canvas().(*mobileCanvas)
+func (*driver) SetDisableScreenBlanking(disable bool) {
+	setDisableScreenBlank(disable)
+}
+
+func (d *driver) handleLifecycle(e lifecycle.Event, w *window) {
+	c := w.Canvas().(*canvas)
 	switch e.Crosses(lifecycle.StageVisible) {
 	case lifecycle.CrossOn:
 		d.glctx, _ = e.DrawContext.(gl.Context)
@@ -223,9 +244,11 @@ func (d *mobileDriver) handleLifecycle(e lifecycle.Event, w fyne.Window) {
 	}
 	switch e.Crosses(lifecycle.StageFocused) {
 	case lifecycle.CrossOn: // foregrounding
-		fyne.CurrentApp().Lifecycle().(*intapp.Lifecycle).TriggerEnteredForeground()
+		if f := fyne.CurrentApp().Lifecycle().(*intapp.Lifecycle).OnEnteredForeground(); f != nil {
+			w.QueueEvent(f)
+		}
 	case lifecycle.CrossOff: // will enter background
-		if runtime.GOOS == "darwin" {
+		if runtime.GOOS == "darwin" || runtime.GOOS == "ios" {
 			if d.glctx == nil {
 				return
 			}
@@ -234,18 +257,23 @@ func (d *mobileDriver) handleLifecycle(e lifecycle.Event, w fyne.Window) {
 			d.paintWindow(w, s)
 			d.app.Publish()
 		}
-		fyne.CurrentApp().Lifecycle().(*intapp.Lifecycle).TriggerExitedForeground()
+		if f := fyne.CurrentApp().Lifecycle().(*intapp.Lifecycle).OnExitedForeground(); f != nil {
+			w.QueueEvent(f)
+		}
 	}
 }
 
-func (d *mobileDriver) handlePaint(e paint.Event, w fyne.Window) {
-	c := w.Canvas().(*mobileCanvas)
+func (d *driver) handlePaint(e paint.Event, w *window) {
+	c := w.Canvas().(*canvas)
+	if e.Window != 0 { // not all paint events come from hardware
+		w.handle = e.Window
+	}
 	d.painting = false
 	if d.glctx == nil || e.External {
 		return
 	}
-	if !c.inited {
-		c.inited = true
+	if !c.initialized {
+		c.initialized = true
 		c.Painter().Init() // we cannot init until the context is set above
 	}
 
@@ -265,19 +293,24 @@ func (d *mobileDriver) handlePaint(e paint.Event, w fyne.Window) {
 	cache.Clean(canvasNeedRefresh)
 }
 
-func (d *mobileDriver) onStart() {
-	fyne.CurrentApp().Lifecycle().(*intapp.Lifecycle).TriggerStarted()
+func (d *driver) onStart() {
+	if f := fyne.CurrentApp().Lifecycle().(*intapp.Lifecycle).OnStarted(); f != nil {
+		go f() // don't block main, we don't have window event queue
+	}
 }
 
-func (d *mobileDriver) onStop() {
-	fyne.CurrentApp().Lifecycle().(*intapp.Lifecycle).TriggerStopped()
+func (d *driver) onStop() {
+	l := fyne.CurrentApp().Lifecycle().(*intapp.Lifecycle)
+	if f := l.OnStopped(); f != nil {
+		l.QueueEvent(f)
+	}
 }
 
-func (d *mobileDriver) paintWindow(window fyne.Window, size fyne.Size) {
+func (d *driver) paintWindow(window fyne.Window, size fyne.Size) {
 	clips := &internal.ClipStack{}
-	c := window.Canvas().(*mobileCanvas)
+	c := window.Canvas().(*canvas)
 
-	r, g, b, a := theme.BackgroundColor().RGBA()
+	r, g, b, a := theme.Color(theme.ColorNameBackground).RGBA()
 	max16bit := float32(255 * 255)
 	d.glctx.ClearColor(float32(r)/max16bit, float32(g)/max16bit, float32(b)/max16bit, float32(a)/max16bit)
 	d.glctx.Clear(gl.ColorBufferBit)
@@ -294,7 +327,7 @@ func (d *mobileDriver) paintWindow(window fyne.Window, size fyne.Size) {
 		}
 		c.Painter().Paint(obj, pos, size)
 	}
-	afterDraw := func(node *common.RenderCacheNode) {
+	afterDraw := func(node *common.RenderCacheNode, pos fyne.Position) {
 		if _, ok := node.Obj().(fyne.Scrollable); ok {
 			c.Painter().StopClipping()
 			clips.Pop()
@@ -302,12 +335,16 @@ func (d *mobileDriver) paintWindow(window fyne.Window, size fyne.Size) {
 				c.Painter().StartClipping(top.Rect())
 			}
 		}
+
+		if build.Mode == fyne.BuildDebug {
+			c.DrawDebugOverlay(node.Obj(), pos, size)
+		}
 	}
 
 	c.WalkTrees(draw, afterDraw)
 }
 
-func (d *mobileDriver) sendPaintEvent() {
+func (d *driver) sendPaintEvent() {
 	if d.painting {
 		return
 	}
@@ -315,7 +352,7 @@ func (d *mobileDriver) sendPaintEvent() {
 	d.painting = true
 }
 
-func (d *mobileDriver) setTheme(dark bool) {
+func (d *driver) setTheme(dark bool) {
 	var mode fyne.ThemeVariant
 	if dark {
 		mode = theme.VariantDark
@@ -329,17 +366,17 @@ func (d *mobileDriver) setTheme(dark bool) {
 	d.theme = mode
 }
 
-func (d *mobileDriver) tapDownCanvas(w *window, x, y float32, tapID touch.Sequence) {
-	tapX := internal.UnscaleInt(w.canvas, int(x))
-	tapY := internal.UnscaleInt(w.canvas, int(y))
+func (d *driver) tapDownCanvas(w *window, x, y float32, tapID touch.Sequence) {
+	tapX := scale.ToFyneCoordinate(w.canvas, int(x))
+	tapY := scale.ToFyneCoordinate(w.canvas, int(y))
 	pos := fyne.NewPos(tapX, tapY+tapYOffset)
 
 	w.canvas.tapDown(pos, int(tapID))
 }
 
-func (d *mobileDriver) tapMoveCanvas(w *window, x, y float32, tapID touch.Sequence) {
-	tapX := internal.UnscaleInt(w.canvas, int(x))
-	tapY := internal.UnscaleInt(w.canvas, int(y))
+func (d *driver) tapMoveCanvas(w *window, x, y float32, tapID touch.Sequence) {
+	tapX := scale.ToFyneCoordinate(w.canvas, int(x))
+	tapY := scale.ToFyneCoordinate(w.canvas, int(y))
 	pos := fyne.NewPos(tapX, tapY+tapYOffset)
 
 	w.canvas.tapMove(pos, int(tapID), func(wid fyne.Draggable, ev *fyne.DragEvent) {
@@ -347,9 +384,9 @@ func (d *mobileDriver) tapMoveCanvas(w *window, x, y float32, tapID touch.Sequen
 	})
 }
 
-func (d *mobileDriver) tapUpCanvas(w *window, x, y float32, tapID touch.Sequence) {
-	tapX := internal.UnscaleInt(w.canvas, int(x))
-	tapY := internal.UnscaleInt(w.canvas, int(y))
+func (d *driver) tapUpCanvas(w *window, x, y float32, tapID touch.Sequence) {
+	tapX := scale.ToFyneCoordinate(w.canvas, int(x))
+	tapY := scale.ToFyneCoordinate(w.canvas, int(y))
 	pos := fyne.NewPos(tapX, tapY+tapYOffset)
 
 	w.canvas.tapUp(pos, int(tapID), func(wid fyne.Tappable, ev *fyne.PointEvent) {
@@ -453,6 +490,8 @@ var keyCodeMap = map[key.Code]fyne.KeyName{
 	key.CodeBackslash:          fyne.KeyBackslash,
 	key.CodeRightSquareBracket: fyne.KeyRightBracket,
 	key.CodeGraveAccent:        fyne.KeyBackTick,
+
+	key.CodeBackButton: mobile.KeyBack,
 }
 
 func keyToName(code key.Code) fyne.KeyName {
@@ -472,7 +511,7 @@ func runeToPrintable(r rune) rune {
 	return 0
 }
 
-func (d *mobileDriver) typeDownCanvas(canvas *mobileCanvas, r rune, code key.Code, mod key.Modifiers) {
+func (d *driver) typeDownCanvas(canvas *canvas, r rune, code key.Code, mod key.Modifiers) {
 	keyName := keyToName(code)
 	switch keyName {
 	case fyne.KeyTab:
@@ -503,8 +542,12 @@ func (d *mobileDriver) typeDownCanvas(canvas *mobileCanvas, r rune, code key.Cod
 			canvas.Focused().TypedRune(r)
 		}
 	} else {
-		if keyName != "" && canvas.onTypedKey != nil {
-			canvas.onTypedKey(keyEvent)
+		if keyName != "" {
+			if canvas.onTypedKey != nil {
+				canvas.onTypedKey(keyEvent)
+			} else if keyName == mobile.KeyBack {
+				d.GoBack()
+			}
 		}
 		if r > 0 && canvas.onTypedRune != nil {
 			canvas.onTypedRune(r)
@@ -512,27 +555,27 @@ func (d *mobileDriver) typeDownCanvas(canvas *mobileCanvas, r rune, code key.Cod
 	}
 }
 
-func (d *mobileDriver) typeUpCanvas(_ *mobileCanvas, _ rune, _ key.Code, _ key.Modifiers) {
+func (d *driver) typeUpCanvas(_ *canvas, _ rune, _ key.Code, _ key.Modifiers) {
 }
 
-func (d *mobileDriver) Device() fyne.Device {
-	if d.device == nil {
-		d.device = &device{}
-	}
-
-	return d.device
+func (d *driver) Device() fyne.Device {
+	return &d.device
 }
 
-func (d *mobileDriver) SetOnConfigurationChanged(f func(*Configuration)) {
+func (d *driver) SetOnConfigurationChanged(f func(*Configuration)) {
 	d.onConfigChanged = f
 }
 
+func (d *driver) DoubleTapDelay() time.Duration {
+	return tapDoubleDelay
+}
+
 // NewGoMobileDriver sets up a new Driver instance implemented using the Go
 // Mobile extension and OpenGL bindings.
 func NewGoMobileDriver() fyne.Driver {
-	d := new(mobileDriver)
-	d.theme = fyne.ThemeVariant(2) // unspecified
-	d.animation = &animation.Runner{}
+	d := &driver{
+		theme: fyne.ThemeVariant(2), // unspecified
+	}
 
 	registerRepository(d)
 	return d
