@@ -1,5 +1,4 @@
 //go:build !ios && !android
-// +build !ios,!android
 
 package mobile
 
@@ -11,6 +10,11 @@ import (
 	"fyne.io/fyne/v2/storage/repository"
 )
 
+func deleteURI(u fyne.URI) error {
+	// no-op as we use the internal FileRepository
+	return nil
+}
+
 func existsURI(fyne.URI) (bool, error) {
 	// no-op as we use the internal FileRepository
 	return false, nil
@@ -26,6 +30,6 @@ func nativeFileSave(*fileSave) (io.WriteCloser, error) {
 	return nil, nil
 }
 
-func registerRepository(d *mobileDriver) {
+func registerRepository(d *driver) {
 	repository.Register("file", intRepo.NewFileRepository())
 }
