@@ -1,5 +1,4 @@
 //go:build ios
-// +build ios
 
 #import <Foundation/Foundation.h>
 
@@ -9,7 +8,7 @@ NSArray *listForURL(const char* urlCstr) {
     NSString *urlStr = [NSString stringWithUTF8String:urlCstr];
     NSURL *url = [NSURL URLWithString:urlStr];
 
-    return [[NSFileManager defaultManager] contentsOfDirectoryAtURL:url includingPropertiesForKeys:nil options:nil error:nil];
+    return [[NSFileManager defaultManager] contentsOfDirectoryAtURL:url includingPropertiesForKeys:nil options:0 error:nil];
 }
 
 bool iosCanList(const char* url) {
@@ -26,5 +25,5 @@ bool iosCreateListable(const char* urlCstr) {
 char* iosList(const char* url) {
     NSArray *children = listForURL(url);
 
-    return [[children componentsJoinedByString:@"|"] UTF8String];
+    return (char *) [[children componentsJoinedByString:@"|"] UTF8String];
 }
