@@ -69,7 +69,6 @@ typedef enum {
 	glfnGetShaderiv,
 	glfnGetTexParameteriv,
 	glfnGetUniformLocation,
-	glfnPixelStorei,
 	glfnLinkProgram,
 	glfnReadPixels,
 	glfnScissor,
@@ -77,6 +76,7 @@ typedef enum {
 	glfnTexImage2D,
 	glfnTexParameteri,
 	glfnUniform1f,
+	glfnUniform2f,
 	glfnUniform4f,
 	glfnUniform4fv,
 	glfnUseProgram,
