@@ -1,5 +1,4 @@
 //go:build !ci
-// +build !ci
 
 package gl
 
@@ -9,6 +8,7 @@ import (
 
 	"fyne.io/fyne/v2"
 	"fyne.io/fyne/v2/canvas"
+	"fyne.io/fyne/v2/test"
 	"fyne.io/fyne/v2/theme"
 )
 
@@ -21,7 +21,9 @@ func TestDrawImage_Ratio(t *testing.T) {
 	//	win := d.CreateWindow("Test")
 	//	c := win.Canvas().(*glCanvas)
 
-	img := canvas.NewImageFromResource(theme.FyneLogo())
+	test.NewApp() // Need an app started to get safeIconLookup to work.
+
+	img := canvas.NewImageFromResource(theme.ComputerIcon())
 	img.Resize(fyne.NewSize(10, 10))
 	//	c.newGlImageTexture(img)
 	//	assert.Equal(t, float32(1.0), c.aspects[img])
@@ -32,8 +34,10 @@ func TestDrawImage_Ratio2(t *testing.T) {
 	//	win := d.CreateWindow("Test")
 	//	c := win.Canvas().(*glCanvas)
 
+	test.NewApp() // Need an app started to get safeIconLookup to work.
+
 	// make sure we haven't used the visual ratio
-	img := canvas.NewImageFromResource(theme.FyneLogo())
+	img := canvas.NewImageFromResource(theme.ComputerIcon())
 	img.Resize(fyne.NewSize(20, 10))
 	//	c.newGlImageTexture(img)
 	//	assert.Equal(t, float32(1.0), c.aspects[img])
