@@ -20,12 +20,15 @@ func shaderSourceNamed(name string) ([]byte, []byte) {
 		return shaderSimpleVert.StaticContent, shaderSimpleFrag.StaticContent
 	case "simple_es":
 		return shaderSimpleesVert.StaticContent, shaderSimpleesFrag.StaticContent
-	case "single_channel":
-		return shaderSimpleVert.StaticContent, shaderSinglechannelFrag.StaticContent
-	case "single_channel_es":
-		return shaderSimpleesVert.StaticContent, shaderSinglechannelesFrag.StaticContent
+	case "rectangle":
+		return shaderRectangleVert.StaticContent, shaderRectangleFrag.StaticContent
+	case "round_rectangle":
+		return shaderRectangleVert.StaticContent, shaderRoundrectangleFrag.StaticContent
+	case "rectangle_es":
+		return shaderRectangleesVert.StaticContent, shaderRectangleesFrag.StaticContent
+	case "round_rectangle_es":
+		return shaderRectangleesVert.StaticContent, shaderRoundrectangleesFrag.StaticContent
 	}
-
 	return nil, nil
 }
 
@@ -60,21 +63,22 @@ func NewPainter(c fyne.Canvas, ctx driver.WithContext) Painter {
 }
 
 type painter struct {
-	canvas               fyne.Canvas
-	ctx                  context
-	contextProvider      driver.WithContext
-	program              Program
-	singleChannelProgram Program
-	lineProgram          Program
-	texScale             float32
-	pixScale             float32 // pre-calculate scale*texScale for each draw
+	canvas                fyne.Canvas
+	ctx                   context
+	contextProvider       driver.WithContext
+	program               Program
+	lineProgram           Program
+	rectangleProgram      Program
+	roundRectangleProgram Program
+	texScale              float32
+	pixScale              float32 // pre-calculate scale*texScale for each draw
 }
 
 // Declare conformity to Painter interface
 var _ Painter = (*painter)(nil)
 
 func (p *painter) Clear() {
-	r, g, b, a := theme.BackgroundColor().RGBA()
+	r, g, b, a := theme.Color(theme.ColorNameBackground).RGBA()
 	p.ctx.ClearColor(float32(r)/max16bit, float32(g)/max16bit, float32(b)/max16bit, float32(a)/max16bit)
 	p.ctx.Clear(bitColorBuffer | bitDepthBuffer)
 	p.logError()
@@ -179,5 +183,5 @@ func (p *painter) createProgram(shaderFilename string) Program {
 }
 
 func (p *painter) logError() {
-	logGLError(p.ctx.GetError())
+	logGLError(p.ctx.GetError)
 }
