@@ -8,14 +8,14 @@ import (
 // Declare conformity with Layout interface
 var _ fyne.Layout = (*paddedLayout)(nil)
 
-type paddedLayout struct {
-}
+type paddedLayout struct{}
 
 // Layout is called to pack all child objects into a specified size.
 // For PaddedLayout this sets all children to the full size passed minus padding all around.
-func (l *paddedLayout) Layout(objects []fyne.CanvasObject, size fyne.Size) {
-	pos := fyne.NewPos(theme.Padding(), theme.Padding())
-	siz := fyne.NewSize(size.Width-2*theme.Padding(), size.Height-2*theme.Padding())
+func (l paddedLayout) Layout(objects []fyne.CanvasObject, size fyne.Size) {
+	padding := theme.Padding()
+	pos := fyne.NewSquareOffsetPos(padding)
+	siz := fyne.NewSize(size.Width-2*padding, size.Height-2*padding)
 	for _, child := range objects {
 		child.Resize(siz)
 		child.Move(pos)
@@ -24,7 +24,7 @@ func (l *paddedLayout) Layout(objects []fyne.CanvasObject, size fyne.Size) {
 
 // MinSize finds the smallest size that satisfies all the child objects.
 // For PaddedLayout this is determined simply as the MinSize of the largest child plus padding all around.
-func (l *paddedLayout) MinSize(objects []fyne.CanvasObject) (min fyne.Size) {
+func (l paddedLayout) MinSize(objects []fyne.CanvasObject) (min fyne.Size) {
 	for _, child := range objects {
 		if !child.Visible() {
 			continue
@@ -32,7 +32,7 @@ func (l *paddedLayout) MinSize(objects []fyne.CanvasObject) (min fyne.Size) {
 
 		min = min.Max(child.MinSize())
 	}
-	min = min.Add(fyne.NewSize(2*theme.Padding(), 2*theme.Padding()))
+	min = min.Add(fyne.NewSquareSize(2 * theme.Padding()))
 	return
 }
 
@@ -40,5 +40,5 @@ func (l *paddedLayout) MinSize(objects []fyne.CanvasObject) (min fyne.Size) {
 //
 // Since: 1.4
 func NewPaddedLayout() fyne.Layout {
-	return &paddedLayout{}
+	return paddedLayout{}
 }
