@@ -8,13 +8,19 @@ import (
 	"github.com/stretchr/testify/assert"
 
 	"fyne.io/fyne/v2"
-	"fyne.io/fyne/v2/canvas"
+	fynecanvas "fyne.io/fyne/v2/canvas"
 	"fyne.io/fyne/v2/container"
 	"fyne.io/fyne/v2/layout"
 	"fyne.io/fyne/v2/theme"
 	"fyne.io/fyne/v2/widget"
 )
 
+func Test_knownColor(t *testing.T) {
+	for _, name := range knownColorNames {
+		assert.NotEmpty(t, knownColor(theme.Color(name)), "color name %s is not known", name)
+	}
+}
+
 func Test_snapshot(t *testing.T) {
 	// content elements
 	for name, tt := range map[string]struct {
@@ -25,7 +31,7 @@ func Test_snapshot(t *testing.T) {
 		want     string
 	}{
 		"circle": {
-			content: canvas.NewCircle(color.NRGBA{R: 200, G: 100, B: 0, A: 50}),
+			content: fynecanvas.NewCircle(color.NRGBA{R: 200, G: 100, B: 0, A: 50}),
 			size:    fyne.NewSize(42, 42),
 			pos:     fyne.NewPos(17, 17),
 			want: "<canvas size=\"100x100\">\n" +
@@ -35,26 +41,18 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"circle with theme color": { // we won’t test _all_ valid values … it’s not that important
-			content: canvas.NewCircle(theme.ScrollBarColor()),
+			content: fynecanvas.NewCircle(theme.Color(theme.ColorNameScrollBar)),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
 				"\t\t<circle fillColor=\"scrollbar\" size=\"100x100\"/>\n" +
 				"\t</content>\n" +
 				"</canvas>\n",
 		},
-		"circle with named primary color": { // we won’t test _all_ valid values … it’s not that important
-			content: canvas.NewCircle(theme.PrimaryColorNamed(theme.ColorPurple)),
-			want: "<canvas size=\"100x100\">\n" +
-				"\t<content>\n" +
-				"\t\t<circle fillColor=\"purple\" size=\"100x100\"/>\n" +
-				"\t</content>\n" +
-				"</canvas>\n",
-		},
 		"circle with stroke": {
 			content: func() fyne.CanvasObject {
-				c := canvas.NewCircle(color.NRGBA{R: 200, G: 100, B: 0, A: 50})
+				c := fynecanvas.NewCircle(color.NRGBA{R: 200, G: 100, B: 0, A: 50})
 				c.StrokeWidth = 3
-				c.StrokeColor = theme.BackgroundColor()
+				c.StrokeColor = theme.Color(theme.ColorNameBackground)
 				return c
 			}(),
 			size: fyne.NewSize(42, 42),
@@ -66,7 +64,7 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"image from file": {
-			content: canvas.NewImageFromFile("testdata/markup_renderer_image.svg"),
+			content: fynecanvas.NewImageFromFile("testdata/markup_renderer_image.svg"),
 			size:    fyne.NewSize(66, 33),
 			pos:     fyne.NewPos(10, 20),
 			want: "<canvas size=\"100x100\">\n" +
@@ -76,7 +74,7 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"image from image": {
-			content: canvas.NewImageFromImage(image.NewUniform(color.Transparent)),
+			content: fynecanvas.NewImageFromImage(image.NewUniform(color.Transparent)),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
 				"\t\t<image img size=\"100x100\"/>\n" +
@@ -84,7 +82,7 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"image from resource": {
-			content: canvas.NewImageFromResource(fyne.NewStaticResource("resource name", []byte{})),
+			content: fynecanvas.NewImageFromResource(fyne.NewStaticResource("resource name", []byte{})),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
 				"\t\t<image rsc=\"resource name\" size=\"100x100\"/>\n" +
@@ -92,23 +90,23 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"image theme resource": { // we won’t test _all_ valid values … it’s not that important
-			content: canvas.NewImageFromResource(theme.VolumeDownIcon()),
+			content: fynecanvas.NewImageFromResource(theme.VolumeDownIcon()),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
-				"\t\t<image rsc=\"volumeDownIcon\" size=\"100x100\"/>\n" +
+				"\t\t<image rsc=\"volumeDownIcon\" size=\"100x100\" themed=\"foreground\"/>\n" +
 				"\t</content>\n" +
 				"</canvas>\n",
 		},
 		"image themed resource": {
-			content: canvas.NewImageFromResource(theme.NewThemedResource(fyne.NewStaticResource("resource name", []byte{}))),
+			content: fynecanvas.NewImageFromResource(theme.NewThemedResource(fyne.NewStaticResource("resource name", []byte{}))),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
-				"\t\t<image rsc=\"resource name\" size=\"100x100\" themed=\"default\"/>\n" +
+				"\t\t<image rsc=\"resource name\" size=\"100x100\" themed=\"foreground\"/>\n" +
 				"\t</content>\n" +
 				"</canvas>\n",
 		},
 		"image disabled resource": {
-			content: canvas.NewImageFromResource(theme.NewDisabledResource(theme.CancelIcon())),
+			content: fynecanvas.NewImageFromResource(theme.NewDisabledResource(theme.CancelIcon())),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
 				"\t\t<image rsc=\"cancelIcon\" size=\"100x100\" themed=\"disabled\"/>\n" +
@@ -116,7 +114,7 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"image error themed resource": {
-			content: canvas.NewImageFromResource(theme.NewErrorThemedResource(theme.ErrorIcon())),
+			content: fynecanvas.NewImageFromResource(theme.NewErrorThemedResource(theme.ErrorIcon())),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
 				"\t\t<image rsc=\"errorIcon\" size=\"100x100\" themed=\"error\"/>\n" +
@@ -124,7 +122,7 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"image inverted themed resource": {
-			content: canvas.NewImageFromResource(theme.NewInvertedThemedResource(theme.WarningIcon())),
+			content: fynecanvas.NewImageFromResource(theme.NewInvertedThemedResource(theme.WarningIcon())),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
 				"\t\t<image rsc=\"warningIcon\" size=\"100x100\" themed=\"inverted\"/>\n" +
@@ -132,7 +130,7 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"image primary themed resource": {
-			content: canvas.NewImageFromResource(theme.NewPrimaryThemedResource(theme.VolumeDownIcon())),
+			content: fynecanvas.NewImageFromResource(theme.NewPrimaryThemedResource(theme.VolumeDownIcon())),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
 				"\t\t<image rsc=\"volumeDownIcon\" size=\"100x100\" themed=\"primary\"/>\n" +
@@ -141,63 +139,63 @@ func Test_snapshot(t *testing.T) {
 		},
 		"image with translucency": {
 			content: func() fyne.CanvasObject {
-				img := canvas.NewImageFromResource(theme.ZoomOutIcon())
+				img := fynecanvas.NewImageFromResource(theme.ZoomOutIcon())
 				img.Translucency = 1.3
 				return img
 			}(),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
-				"\t\t<image rsc=\"zoomOutIcon\" size=\"100x100\" translucency=\"1.3\"/>\n" +
+				"\t\t<image rsc=\"zoomOutIcon\" size=\"100x100\" themed=\"foreground\" translucency=\"1.3\"/>\n" +
 				"\t</content>\n" +
 				"</canvas>\n",
 		},
 		"image with fill mode contain": {
 			content: func() fyne.CanvasObject {
-				img := canvas.NewImageFromResource(theme.ZoomOutIcon())
-				img.FillMode = canvas.ImageFillContain
+				img := fynecanvas.NewImageFromResource(theme.ZoomOutIcon())
+				img.FillMode = fynecanvas.ImageFillContain
 				return img
 			}(),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
-				"\t\t<image fillMode=\"contain\" rsc=\"zoomOutIcon\" size=\"100x100\"/>\n" +
+				"\t\t<image fillMode=\"contain\" rsc=\"zoomOutIcon\" size=\"100x100\" themed=\"foreground\"/>\n" +
 				"\t</content>\n" +
 				"</canvas>\n",
 		},
 		"image with fill mode original": {
 			content: func() fyne.CanvasObject {
-				img := canvas.NewImageFromResource(theme.ZoomInIcon())
-				img.FillMode = canvas.ImageFillOriginal
+				img := fynecanvas.NewImageFromResource(theme.ZoomInIcon())
+				img.FillMode = fynecanvas.ImageFillOriginal
 				return img
 			}(),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
-				"\t\t<image fillMode=\"original\" rsc=\"zoomInIcon\" size=\"100x100\"/>\n" +
+				"\t\t<image fillMode=\"original\" rsc=\"zoomInIcon\" size=\"100x100\" themed=\"foreground\"/>\n" +
 				"\t</content>\n" +
 				"</canvas>\n",
 		},
 		"image with scale mode pixels": {
 			content: func() fyne.CanvasObject {
-				img := canvas.NewImageFromResource(theme.ViewRestoreIcon())
-				img.ScaleMode = canvas.ImageScalePixels
+				img := fynecanvas.NewImageFromResource(theme.ViewRestoreIcon())
+				img.ScaleMode = fynecanvas.ImageScalePixels
 				return img
 			}(),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
-				"\t\t<image rsc=\"viewRestoreIcon\" scaleMode=\"pixels\" size=\"100x100\"/>\n" +
+				"\t\t<image rsc=\"viewRestoreIcon\" scaleMode=\"pixels\" size=\"100x100\" themed=\"foreground\"/>\n" +
 				"\t</content>\n" +
 				"</canvas>\n",
 		},
 		"image with inline icon size": {
-			content: canvas.NewImageFromResource(theme.VisibilityIcon()),
+			content: fynecanvas.NewImageFromResource(theme.VisibilityIcon()),
 			size:    fyne.NewSize(theme.IconInlineSize(), theme.IconInlineSize()),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
-				"\t\t<image rsc=\"visibilityIcon\" size=\"iconInlineSize\"/>\n" +
+				"\t\t<image rsc=\"visibilityIcon\" size=\"iconInlineSize\" themed=\"foreground\"/>\n" +
 				"\t</content>\n" +
 				"</canvas>\n",
 		},
 		"line": {
-			content: canvas.NewLine(color.NRGBA{R: 17, G: 42, B: 128, A: 255}),
+			content: fynecanvas.NewLine(color.NRGBA{R: 17, G: 42, B: 128, A: 255}),
 			pos:     fyne.NewPos(13, 13),
 			size:    fyne.NewSize(10, 50),
 			want: "<canvas size=\"100x100\">\n" +
@@ -207,24 +205,16 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"line with theme color": { // we won’t test _all_ valid values … it’s not that important
-			content: canvas.NewLine(theme.ShadowColor()),
+			content: fynecanvas.NewLine(theme.Color(theme.ColorNameShadow)),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
 				"\t\t<line size=\"100x100\" strokeColor=\"shadow\"/>\n" +
 				"\t</content>\n" +
 				"</canvas>\n",
 		},
-		"line with named primary color": { // we won’t test _all_ valid values … it’s not that important
-			content: canvas.NewLine(theme.PrimaryColorNamed(theme.ColorBrown)),
-			want: "<canvas size=\"100x100\">\n" +
-				"\t<content>\n" +
-				"\t\t<line size=\"100x100\" strokeColor=\"brown\"/>\n" +
-				"\t</content>\n" +
-				"</canvas>\n",
-		},
 		"line with stroke width": {
 			content: func() fyne.CanvasObject {
-				l := canvas.NewLine(color.NRGBA{R: 17, G: 42, B: 128, A: 255})
+				l := fynecanvas.NewLine(color.NRGBA{R: 17, G: 42, B: 128, A: 255})
 				l.StrokeWidth = 1.125
 				return l
 			}(),
@@ -237,7 +227,7 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"linear gradient": {
-			content: canvas.NewLinearGradient(color.NRGBA{R: 1, G: 2, B: 3, A: 4}, theme.DisabledColor(), 13.25),
+			content: fynecanvas.NewLinearGradient(color.NRGBA{R: 1, G: 2, B: 3, A: 4}, theme.Color(theme.ColorNameDisabled), 13.25),
 			pos:     fyne.NewPos(6, 13),
 			size:    fyne.NewSize(50, 10),
 			want: "<canvas size=\"100x100\">\n" +
@@ -247,7 +237,7 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"radial gradient": {
-			content: canvas.NewRadialGradient(color.NRGBA{R: 1, G: 2, B: 3, A: 4}, theme.DisabledColor()),
+			content: fynecanvas.NewRadialGradient(color.NRGBA{R: 1, G: 2, B: 3, A: 4}, theme.Color(theme.ColorNameDisabled)),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
 				"\t\t<radialGradient endColor=\"disabled\" size=\"100x100\" startColor=\"rgba(1,2,3,4)\"/>\n" +
@@ -256,7 +246,7 @@ func Test_snapshot(t *testing.T) {
 		},
 		"radial gradient with offset": {
 			content: func() fyne.CanvasObject {
-				g := canvas.NewRadialGradient(color.NRGBA{R: 1, G: 2, B: 3, A: 4}, theme.DisabledColor())
+				g := fynecanvas.NewRadialGradient(color.NRGBA{R: 1, G: 2, B: 3, A: 4}, theme.Color(theme.ColorNameDisabled))
 				g.CenterOffsetX = 1.5
 				g.CenterOffsetY = -13.7
 				return g
@@ -268,7 +258,7 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"raster": {
-			content: &canvas.Raster{Translucency: 1},
+			content: &fynecanvas.Raster{Translucency: 1},
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
 				"\t\t<raster size=\"100x100\" translucency=\"1\"/>\n" +
@@ -276,7 +266,7 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"rectangle": {
-			content: canvas.NewRectangle(color.NRGBA{R: 100, G: 200, B: 50, A: 0}),
+			content: fynecanvas.NewRectangle(color.NRGBA{R: 100, G: 200, B: 50, A: 0}),
 			size:    fyne.NewSize(17, 42),
 			pos:     fyne.NewPos(42, 17),
 			want: "<canvas size=\"100x100\">\n" +
@@ -286,26 +276,18 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"rectangle with theme color": { // we won’t test _all_ valid values … it’s not that important
-			content: canvas.NewRectangle(theme.HoverColor()),
+			content: fynecanvas.NewRectangle(theme.Color(theme.ColorNameHover)),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
 				"\t\t<rectangle fillColor=\"hover\" size=\"100x100\"/>\n" +
 				"\t</content>\n" +
 				"</canvas>\n",
 		},
-		"rectangle with named primary color": { // we won’t test _all_ valid values … it’s not that important
-			content: canvas.NewRectangle(theme.PrimaryColorNamed(theme.ColorOrange)),
-			want: "<canvas size=\"100x100\">\n" +
-				"\t<content>\n" +
-				"\t\t<rectangle fillColor=\"orange\" size=\"100x100\"/>\n" +
-				"\t</content>\n" +
-				"</canvas>\n",
-		},
 		"rectangle with stroke": {
 			content: func() fyne.CanvasObject {
-				r := canvas.NewRectangle(color.NRGBA{R: 200, G: 100, B: 0, A: 50})
+				r := fynecanvas.NewRectangle(color.NRGBA{R: 200, G: 100, B: 0, A: 50})
 				r.StrokeWidth = 6.375
-				r.StrokeColor = theme.PlaceHolderColor()
+				r.StrokeColor = theme.Color(theme.ColorNamePlaceHolder)
 				return r
 			}(),
 			size: fyne.NewSize(42, 42),
@@ -317,17 +299,17 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"text": {
-			content: canvas.NewText("foo", theme.PrimaryColorNamed(theme.ColorYellow)),
+			content: fynecanvas.NewText("foo", color.NRGBA{R: 123, G: 234, B: 56, A: 0}),
 			size:    fyne.NewSize(50, 50),
 			pos:     fyne.NewPos(20, 20),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
-				"\t\t<text color=\"yellow\" pos=\"20,20\" size=\"50x50\">foo</text>\n" +
+				"\t\t<text color=\"rgba(123,234,56,0)\" pos=\"20,20\" size=\"50x50\">foo</text>\n" +
 				"\t</content>\n" +
 				"</canvas>\n",
 		},
-		"text with text color": {
-			content: canvas.NewText("bar", theme.ForegroundColor()),
+		"text with theme color": {
+			content: fynecanvas.NewText("bar", theme.Color(theme.ColorNameForeground)),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
 				"\t\t<text size=\"100x100\">bar</text>\n" +
@@ -336,7 +318,7 @@ func Test_snapshot(t *testing.T) {
 		},
 		"text with alignment center": {
 			content: func() fyne.CanvasObject {
-				t := canvas.NewText("bar", theme.ForegroundColor())
+				t := fynecanvas.NewText("bar", theme.Color(theme.ColorNameForeground))
 				t.Alignment = fyne.TextAlignCenter
 				return t
 			}(),
@@ -348,7 +330,7 @@ func Test_snapshot(t *testing.T) {
 		},
 		"text with alignment trailing": {
 			content: func() fyne.CanvasObject {
-				txt := canvas.NewText("bar", theme.ForegroundColor())
+				txt := fynecanvas.NewText("bar", theme.Color(theme.ColorNameForeground))
 				txt.Alignment = fyne.TextAlignTrailing
 				return txt
 			}(),
@@ -360,7 +342,7 @@ func Test_snapshot(t *testing.T) {
 		},
 		"text with size": {
 			content: func() fyne.CanvasObject {
-				txt := canvas.NewText("big", theme.ForegroundColor())
+				txt := fynecanvas.NewText("big", theme.Color(theme.ColorNameForeground))
 				txt.TextSize = 42
 				return txt
 			}(),
@@ -372,7 +354,7 @@ func Test_snapshot(t *testing.T) {
 		},
 		"text bold": {
 			content: func() fyne.CanvasObject {
-				txt := canvas.NewText("bold", theme.ForegroundColor())
+				txt := fynecanvas.NewText("bold", theme.Color(theme.ColorNameForeground))
 				txt.TextStyle.Bold = true
 				return txt
 			}(),
@@ -384,7 +366,7 @@ func Test_snapshot(t *testing.T) {
 		},
 		"text italic": {
 			content: func() fyne.CanvasObject {
-				txt := canvas.NewText("italic", theme.ForegroundColor())
+				txt := fynecanvas.NewText("italic", theme.Color(theme.ColorNameForeground))
 				txt.TextStyle.Italic = true
 				return txt
 			}(),
@@ -396,7 +378,7 @@ func Test_snapshot(t *testing.T) {
 		},
 		"text monospace": {
 			content: func() fyne.CanvasObject {
-				txt := canvas.NewText("mono", theme.ForegroundColor())
+				txt := fynecanvas.NewText("mono", theme.Color(theme.ColorNameForeground))
 				txt.TextStyle.Monospace = true
 				return txt
 			}(),
@@ -407,7 +389,7 @@ func Test_snapshot(t *testing.T) {
 				"</canvas>\n",
 		},
 		"container": {
-			content: container.NewVBox(canvas.NewCircle(color.Black), canvas.NewLine(color.NRGBA{R: 250, G: 250, B: 250, A: 250})),
+			content: container.NewVBox(fynecanvas.NewCircle(color.Black), fynecanvas.NewLine(color.NRGBA{R: 250, G: 250, B: 250, A: 250})),
 			want: "<canvas size=\"100x100\">\n" +
 				"\t<content>\n" +
 				"\t\t<container size=\"100x100\">\n" +
@@ -429,8 +411,8 @@ func Test_snapshot(t *testing.T) {
 		"widget with subobjects": {
 			content: &markupRendererTestWidget{
 				objs: []fyne.CanvasObject{
-					canvas.NewCircle(color.Black),
-					canvas.NewLine(color.NRGBA{R: 250, G: 250, B: 250, A: 250}),
+					fynecanvas.NewCircle(color.Black),
+					fynecanvas.NewLine(color.NRGBA{R: 250, G: 250, B: 250, A: 250}),
 				},
 			},
 			want: "<canvas size=\"100x100\">\n" +
@@ -452,9 +434,9 @@ func Test_snapshot(t *testing.T) {
 		},
 		"invisible": {
 			content: func() fyne.CanvasObject {
-				c := canvas.NewCircle(color.Black)
+				c := fynecanvas.NewCircle(color.Black)
 				c.Hide()
-				l := canvas.NewLine(color.NRGBA{R: 250, G: 250, B: 250, A: 250})
+				l := fynecanvas.NewLine(color.NRGBA{R: 250, G: 250, B: 250, A: 250})
 				l.Hide()
 				w := widget.NewButton("tap me if you can", nil)
 				w.Hide()
@@ -486,7 +468,7 @@ func Test_snapshot(t *testing.T) {
 	t.Run("canvas with padding", func(t *testing.T) {
 		c := NewCanvas()
 		c.SetPadded(true)
-		c.SetContent(canvas.NewCircle(color.Black))
+		c.SetContent(fynecanvas.NewCircle(color.Black))
 		c.Resize(fyne.NewSize(100, 100))
 		assert.Equal(
 			t,
@@ -502,9 +484,9 @@ func Test_snapshot(t *testing.T) {
 	t.Run("canvas with overlays", func(t *testing.T) {
 		c := NewCanvas()
 		c.SetPadded(true)
-		c.SetContent(canvas.NewCircle(color.Black))
-		c.Overlays().Add(canvas.NewRectangle(color.NRGBA{R: 250, G: 250, B: 250, A: 250}))
-		c.Overlays().Add(canvas.NewRectangle(color.Transparent))
+		c.SetContent(fynecanvas.NewCircle(color.Black))
+		c.Overlays().Add(fynecanvas.NewRectangle(color.NRGBA{R: 250, G: 250, B: 250, A: 250}))
+		c.Overlays().Add(fynecanvas.NewRectangle(color.Transparent))
 		c.Resize(fyne.NewSize(100, 100))
 		assert.Equal(
 			t,
