@@ -1,4 +1,4 @@
-package theme
+package theme_test
 
 import (
 	"fmt"
@@ -8,6 +8,7 @@ import (
 	"github.com/stretchr/testify/assert"
 
 	"fyne.io/fyne/v2"
+	"fyne.io/fyne/v2/theme"
 )
 
 func init() {
@@ -32,38 +33,50 @@ func helperLoadRes(t *testing.T, name string) fyne.Resource {
 }
 
 func TestIconThemeChangeDoesNotChangeName(t *testing.T) {
-	fyne.CurrentApp().Settings().SetTheme(DarkTheme())
-	cancel := CancelIcon()
+	fyne.CurrentApp().Settings().SetTheme(theme.DarkTheme())
+	cancel := theme.CancelIcon()
 	name := cancel.Name()
 
-	fyne.CurrentApp().Settings().SetTheme(LightTheme())
+	fyne.CurrentApp().Settings().SetTheme(theme.LightTheme())
 	assert.Equal(t, name, cancel.Name())
 }
 
 func TestIconThemeChangeContent(t *testing.T) {
-	fyne.CurrentApp().Settings().SetTheme(DarkTheme())
-	cancel := CancelIcon()
+	fyne.CurrentApp().Settings().SetTheme(theme.DarkTheme())
+	cancel := theme.CancelIcon()
 	content := cancel.Content()
 
-	fyne.CurrentApp().Settings().SetTheme(LightTheme())
+	fyne.CurrentApp().Settings().SetTheme(theme.LightTheme())
 	assert.NotEqual(t, content, cancel.Content())
 }
 
 func TestNewThemedResource_StaticResourceSupport(t *testing.T) {
-	fyne.CurrentApp().Settings().SetTheme(DarkTheme())
-	custom := NewThemedResource(helperNewStaticResource())
+	fyne.CurrentApp().Settings().SetTheme(theme.DarkTheme())
+	custom := theme.NewThemedResource(helperNewStaticResource())
 	content := custom.Content()
 	name := custom.Name()
 
-	fyne.CurrentApp().Settings().SetTheme(LightTheme())
+	fyne.CurrentApp().Settings().SetTheme(theme.LightTheme())
 	assert.NotEqual(t, content, custom.Content())
 	assert.Equal(t, name, custom.Name())
 }
 
-func TestNewDisabledResource(t *testing.T) {
-	fyne.CurrentApp().Settings().SetTheme(DarkTheme())
+func TestNewColoredResource(t *testing.T) {
+	fyne.CurrentApp().Settings().SetTheme(theme.DarkTheme())
 	source := helperNewStaticResource()
-	custom := NewDisabledResource(source)
+	custom := theme.NewColoredResource(source, theme.ColorNameSuccess)
+
+	assert.Equal(t, theme.ColorNameSuccess, custom.ColorName)
+	assert.Equal(t, custom.Name(), fmt.Sprintf("success_%v", source.Name()))
+
+	custom = theme.NewColoredResource(source, theme.ColorNamePrimary)
+	assert.Equal(t, custom.Name(), fmt.Sprintf("primary_%v", source.Name()))
+}
+
+func TestNewDisabledResource(t *testing.T) {
+	fyne.CurrentApp().Settings().SetTheme(theme.DarkTheme())
+	source := helperNewStaticResource()
+	custom := theme.NewDisabledResource(source)
 	name := custom.Name()
 
 	assert.Equal(t, name, fmt.Sprintf("disabled_%v", source.Name()))
@@ -71,363 +84,366 @@ func TestNewDisabledResource(t *testing.T) {
 
 func TestThemedResource_Invert(t *testing.T) {
 	staticResource := helperLoadRes(t, "cancel_Paths.svg")
-	inverted := NewInvertedThemedResource(staticResource)
+	inverted := theme.NewInvertedThemedResource(staticResource)
 	assert.Equal(t, "inverted_"+staticResource.Name(), inverted.Name())
 }
 
 func TestThemedResource_Name(t *testing.T) {
 	staticResource := helperLoadRes(t, "cancel_Paths.svg")
-	themedResource := &ThemedResource{
-		source: staticResource,
-	}
+	themedResource := theme.NewThemedResource(staticResource)
 	assert.Equal(t, "foreground_"+staticResource.Name(), themedResource.Name())
 }
 
 func TestThemedResource_Content_NoGroupsFile(t *testing.T) {
-	fyne.CurrentApp().Settings().SetTheme(DarkTheme())
+	fyne.CurrentApp().Settings().SetTheme(theme.DarkTheme())
 	staticResource := helperLoadRes(t, "cancel_Paths.svg")
-	themedResource := &ThemedResource{
-		source: staticResource,
-	}
+	themedResource := theme.NewThemedResource(staticResource)
 	assert.NotEqual(t, staticResource.Content(), themedResource.Content())
 }
 
 func TestThemedResource_Content_GroupPathFile(t *testing.T) {
-	fyne.CurrentApp().Settings().SetTheme(DarkTheme())
+	fyne.CurrentApp().Settings().SetTheme(theme.DarkTheme())
 	staticResource := helperLoadRes(t, "check_GroupPaths.svg")
-	themedResource := &ThemedResource{
-		source: staticResource,
-	}
+	themedResource := theme.NewThemedResource(staticResource)
 	assert.NotEqual(t, staticResource.Content(), themedResource.Content())
 }
 
 func TestThemedResource_Content_GroupRectFile(t *testing.T) {
-	fyne.CurrentApp().Settings().SetTheme(DarkTheme())
+	fyne.CurrentApp().Settings().SetTheme(theme.DarkTheme())
 	staticResource := helperLoadRes(t, "info_GroupRects.svg")
-	themedResource := &ThemedResource{
-		source: staticResource,
-	}
+	themedResource := theme.NewThemedResource(staticResource)
 	assert.NotEqual(t, staticResource.Content(), themedResource.Content())
 }
 
 func TestThemedResource_Content_GroupPolygonsFile(t *testing.T) {
-	fyne.CurrentApp().Settings().SetTheme(DarkTheme())
+	fyne.CurrentApp().Settings().SetTheme(theme.DarkTheme())
 	staticResource := helperLoadRes(t, "warning_GroupPolygons.svg")
-	themedResource := &ThemedResource{
-		source: staticResource,
-	}
+	themedResource := theme.NewThemedResource(staticResource)
 	assert.NotEqual(t, staticResource.Content(), themedResource.Content())
 }
 
-// a black svg object omits the fill tag, this checks it it still properly updated
+// a black svg object omits the fill tag, this checks if it still properly updated
 func TestThemedResource_Content_BlackFillIsUpdated(t *testing.T) {
-	fyne.CurrentApp().Settings().SetTheme(DarkTheme())
+	fyne.CurrentApp().Settings().SetTheme(theme.DarkTheme())
 	staticResource := helperLoadRes(t, "cancel_PathsBlackFill.svg")
-	themedResource := &ThemedResource{
-		source: staticResource,
-	}
+	themedResource := theme.NewThemedResource(staticResource)
 	assert.NotEqual(t, staticResource.Content(), themedResource.Content())
 }
 
+func TestThemedResource_Error(t *testing.T) {
+	fyne.CurrentApp().Settings().SetTheme(theme.DarkTheme())
+	source := helperNewStaticResource()
+	custom := theme.NewThemedResource(source)
+	custom.ColorName = theme.ColorNameError
+
+	assert.Equal(t, custom.Name(), fmt.Sprintf("error_%v", source.Name()))
+	custom2 := theme.NewErrorThemedResource(source)
+	assert.Equal(t, custom2.Name(), fmt.Sprintf("error_%v", source.Name()))
+}
+
 func TestThemedResource_Success(t *testing.T) {
-	fyne.CurrentApp().Settings().SetTheme(DarkTheme())
+	fyne.CurrentApp().Settings().SetTheme(theme.DarkTheme())
 	source := helperNewStaticResource()
-	custom := NewThemedResource(source)
-	custom.ColorName = ColorNameSuccess
-	name := custom.Name()
+	custom := theme.NewThemedResource(source)
+	custom.ColorName = theme.ColorNameSuccess
 
-	assert.Equal(t, name, fmt.Sprintf("success_%v", source.Name()))
+	assert.Equal(t, custom.Name(), fmt.Sprintf("success_%v", source.Name()))
+
+	custom = theme.NewSuccessThemedResource(source)
+	assert.Equal(t, custom.Name(), fmt.Sprintf("success_%v", source.Name()))
 }
 
 func TestThemedResource_Warning(t *testing.T) {
-	fyne.CurrentApp().Settings().SetTheme(DarkTheme())
+	fyne.CurrentApp().Settings().SetTheme(theme.DarkTheme())
 	source := helperNewStaticResource()
-	custom := NewThemedResource(source)
-	custom.ColorName = ColorNameWarning
-	name := custom.Name()
+	custom := theme.NewThemedResource(source)
+	custom.ColorName = theme.ColorNameWarning
 
-	assert.Equal(t, name, fmt.Sprintf("warning_%v", source.Name()))
+	assert.Equal(t, custom.Name(), fmt.Sprintf("warning_%v", source.Name()))
+	custom = theme.NewWarningThemedResource(source)
+	assert.Equal(t, custom.Name(), fmt.Sprintf("warning_%v", source.Name()))
 }
 
 func TestDisabledResource_Name(t *testing.T) {
 	staticResource := helperLoadRes(t, "cancel_Paths.svg")
-	disabledResource := &DisabledResource{
-		source: staticResource,
-	}
+	disabledResource := theme.NewDisabledResource(staticResource)
 	assert.Equal(t, fmt.Sprintf("disabled_%v", staticResource.Name()), disabledResource.Name())
 }
 
 func TestDisabledResource_Content_NoGroupsFile(t *testing.T) {
-	fyne.CurrentApp().Settings().SetTheme(DarkTheme())
+	fyne.CurrentApp().Settings().SetTheme(theme.DarkTheme())
 	staticResource := helperLoadRes(t, "cancel_Paths.svg")
-	disabledResource := &DisabledResource{
-		source: staticResource,
-	}
+	disabledResource := theme.NewDisabledResource(staticResource)
 	assert.NotEqual(t, staticResource.Content(), disabledResource.Content())
 }
 
 // Test Asset Sources
 func Test_FyneLogo_FileSource(t *testing.T) {
-	result := FyneLogo().Name()
+	result := theme.FyneLogo().Name()
 	assert.Equal(t, "fyne.png", result)
 }
 
 func Test_CancelIcon_FileSource(t *testing.T) {
-	result := CancelIcon().Name()
+	result := theme.CancelIcon().Name()
 	assert.Equal(t, "foreground_cancel.svg", result)
 }
 
 func Test_ConfirmIcon_FileSource(t *testing.T) {
-	result := ConfirmIcon().Name()
+	result := theme.ConfirmIcon().Name()
 	assert.Equal(t, "foreground_check.svg", result)
 }
 
 func Test_DeleteIcon_FileSource(t *testing.T) {
-	result := DeleteIcon().Name()
+	result := theme.DeleteIcon().Name()
 	assert.Equal(t, "foreground_delete.svg", result)
 }
 
 func Test_SearchIcon_FileSource(t *testing.T) {
-	result := SearchIcon().Name()
+	result := theme.SearchIcon().Name()
 	assert.Equal(t, "foreground_search.svg", result)
 }
 
 func Test_SearchReplaceIcon_FileSource(t *testing.T) {
-	result := SearchReplaceIcon().Name()
+	result := theme.SearchReplaceIcon().Name()
 	assert.Equal(t, "foreground_search-replace.svg", result)
 }
 
 func Test_CheckButtonIcon_FileSource(t *testing.T) {
-	result := CheckButtonIcon().Name()
+	result := theme.CheckButtonIcon().Name()
 	assert.Equal(t, "foreground_check-box.svg", result)
 }
 
 func Test_CheckButtonCheckedIcon_FileSource(t *testing.T) {
-	result := CheckButtonCheckedIcon().Name()
+	result := theme.CheckButtonCheckedIcon().Name()
 	assert.Equal(t, "foreground_check-box-checked.svg", result)
 }
 
 func Test_RadioButtonIcon_FileSource(t *testing.T) {
-	result := RadioButtonIcon().Name()
+	result := theme.RadioButtonIcon().Name()
 	assert.Equal(t, "foreground_radio-button.svg", result)
 }
 
 func Test_RadioButtonCheckedIcon_FileSource(t *testing.T) {
-	result := RadioButtonCheckedIcon().Name()
+	result := theme.RadioButtonCheckedIcon().Name()
 	assert.Equal(t, "foreground_radio-button-checked.svg", result)
 }
 
 func Test_ContentAddIcon_FileSource(t *testing.T) {
-	result := ContentAddIcon().Name()
+	result := theme.ContentAddIcon().Name()
 	assert.Equal(t, "foreground_content-add.svg", result)
 }
 
 func Test_ContentRemoveIcon_FileSource(t *testing.T) {
-	result := ContentRemoveIcon().Name()
+	result := theme.ContentRemoveIcon().Name()
 	assert.Equal(t, "foreground_content-remove.svg", result)
 }
 
 func Test_ContentClearIcon_FileSource(t *testing.T) {
-	result := ContentClearIcon().Name()
+	result := theme.ContentClearIcon().Name()
 	assert.Equal(t, "foreground_cancel.svg", result)
 }
 
 func Test_ContentCutIcon_FileSource(t *testing.T) {
-	result := ContentCutIcon().Name()
+	result := theme.ContentCutIcon().Name()
 	assert.Equal(t, "foreground_content-cut.svg", result)
 }
 
 func Test_ContentCopyIcon_FileSource(t *testing.T) {
-	result := ContentCopyIcon().Name()
+	result := theme.ContentCopyIcon().Name()
 	assert.Equal(t, "foreground_content-copy.svg", result)
 }
 
 func Test_ContentPasteIcon_FileSource(t *testing.T) {
-	result := ContentPasteIcon().Name()
+	result := theme.ContentPasteIcon().Name()
 	assert.Equal(t, "foreground_content-paste.svg", result)
 }
 
 func Test_ContentRedoIcon_FileSource(t *testing.T) {
-	result := ContentRedoIcon().Name()
+	result := theme.ContentRedoIcon().Name()
 	assert.Equal(t, "foreground_content-redo.svg", result)
 }
 
 func Test_ContentUndoIcon_FileSource(t *testing.T) {
-	result := ContentUndoIcon().Name()
+	result := theme.ContentUndoIcon().Name()
 	assert.Equal(t, "foreground_content-undo.svg", result)
 }
 
 func Test_DocumentCreateIcon_FileSource(t *testing.T) {
-	result := DocumentCreateIcon().Name()
+	result := theme.DocumentCreateIcon().Name()
 	assert.Equal(t, "foreground_document-create.svg", result)
 }
 
 func Test_DocumentPrintIcon_FileSource(t *testing.T) {
-	result := DocumentPrintIcon().Name()
+	result := theme.DocumentPrintIcon().Name()
 	assert.Equal(t, "foreground_document-print.svg", result)
 }
 
 func Test_DocumentSaveIcon_FileSource(t *testing.T) {
-	result := DocumentSaveIcon().Name()
+	result := theme.DocumentSaveIcon().Name()
 	assert.Equal(t, "foreground_document-save.svg", result)
 }
 
 func Test_InfoIcon_FileSource(t *testing.T) {
-	result := InfoIcon().Name()
+	result := theme.InfoIcon().Name()
 	assert.Equal(t, "foreground_info.svg", result)
 }
 
 func Test_QuestionIcon_FileSource(t *testing.T) {
-	result := QuestionIcon().Name()
+	result := theme.QuestionIcon().Name()
 	assert.Equal(t, "foreground_question.svg", result)
 }
 
 func Test_WarningIcon_FileSource(t *testing.T) {
-	result := WarningIcon().Name()
+	result := theme.WarningIcon().Name()
 	assert.Equal(t, "foreground_warning.svg", result)
 }
 
+func Test_BrokenImageIcon_FileSource(t *testing.T) {
+	result := theme.BrokenImageIcon().Name()
+	assert.Equal(t, "foreground_broken-image.svg", result)
+}
+
 func Test_FolderIcon_FileSource(t *testing.T) {
-	result := FolderIcon().Name()
+	result := theme.FolderIcon().Name()
 	assert.Equal(t, "foreground_folder.svg", result)
 }
 
 func Test_FolderNewIcon_FileSource(t *testing.T) {
-	result := FolderNewIcon().Name()
+	result := theme.FolderNewIcon().Name()
 	assert.Equal(t, "foreground_folder-new.svg", result)
 }
 
 func Test_FolderOpenIcon_FileSource(t *testing.T) {
-	result := FolderOpenIcon().Name()
+	result := theme.FolderOpenIcon().Name()
 	assert.Equal(t, "foreground_folder-open.svg", result)
 }
 
 func Test_HelpIcon_FileSource(t *testing.T) {
-	result := HelpIcon().Name()
+	result := theme.HelpIcon().Name()
 	assert.Equal(t, "foreground_help.svg", result)
 }
 
 func Test_HomeIcon_FileSource(t *testing.T) {
-	result := HomeIcon().Name()
+	result := theme.HomeIcon().Name()
 	assert.Equal(t, "foreground_home.svg", result)
 }
 
 func Test_SettingsIcon_FileSource(t *testing.T) {
-	result := SettingsIcon().Name()
+	result := theme.SettingsIcon().Name()
 	assert.Equal(t, "foreground_settings.svg", result)
 }
 
 func Test_MailAttachmentIcon_FileSource(t *testing.T) {
-	result := MailAttachmentIcon().Name()
+	result := theme.MailAttachmentIcon().Name()
 	assert.Equal(t, "foreground_mail-attachment.svg", result)
 }
 
 func Test_MailComposeIcon_FileSource(t *testing.T) {
-	result := MailComposeIcon().Name()
+	result := theme.MailComposeIcon().Name()
 	assert.Equal(t, "foreground_mail-compose.svg", result)
 }
 
 func Test_MailForwardIcon_FileSource(t *testing.T) {
-	result := MailForwardIcon().Name()
+	result := theme.MailForwardIcon().Name()
 	assert.Equal(t, "foreground_mail-forward.svg", result)
 }
 
 func Test_MailReplyIcon_FileSource(t *testing.T) {
-	result := MailReplyIcon().Name()
+	result := theme.MailReplyIcon().Name()
 	assert.Equal(t, "foreground_mail-reply.svg", result)
 }
 
 func Test_MailReplyAllIcon_FileSource(t *testing.T) {
-	result := MailReplyAllIcon().Name()
+	result := theme.MailReplyAllIcon().Name()
 	assert.Equal(t, "foreground_mail-reply_all.svg", result)
 }
 
 func Test_MailSendIcon_FileSource(t *testing.T) {
-	result := MailSendIcon().Name()
+	result := theme.MailSendIcon().Name()
 	assert.Equal(t, "foreground_mail-send.svg", result)
 }
 
 func Test_MoveDownIcon_FileSource(t *testing.T) {
-	result := MoveDownIcon().Name()
+	result := theme.MoveDownIcon().Name()
 	assert.Equal(t, "foreground_arrow-down.svg", result)
 }
 
 func Test_MoveUpIcon_FileSource(t *testing.T) {
-	result := MoveUpIcon().Name()
+	result := theme.MoveUpIcon().Name()
 	assert.Equal(t, "foreground_arrow-up.svg", result)
 }
 
 func Test_NavigateBackIcon_FileSource(t *testing.T) {
-	result := NavigateBackIcon().Name()
+	result := theme.NavigateBackIcon().Name()
 	assert.Equal(t, "foreground_arrow-back.svg", result)
 }
 
 func Test_NavigateNextIcon_FileSource(t *testing.T) {
-	result := NavigateNextIcon().Name()
+	result := theme.NavigateNextIcon().Name()
 	assert.Equal(t, "foreground_arrow-forward.svg", result)
 }
 
 func Test_ViewFullScreenIcon_FileSource(t *testing.T) {
-	result := ViewFullScreenIcon().Name()
+	result := theme.ViewFullScreenIcon().Name()
 	assert.Equal(t, "foreground_view-fullscreen.svg", result)
 }
 
 func Test_ViewRestoreIcon_FileSource(t *testing.T) {
-	result := ViewRestoreIcon().Name()
+	result := theme.ViewRestoreIcon().Name()
 	assert.Equal(t, "foreground_view-zoom-fit.svg", result)
 }
 
 func Test_ViewRefreshIcon_FileSource(t *testing.T) {
-	result := ViewRefreshIcon().Name()
+	result := theme.ViewRefreshIcon().Name()
 	assert.Equal(t, "foreground_view-refresh.svg", result)
 }
 
 func Test_ZoomFitIcon_FileSource(t *testing.T) {
-	result := ZoomFitIcon().Name()
+	result := theme.ZoomFitIcon().Name()
 	assert.Equal(t, "foreground_view-zoom-fit.svg", result)
 }
 
 func Test_ZoomInIcon_FileSource(t *testing.T) {
-	result := ZoomInIcon().Name()
+	result := theme.ZoomInIcon().Name()
 	assert.Equal(t, "foreground_view-zoom-in.svg", result)
 }
 
 func Test_ZoomOutIcon_FileSource(t *testing.T) {
-	result := ZoomOutIcon().Name()
+	result := theme.ZoomOutIcon().Name()
 	assert.Equal(t, "foreground_view-zoom-out.svg", result)
 }
 
 func Test_VisibilityIcon_FileSource(t *testing.T) {
-	result := VisibilityIcon().Name()
+	result := theme.VisibilityIcon().Name()
 	assert.Equal(t, "foreground_visibility.svg", result)
 }
 
 func Test_VisibilityOffIcon_FileSource(t *testing.T) {
-	result := VisibilityOffIcon().Name()
+	result := theme.VisibilityOffIcon().Name()
 	assert.Equal(t, "foreground_visibility-off.svg", result)
 }
 
 func Test_AccountIcon_FileSource(t *testing.T) {
-	result := AccountIcon().Name()
+	result := theme.AccountIcon().Name()
 	assert.Equal(t, "foreground_account.svg", result)
 }
 
 func Test_LoginIcon_FileSource(t *testing.T) {
-	result := LoginIcon().Name()
+	result := theme.LoginIcon().Name()
 	assert.Equal(t, "foreground_login.svg", result)
 }
 
 func Test_LogoutIcon_FileSource(t *testing.T) {
-	result := LogoutIcon().Name()
+	result := theme.LogoutIcon().Name()
 	assert.Equal(t, "foreground_logout.svg", result)
 }
 
 func Test_ListIcon_FileSource(t *testing.T) {
-	result := ListIcon().Name()
+	result := theme.ListIcon().Name()
 	assert.Equal(t, "foreground_list.svg", result)
 }
 
 func Test_GridIcon_FileSource(t *testing.T) {
-	result := GridIcon().Name()
+	result := theme.GridIcon().Name()
 	assert.Equal(t, "foreground_grid.svg", result)
 }
