@@ -7,6 +7,7 @@ import (
 	"fyne.io/fyne/v2/canvas"
 	"fyne.io/fyne/v2/driver/desktop"
 	col "fyne.io/fyne/v2/internal/color"
+	"fyne.io/fyne/v2/internal/svg"
 	"fyne.io/fyne/v2/internal/widget"
 	"fyne.io/fyne/v2/layout"
 	"fyne.io/fyne/v2/theme"
@@ -21,7 +22,9 @@ type ButtonIconPlacement int
 // ButtonImportance represents how prominent the button should appear
 //
 // Since: 1.4
-type ButtonImportance int
+//
+// Deprecated: Use widget.Importance instead
+type ButtonImportance = Importance
 
 // ButtonStyle determines the behaviour and rendering of a button.
 type ButtonStyle int
@@ -42,24 +45,6 @@ const (
 	ButtonIconTrailingText
 )
 
-const (
-	// MediumImportance applies a standard appearance.
-	MediumImportance ButtonImportance = iota
-	// HighImportance applies a prominent appearance.
-	HighImportance
-	// LowImportance applies a subtle appearance.
-	LowImportance
-
-	// DangerImportance applies an error theme to the button.
-	//
-	// Since 2.3
-	DangerImportance
-	// WarningImportance applies an error theme to the button.
-	//
-	// Since 2.3
-	WarningImportance
-)
-
 var _ fyne.Focusable = (*Button)(nil)
 
 // Button widget has a text label and triggers an event func when clicked
@@ -70,7 +55,7 @@ type Button struct {
 	// Specify how prominent the button should be, High will highlight the button and Low will remove some decoration.
 	//
 	// Since: 1.4
-	Importance    ButtonImportance
+	Importance    Importance
 	Alignment     ButtonAlign
 	IconPlacement ButtonIconPlacement
 
@@ -78,7 +63,6 @@ type Button struct {
 
 	hovered, focused bool
 	tapAnim          *fyne.Animation
-	background       *canvas.Rectangle
 }
 
 // NewButton creates a new button widget with the set label and tap handler
@@ -107,23 +91,30 @@ func NewButtonWithIcon(label string, icon fyne.Resource, tapped func()) *Button
 // CreateRenderer is a private method to Fyne which links this widget to its renderer
 func (b *Button) CreateRenderer() fyne.WidgetRenderer {
 	b.ExtendBaseWidget(b)
+	th := b.Theme()
+	v := fyne.CurrentApp().Settings().ThemeVariant()
+
+	b.propertyLock.RLock()
+	defer b.propertyLock.RUnlock()
+
 	seg := &TextSegment{Text: b.Text, Style: RichTextStyleStrong}
 	seg.Style.Alignment = fyne.TextAlignCenter
 	text := NewRichText(seg)
-	text.inset = fyne.NewSize(theme.InnerPadding(), theme.InnerPadding())
+	text.inset = fyne.NewSquareSize(th.Size(theme.SizeNameInnerPadding))
 
-	b.background = canvas.NewRectangle(theme.ButtonColor())
+	background := canvas.NewRectangle(th.Color(theme.ColorNameButton, v))
+	background.CornerRadius = th.Size(theme.SizeNameInputRadius)
 	tapBG := canvas.NewRectangle(color.Transparent)
-	b.tapAnim = newButtonTapAnimation(tapBG, b)
+	b.tapAnim = newButtonTapAnimation(tapBG, b, th)
 	b.tapAnim.Curve = fyne.AnimationEaseOut
 	objects := []fyne.CanvasObject{
-		b.background,
+		background,
 		tapBG,
 		text,
 	}
 	r := &buttonRenderer{
 		BaseRenderer: widget.NewBaseRenderer(objects),
-		background:   b.background,
+		background:   background,
 		tapBG:        tapBG,
 		button:       b,
 		label:        text,
@@ -160,8 +151,7 @@ func (b *Button) MinSize() fyne.Size {
 // MouseIn is called when a desktop pointer enters the widget
 func (b *Button) MouseIn(*desktop.MouseEvent) {
 	b.hovered = true
-
-	b.applyButtonTheme()
+	b.Refresh()
 }
 
 // MouseMoved is called when a desktop pointer hovers over the widget
@@ -171,20 +161,23 @@ func (b *Button) MouseMoved(*desktop.MouseEvent) {
 // MouseOut is called when a desktop pointer exits the widget
 func (b *Button) MouseOut() {
 	b.hovered = false
-
-	b.applyButtonTheme()
+	b.Refresh()
 }
 
 // SetIcon updates the icon on a label - pass nil to hide an icon
 func (b *Button) SetIcon(icon fyne.Resource) {
+	b.propertyLock.Lock()
 	b.Icon = icon
+	b.propertyLock.Unlock()
 
 	b.Refresh()
 }
 
 // SetText allows the button label to be changed
 func (b *Button) SetText(text string) {
+	b.propertyLock.Lock()
 	b.Text = text
+	b.propertyLock.Unlock()
 
 	b.Refresh()
 }
@@ -196,10 +189,10 @@ func (b *Button) Tapped(*fyne.PointEvent) {
 	}
 
 	b.tapAnimation()
-	b.applyButtonTheme()
+	b.Refresh()
 
-	if b.OnTapped != nil {
-		b.OnTapped()
+	if onTapped := b.OnTapped; onTapped != nil {
+		onTapped()
 	}
 }
 
@@ -214,63 +207,15 @@ func (b *Button) TypedKey(ev *fyne.KeyEvent) {
 	}
 }
 
-func (b *Button) applyButtonTheme() {
-	if b.background == nil {
-		return
-	}
-
-	b.background.FillColor = b.buttonColor()
-	b.background.Refresh()
-}
-
-func (b *Button) buttonColor() color.Color {
-	switch {
-	case b.Disabled():
-		if b.Importance == LowImportance {
-			return color.Transparent
-		}
-		return theme.DisabledButtonColor()
-	case b.focused:
-		bg := theme.ButtonColor()
-		if b.Importance == HighImportance {
-			bg = theme.PrimaryColor()
-		} else if b.Importance == DangerImportance {
-			bg = theme.ErrorColor()
-		} else if b.Importance == WarningImportance {
-			bg = theme.WarningColor()
-		}
-
-		return blendColor(bg, theme.FocusColor())
-	case b.hovered:
-		bg := theme.ButtonColor()
-		if b.Importance == HighImportance {
-			bg = theme.PrimaryColor()
-		} else if b.Importance == DangerImportance {
-			bg = theme.ErrorColor()
-		} else if b.Importance == WarningImportance {
-			bg = theme.WarningColor()
-		}
-
-		return blendColor(bg, theme.HoverColor())
-	case b.Importance == HighImportance:
-		return theme.PrimaryColor()
-	case b.Importance == LowImportance:
-		return color.Transparent
-	case b.Importance == DangerImportance:
-		return theme.ErrorColor()
-	case b.Importance == WarningImportance:
-		return theme.WarningColor()
-	default:
-		return theme.ButtonColor()
-	}
-}
-
 func (b *Button) tapAnimation() {
 	if b.tapAnim == nil {
 		return
 	}
 	b.tapAnim.Stop()
-	b.tapAnim.Start()
+
+	if fyne.CurrentApp().Settings().ShowAnimations() {
+		b.tapAnim.Start()
+	}
 }
 
 type buttonRenderer struct {
@@ -287,16 +232,22 @@ type buttonRenderer struct {
 // Layout the components of the button widget
 func (r *buttonRenderer) Layout(size fyne.Size) {
 	r.background.Resize(size)
+	r.tapBG.Resize(size)
 
+	th := r.button.Theme()
+	padding := r.padding(th)
 	hasIcon := r.icon != nil
 	hasLabel := r.label.Segments[0].(*TextSegment).Text != ""
 	if !hasIcon && !hasLabel {
 		// Nothing to layout
 		return
 	}
-	iconSize := fyne.NewSize(theme.IconInlineSize(), theme.IconInlineSize())
+	iconSize := fyne.NewSquareSize(th.Size(theme.SizeNameInlineIcon))
 	labelSize := r.label.MinSize()
-	padding := r.padding()
+
+	r.button.propertyLock.RLock()
+	defer r.button.propertyLock.RUnlock()
+
 	if hasLabel {
 		if hasIcon {
 			// Both
@@ -329,81 +280,131 @@ func (r *buttonRenderer) Layout(size fyne.Size) {
 // This is based on the contained text, any icon that is set and a standard
 // amount of padding added.
 func (r *buttonRenderer) MinSize() (size fyne.Size) {
+	th := r.button.Theme()
 	hasIcon := r.icon != nil
 	hasLabel := r.label.Segments[0].(*TextSegment).Text != ""
-	iconSize := fyne.NewSize(theme.IconInlineSize(), theme.IconInlineSize())
+	iconSize := fyne.NewSquareSize(th.Size(theme.SizeNameInlineIcon))
 	labelSize := r.label.MinSize()
 	if hasLabel {
 		size.Width = labelSize.Width
 	}
 	if hasIcon {
 		if hasLabel {
-			size.Width += theme.Padding()
+			size.Width += th.Size(theme.SizeNamePadding)
 		}
 		size.Width += iconSize.Width
 	}
 	size.Height = fyne.Max(labelSize.Height, iconSize.Height)
-	size = size.Add(r.padding())
+	size = size.Add(r.padding(th))
 	return
 }
 
 func (r *buttonRenderer) Refresh() {
-	r.label.inset = fyne.NewSize(theme.InnerPadding(), theme.InnerPadding())
+	th := r.button.Theme()
+	r.label.inset = fyne.NewSquareSize(th.Size(theme.SizeNameInnerPadding))
+
+	r.button.propertyLock.RLock()
 	r.label.Segments[0].(*TextSegment).Text = r.button.Text
 	r.updateIconAndText()
 	r.applyTheme()
+	r.button.propertyLock.RUnlock()
+
 	r.background.Refresh()
 	r.Layout(r.button.Size())
 	canvas.Refresh(r.button.super())
 }
 
 // applyTheme updates this button to match the current theme
+// must be called with the button propertyLock RLocked
 func (r *buttonRenderer) applyTheme() {
-	r.button.applyButtonTheme()
-	r.label.Segments[0].(*TextSegment).Style.ColorName = theme.ColorNameForeground
-	switch {
-	case r.button.disabled:
-		r.label.Segments[0].(*TextSegment).Style.ColorName = theme.ColorNameDisabled
-	case r.button.Importance == HighImportance || r.button.Importance == DangerImportance || r.button.Importance == WarningImportance:
-		if r.button.focused {
-			r.label.Segments[0].(*TextSegment).Style.ColorName = theme.ColorNameForeground
-		} else {
-			r.label.Segments[0].(*TextSegment).Style.ColorName = theme.ColorNameBackground
+	th := r.button.themeWithLock()
+	v := fyne.CurrentApp().Settings().ThemeVariant()
+	fgColorName, bgColorName, bgBlendName := r.buttonColorNames()
+	if bg := r.background; bg != nil {
+		bgColor := color.Color(color.Transparent)
+		if bgColorName != "" {
+			bgColor = th.Color(bgColorName, v)
 		}
+		if bgBlendName != "" {
+			bgColor = blendColor(bgColor, th.Color(bgBlendName, v))
+		}
+		bg.FillColor = bgColor
+		bg.CornerRadius = th.Size(theme.SizeNameInputRadius)
+		bg.Refresh()
 	}
+
+	r.label.Segments[0].(*TextSegment).Style.ColorName = fgColorName
 	r.label.Refresh()
 	if r.icon != nil && r.icon.Resource != nil {
-		switch res := r.icon.Resource.(type) {
-		case *theme.ThemedResource:
-			if r.button.Importance == HighImportance || r.button.Importance == DangerImportance || r.button.Importance == WarningImportance {
-				r.icon.Resource = theme.NewInvertedThemedResource(res)
-				r.icon.Refresh()
-			}
-		case *theme.InvertedThemedResource:
-			if r.button.Importance != HighImportance && r.button.Importance != DangerImportance && r.button.Importance != WarningImportance {
-				r.icon.Resource = res.Original()
-				r.icon.Refresh()
+		icon := r.icon.Resource
+		if r.button.Importance != MediumImportance {
+			if thRes, ok := icon.(fyne.ThemedResource); ok {
+				if thRes.ThemeColorName() != fgColorName {
+					icon = theme.NewColoredResource(icon, fgColorName)
+				}
 			}
 		}
+		r.icon.Resource = icon
+		r.icon.Refresh()
 	}
 }
 
-func (r *buttonRenderer) padding() fyne.Size {
-	return fyne.NewSize(theme.InnerPadding()*2, theme.InnerPadding()*2)
+func (r *buttonRenderer) buttonColorNames() (foreground, background, backgroundBlend fyne.ThemeColorName) {
+	foreground = theme.ColorNameForeground
+	b := r.button
+	if b.Disabled() {
+		foreground = theme.ColorNameDisabled
+		if b.Importance != LowImportance {
+			background = theme.ColorNameDisabledButton
+		}
+	} else if b.focused {
+		backgroundBlend = theme.ColorNameFocus
+	} else if b.hovered {
+		backgroundBlend = theme.ColorNameHover
+	}
+	if background == "" {
+		switch b.Importance {
+		case DangerImportance:
+			foreground = theme.ColorNameForegroundOnError
+			background = theme.ColorNameError
+		case HighImportance:
+			foreground = theme.ColorNameForegroundOnPrimary
+			background = theme.ColorNamePrimary
+		case LowImportance:
+			if backgroundBlend != "" {
+				background = theme.ColorNameButton
+			}
+		case SuccessImportance:
+			foreground = theme.ColorNameForegroundOnSuccess
+			background = theme.ColorNameSuccess
+		case WarningImportance:
+			foreground = theme.ColorNameForegroundOnWarning
+			background = theme.ColorNameWarning
+		default:
+			background = theme.ColorNameButton
+		}
+	}
+	return
 }
 
+func (r *buttonRenderer) padding(th fyne.Theme) fyne.Size {
+	return fyne.NewSquareSize(th.Size(theme.SizeNameInnerPadding) * 2)
+}
+
+// must be called with r.button.propertyLock RLocked
 func (r *buttonRenderer) updateIconAndText() {
 	if r.button.Icon != nil && r.button.Visible() {
+		icon := r.button.Icon
 		if r.icon == nil {
-			r.icon = canvas.NewImageFromResource(r.button.Icon)
+			r.icon = canvas.NewImageFromResource(icon)
 			r.icon.FillMode = canvas.ImageFillContain
 			r.SetObjects([]fyne.CanvasObject{r.background, r.tapBG, r.label, r.icon})
 		}
-		if r.button.Disabled() {
-			r.icon.Resource = theme.NewDisabledResource(r.button.Icon)
-		} else {
-			r.icon.Resource = r.button.Icon
+		// TODO support disabling bitmap resource not just SVG
+		if r.button.Disabled() && svg.IsResourceSVG(icon) {
+			icon = theme.NewDisabledResource(icon)
 		}
+		r.icon.Resource = icon
 		r.icon.Refresh()
 		r.icon.Show()
 	} else if r.icon != nil {
@@ -447,17 +448,22 @@ func blendColor(under, over color.Color) color.Color {
 
 }
 
-func newButtonTapAnimation(bg *canvas.Rectangle, w fyne.Widget) *fyne.Animation {
+func newButtonTapAnimation(bg *canvas.Rectangle, w fyne.Widget, th fyne.Theme) *fyne.Animation {
+	v := fyne.CurrentApp().Settings().ThemeVariant()
 	return fyne.NewAnimation(canvas.DurationStandard, func(done float32) {
 		mid := w.Size().Width / 2
 		size := mid * done
 		bg.Resize(fyne.NewSize(size*2, w.Size().Height))
 		bg.Move(fyne.NewPos(mid-size, 0))
 
-		r, g, bb, a := col.ToNRGBA(theme.PressedColor())
+		r, g, bb, a := col.ToNRGBA(th.Color(theme.ColorNamePressed, v))
 		aa := uint8(a)
 		fade := aa - uint8(float32(aa)*done)
-		bg.FillColor = &color.NRGBA{R: uint8(r), G: uint8(g), B: uint8(bb), A: fade}
+		if fade > 0 {
+			bg.FillColor = &color.NRGBA{R: uint8(r), G: uint8(g), B: uint8(bb), A: fade}
+		} else {
+			bg.FillColor = color.Transparent
+		}
 		canvas.Refresh(bg)
 	})
 }
