@@ -9,6 +9,7 @@ import (
 	"fyne.io/fyne/v2/canvas"
 	"fyne.io/fyne/v2/driver/desktop"
 	"fyne.io/fyne/v2/internal/cache"
+	intWidget "fyne.io/fyne/v2/internal/widget"
 	"fyne.io/fyne/v2/test"
 	"fyne.io/fyne/v2/theme"
 )
@@ -31,6 +32,7 @@ func TestEntry_Cursor(t *testing.T) {
 func TestEntry_DoubleTapped(t *testing.T) {
 	entry := NewEntry()
 	entry.Wrapping = fyne.TextWrapOff
+	entry.Scroll = intWidget.ScrollNone
 	entry.SetText("The quick brown fox\njumped    over the lazy dog\n")
 	entry.Resize(entry.MinSize())
 
@@ -40,12 +42,16 @@ func TestEntry_DoubleTapped(t *testing.T) {
 	entry.DoubleTapped(ev)
 	assert.Equal(t, "quick", entry.SelectedText())
 
+	entry.doubleTappedAtUnixMillis = 0 // make sure we don't register a triple tap next
+
 	// select the whitespace after 'quick'
 	ev = getClickPosition("The quick", 0)
 	clickPrimary(entry, ev)
 	entry.DoubleTapped(ev)
 	assert.Equal(t, " ", entry.SelectedText())
 
+	entry.doubleTappedAtUnixMillis = 0
+
 	// select all whitespace after 'jumped'
 	ev = getClickPosition("jumped  ", 1)
 	clickPrimary(entry, ev)
@@ -80,6 +86,7 @@ func TestEntry_DoubleTapped_AfterCol(t *testing.T) {
 func TestEntry_DragSelect(t *testing.T) {
 	entry := NewEntry()
 	entry.Wrapping = fyne.TextWrapOff
+	entry.Scroll = intWidget.ScrollNone
 	entry.SetText("The quick brown fox jumped\nover the lazy dog\nThe quick\nbrown fox\njumped over the lazy dog\n")
 	entry.Resize(entry.MinSize())
 
@@ -104,6 +111,7 @@ func TestEntry_DragSelect(t *testing.T) {
 func TestEntry_DragSelectLargeStep(t *testing.T) {
 	entry := NewEntry()
 	entry.Wrapping = fyne.TextWrapOff
+	entry.Scroll = intWidget.ScrollNone
 	entry.SetText("The quick brown fox jumped\nover the lazy dog\nThe quick\nbrown fox\njumped over the lazy dog\n")
 	entry.Resize(entry.MinSize())
 
@@ -193,46 +201,50 @@ func TestEntry_ExpandSelectionForDoubleTap(t *testing.T) {
 	str := []rune(" fish 日本語日  \t  test 本日本 moose  \t")
 
 	// select invalid (before start)
-	start, end := getTextWhitespaceRegion(str, -1)
+	start, end := getTextWhitespaceRegion(str, -1, false)
 	assert.Equal(t, -1, start)
 	assert.Equal(t, -1, end)
 
 	// select whitespace at the end of text
-	start, end = getTextWhitespaceRegion(str, len(str))
+	start, end = getTextWhitespaceRegion(str, len(str), false)
 	assert.Equal(t, 29, start)
 	assert.Equal(t, 32, end)
-	start, end = getTextWhitespaceRegion(str, len(str)+100)
+	start, end = getTextWhitespaceRegion(str, len(str)+100, false)
 	assert.Equal(t, 29, start)
 	assert.Equal(t, 32, end)
 
 	// select the whitespace
-	start, end = getTextWhitespaceRegion(str, 0)
+	start, end = getTextWhitespaceRegion(str, 0, false)
 	assert.Equal(t, 0, start)
 	assert.Equal(t, 1, end)
+	// select the whitespace - grab adjacent words
+	start, end = getTextWhitespaceRegion(str, 0, true)
+	assert.Equal(t, 0, start)
+	assert.Equal(t, 5, end)
 
 	// select "fish"
-	start, end = getTextWhitespaceRegion(str, 1)
+	start, end = getTextWhitespaceRegion(str, 1, false)
 	assert.Equal(t, 1, start)
 	assert.Equal(t, 5, end)
-	start, end = getTextWhitespaceRegion(str, 4)
+	start, end = getTextWhitespaceRegion(str, 4, false)
 	assert.Equal(t, 1, start)
 	assert.Equal(t, 5, end)
 
 	// select "日本語日"
-	start, end = getTextWhitespaceRegion(str, 6)
+	start, end = getTextWhitespaceRegion(str, 7, false)
 	assert.Equal(t, 6, start)
 	assert.Equal(t, 10, end)
-	start, end = getTextWhitespaceRegion(str, 9)
+	start, end = getTextWhitespaceRegion(str, 9, false)
 	assert.Equal(t, 6, start)
 	assert.Equal(t, 10, end)
 
 	// select "  \t  "
-	start, end = getTextWhitespaceRegion(str, 10)
+	start, end = getTextWhitespaceRegion(str, 10, false)
 	assert.Equal(t, 10, start)
 	assert.Equal(t, 15, end)
 
 	// select "  \t"
-	start, end = getTextWhitespaceRegion(str, 30)
+	start, end = getTextWhitespaceRegion(str, 30, false)
 	assert.Equal(t, 29, start)
 	assert.Equal(t, len(str), end)
 }
@@ -240,7 +252,7 @@ func TestEntry_ExpandSelectionForDoubleTap(t *testing.T) {
 func TestEntry_ExpandSelectionWithWordSeparators(t *testing.T) {
 	// select "is_a"
 	str := []rune("This-is_a-test")
-	start, end := getTextWhitespaceRegion(str, 6)
+	start, end := getTextWhitespaceRegion(str, 6, false)
 	assert.Equal(t, 5, start)
 	assert.Equal(t, 9, end)
 }
@@ -267,14 +279,31 @@ func TestEntry_EraseSelection(t *testing.T) {
 	keyPress(&fyne.KeyEvent{Name: fyne.KeyRight})
 	keyPress(&fyne.KeyEvent{Name: fyne.KeyRight})
 
-	e.eraseSelection()
-	e.updateText(e.textProvider().String())
+	_ = e.Theme()
+	e.eraseSelectionAndUpdate()
+	e.updateText(e.textProvider().String(), false)
 	assert.Equal(t, "Testing\nTeng\nTesting", e.Text)
 	a, b := e.selection()
 	assert.Equal(t, -1, a)
 	assert.Equal(t, -1, b)
 }
 
+func TestEntry_CallbackLocking(t *testing.T) {
+	e := &Entry{}
+	called := 0
+	e.OnChanged = func(_ string) {
+		e.propertyLock.Lock()
+		called++ // Just to not have an empty critical section.
+		e.propertyLock.Unlock()
+	}
+
+	_ = e.Theme()
+	test.Type(e, "abc123")
+	e.selectAll()
+	e.TypedKey(&fyne.KeyEvent{Name: fyne.KeyBackspace})
+	assert.Equal(t, 7, called)
+}
+
 func TestEntry_MouseClickAndDragOutsideText(t *testing.T) {
 	entry := NewEntry()
 	entry.SetText("A\nB\n")
@@ -317,6 +346,7 @@ func TestEntry_PasteFromClipboard(t *testing.T) {
 	entry := NewEntry()
 
 	w := test.NewApp().NewWindow("")
+	defer w.Close()
 	w.SetContent(entry)
 
 	testContent := "test"
@@ -334,6 +364,7 @@ func TestEntry_PasteFromClipboard_MultilineWrapping(t *testing.T) {
 	entry.Wrapping = fyne.TextWrapWord
 
 	w := test.NewApp().NewWindow("")
+	defer w.Close()
 	w.SetContent(entry)
 	w.Resize(fyne.NewSize(108, 64))
 
@@ -358,17 +389,29 @@ func TestEntry_PasteFromClipboard_MultilineWrapping(t *testing.T) {
 	assert.Equal(t, 7, entry.CursorColumn)
 }
 
+func TestEntry_PlaceholderTextStyle(t *testing.T) {
+	e := NewEntry()
+	e.TextStyle = fyne.TextStyle{Bold: true, Italic: true}
+
+	w := test.NewTempWindow(t, e)
+	assert.Equal(t, e.TextStyle, e.placeholder.Segments[0].(*TextSegment).Style.TextStyle)
+
+	w.Canvas().Focus(e)
+	assert.Equal(t, e.TextStyle, e.placeholder.Segments[0].(*TextSegment).Style.TextStyle)
+}
+
 func TestEntry_Tab(t *testing.T) {
 	e := NewEntry()
 	e.TextStyle.Monospace = true
 	e.SetText("a\n\tb\nc")
 
+	_ = e.Theme()
 	r := cache.Renderer(e.textProvider()).(*textRenderer)
 	assert.Equal(t, 3, len(r.Objects()))
 	assert.Equal(t, "a", r.Objects()[0].(*canvas.Text).Text)
 	assert.Equal(t, "\tb", r.Objects()[1].(*canvas.Text).Text)
 
-	w := test.NewWindow(e)
+	w := test.NewTempWindow(t, e)
 	w.Resize(fyne.NewSize(86, 86))
 	w.Canvas().Focus(e)
 	test.AssertImageMatches(t, "entry/tab-content.png", w.Canvas().Capture())
@@ -387,7 +430,7 @@ func TestEntry_TabSelection(t *testing.T) {
 
 	assert.Equal(t, "\tb", e.SelectedText())
 
-	w := test.NewWindow(e)
+	w := test.NewTempWindow(t, e)
 	w.Resize(fyne.NewSize(86, 86))
 	w.Canvas().Focus(e)
 	test.AssertImageMatches(t, "entry/tab-select.png", w.Canvas().Capture())
