@@ -19,8 +19,9 @@ type passwordRevealer struct {
 }
 
 func newPasswordRevealer(e *Entry) *passwordRevealer {
+	th := e.Theme()
 	pr := &passwordRevealer{
-		icon:  canvas.NewImageFromResource(theme.VisibilityOffIcon()),
+		icon:  canvas.NewImageFromResource(th.Icon(theme.IconNameVisibilityOff)),
 		entry: e,
 	}
 	pr.ExtendBaseWidget(pr)
@@ -59,24 +60,27 @@ type passwordRevealerRenderer struct {
 }
 
 func (r *passwordRevealerRenderer) Layout(size fyne.Size) {
-	r.icon.Resize(fyne.NewSize(theme.IconInlineSize(), theme.IconInlineSize()))
-	r.icon.Move(fyne.NewPos((size.Width-theme.IconInlineSize())/2, (size.Height-theme.IconInlineSize())/2))
+	iconSize := r.entry.Theme().Size(theme.SizeNameInlineIcon)
+	r.icon.Resize(fyne.NewSquareSize(iconSize))
+	r.icon.Move(fyne.NewPos((size.Width-iconSize)/2, (size.Height-iconSize)/2))
 }
 
 func (r *passwordRevealerRenderer) MinSize() fyne.Size {
-	return fyne.NewSize(theme.IconInlineSize(), theme.IconInlineSize())
+	iconSize := r.entry.Theme().Size(theme.SizeNameInlineIcon)
+	return fyne.NewSquareSize(iconSize)
 }
 
 func (r *passwordRevealerRenderer) Refresh() {
+	th := r.entry.Theme()
 	r.entry.propertyLock.RLock()
 	defer r.entry.propertyLock.RUnlock()
 	if !r.entry.Password {
-		r.icon.Resource = theme.VisibilityIcon()
+		r.icon.Resource = th.Icon(theme.IconNameVisibility)
 	} else {
-		r.icon.Resource = theme.VisibilityOffIcon()
+		r.icon.Resource = th.Icon(theme.IconNameVisibilityOff)
 	}
 
-	if r.entry.disabled {
+	if r.entry.disabled.Load() {
 		r.icon.Resource = theme.NewDisabledResource(r.icon.Resource)
 	}
 	canvas.Refresh(r.icon)
