@@ -2,11 +2,13 @@ package widget_test
 
 import (
 	"image/color"
+	"runtime"
 	"testing"
 	"time"
 
 	"fyne.io/fyne/v2"
 	"fyne.io/fyne/v2/canvas"
+	"fyne.io/fyne/v2/container"
 	"fyne.io/fyne/v2/data/binding"
 	"fyne.io/fyne/v2/driver/desktop"
 	"fyne.io/fyne/v2/internal/driver"
@@ -21,6 +23,7 @@ func TestEntry_Binding(t *testing.T) {
 	entry := widget.NewEntry()
 	entry.SetText("Init")
 	assert.Equal(t, "Init", entry.Text)
+	waitForBinding() // this time it is the de-echo before binding
 
 	str := binding.NewString()
 	entry.Bind(str)
@@ -42,6 +45,21 @@ func TestEntry_Binding(t *testing.T) {
 	assert.Equal(t, "Typed", entry.Text)
 }
 
+func TestEntry_Binding_Bounce(t *testing.T) {
+	entry := widget.NewEntry()
+	entry.SetText("Init")
+	assert.Equal(t, "Init", entry.Text)
+	waitForBinding() // this time it is the de-echo before binding
+
+	str := binding.NewString()
+	entry.Bind(str)
+	str.Set("1")
+	time.Sleep(10 * time.Millisecond)
+	str.Set("2")
+	waitForBinding()
+	assert.Equal(t, "2", entry.Text)
+}
+
 func TestEntry_Binding_Replace(t *testing.T) {
 	entry := widget.NewEntry()
 	str := binding.NewString()
@@ -54,12 +72,12 @@ func TestEntry_Binding_Replace(t *testing.T) {
 	assert.Equal(t, "nt", entry.SelectedText())
 
 	test.Type(entry, "g")
+	waitForBinding()
 	assert.Equal(t, "Cogent", entry.Text)
 }
 
 func TestEntry_Clicked(t *testing.T) {
 	entry, window := setupImageTest(t, true)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	entry.SetText("MMM\nWWW\n")
@@ -156,6 +174,85 @@ func TestEntry_CursorColumn_Jump(t *testing.T) {
 	assert.Equal(t, 1, entry.CursorColumn)
 }
 
+func TestEntry_Control_Word(t *testing.T) {
+	entry := widget.NewMultiLineEntry()
+	entry.SetText("a\nbc")
+	entry.CursorRow = 0
+	entry.CursorColumn = 0
+
+	moveWordModifier := fyne.KeyModifierShortcutDefault
+	if runtime.GOOS == "darwin" {
+		moveWordModifier = fyne.KeyModifierAlt
+	}
+
+	// ctrl-right to move on
+	nextWord := &desktop.CustomShortcut{KeyName: fyne.KeyRight, Modifier: moveWordModifier}
+	entry.TypedShortcut(nextWord)
+	assert.Equal(t, 0, entry.CursorRow)
+	assert.Equal(t, 1, entry.CursorColumn)
+	entry.TypedShortcut(nextWord)
+	assert.Equal(t, 1, entry.CursorRow)
+	assert.Equal(t, 0, entry.CursorColumn)
+	entry.TypedShortcut(nextWord)
+	assert.Equal(t, 1, entry.CursorRow)
+	assert.Equal(t, 2, entry.CursorColumn)
+
+	// ctrl-left to move back
+	prevWord := &desktop.CustomShortcut{KeyName: fyne.KeyLeft, Modifier: moveWordModifier}
+	entry.TypedShortcut(prevWord)
+	assert.Equal(t, 1, entry.CursorRow)
+	assert.Equal(t, 0, entry.CursorColumn)
+	entry.TypedShortcut(prevWord)
+	assert.Equal(t, 0, entry.CursorRow)
+	assert.Equal(t, 1, entry.CursorColumn)
+
+	// select word
+	entry.SetText("word1 word2 word3")
+	entry.CursorRow = 0
+	entry.CursorColumn = 3
+	selectNextWord := &desktop.CustomShortcut{KeyName: fyne.KeyRight, Modifier: moveWordModifier | fyne.KeyModifierShift}
+	entry.TypedShortcut(selectNextWord)
+	assert.Equal(t, "d1", entry.SelectedText())
+	entry.TypedShortcut(selectNextWord)
+	assert.Equal(t, "d1 word2", entry.SelectedText())
+
+	// unselect when no shift press
+	entry.TypedShortcut(nextWord)
+	assert.Equal(t, "", entry.SelectedText())
+}
+
+func TestEntry_Control_DeleteWord(t *testing.T) {
+	entry := widget.NewMultiLineEntry()
+	entry.SetText("Hello world\nhere is a second line")
+	entry.CursorRow = 1
+	entry.CursorColumn = 10 // right before "second"
+	modifier := fyne.KeyModifierControl
+	if runtime.GOOS == "darwin" {
+		modifier = fyne.KeyModifierAlt
+	}
+	// Ctrl+delete - delete word to right ("second")
+	entry.TypedShortcut(&desktop.CustomShortcut{Modifier: modifier, KeyName: fyne.KeyDelete})
+	assert.Equal(t, "Hello world\nhere is a  line", entry.Text)
+	assert.Equal(t, 10, entry.CursorColumn)
+
+	entry.CursorColumn = 8 // right before "a"
+	// Ctrl+backspace - delete word to left ("is")
+	entry.TypedShortcut(&desktop.CustomShortcut{Modifier: modifier, KeyName: fyne.KeyBackspace})
+	assert.Equal(t, "Hello world\nhere a  line", entry.Text)
+	assert.Equal(t, 5, entry.CursorColumn)
+
+	// does nothing when nothing left to delete
+	entry.SetText("")
+	entry.TypedShortcut(&desktop.CustomShortcut{Modifier: modifier, KeyName: fyne.KeyBackspace})
+	assert.Equal(t, "", entry.Text)
+
+	// doesn't crash when trying to delete backward with one space
+	entry.SetText(" ")
+	entry.CursorRow = 0
+	entry.CursorColumn = 1
+	entry.TypedShortcut(&desktop.CustomShortcut{Modifier: modifier, KeyName: fyne.KeyBackspace})
+}
+
 func TestEntry_CursorColumn_Wrap(t *testing.T) {
 	entry := widget.NewMultiLineEntry()
 	entry.SetText("a\nb")
@@ -186,7 +283,7 @@ func TestEntry_CursorColumn_Wrap2(t *testing.T) {
 	entry.SetText("1234")
 	entry.CursorColumn = 3
 
-	w := test.NewWindow(entry)
+	w := test.NewTempWindow(t, entry)
 	w.Resize(fyne.NewSize(72, 64))
 
 	test.Type(entry, "a")
@@ -236,7 +333,6 @@ func TestEntry_CursorRow(t *testing.T) {
 
 func TestEntry_Disableable(t *testing.T) {
 	entry, window := setupImageTest(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	assert.False(t, entry.Disabled())
@@ -277,7 +373,6 @@ func TestEntry_Disableable(t *testing.T) {
 
 func TestEntry_Disabled_TextSelection(t *testing.T) {
 	entry, window := setupImageTest(t, false)
-	defer teardownImageTest(window)
 	entry.SetText("Testing")
 	entry.Disable()
 	c := window.Canvas()
@@ -320,7 +415,6 @@ func TestEntry_EmptySelection(t *testing.T) {
 
 func TestEntry_Focus(t *testing.T) {
 	entry, window := setupImageTest(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	entry.FocusGained()
@@ -335,7 +429,6 @@ func TestEntry_Focus(t *testing.T) {
 
 func TestEntry_FocusWithPopUp(t *testing.T) {
 	entry, window := setupImageTest(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	test.TapSecondaryAt(entry, fyne.NewPos(1, 1))
@@ -382,12 +475,14 @@ func TestEntry_MinSize(t *testing.T) {
 	assert.True(t, min.Height > theme.InnerPadding())
 
 	entry.Wrapping = fyne.TextWrapOff
+	entry.Scroll = container.ScrollNone
 	entry.Refresh()
 	assert.Greater(t, entry.MinSize().Width, min.Width)
 
 	min = entry.MinSize()
 	entry.ActionItem = canvas.NewCircle(color.Black)
-	assert.Equal(t, min.Add(fyne.NewSize(theme.IconInlineSize()+theme.Padding(), 0)), entry.MinSize())
+	entry.Refresh()
+	assert.Equal(t, min.Add(fyne.NewSize(theme.IconInlineSize()+theme.LineSpacing(), 0)), entry.MinSize())
 }
 
 func TestEntryMultiline_MinSize(t *testing.T) {
@@ -401,6 +496,7 @@ func TestEntryMultiline_MinSize(t *testing.T) {
 	assert.True(t, min.Height > theme.InnerPadding())
 
 	entry.Wrapping = fyne.TextWrapOff
+	entry.Scroll = container.ScrollNone
 	entry.Refresh()
 	assert.Greater(t, entry.MinSize().Width, min.Width)
 
@@ -410,6 +506,7 @@ func TestEntryMultiline_MinSize(t *testing.T) {
 
 	min = entry.MinSize()
 	entry.ActionItem = canvas.NewCircle(color.Black)
+	entry.Refresh()
 	assert.Equal(t, min.Add(fyne.NewSize(theme.IconInlineSize()+theme.Padding(), 0)), entry.MinSize())
 }
 
@@ -428,7 +525,6 @@ func TestEntryMultiline_SetMinRowsVisible(t *testing.T) {
 
 func TestEntry_MultilineSelect(t *testing.T) {
 	e, window := setupSelection(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	// Extend the selection down one row
@@ -483,6 +579,15 @@ func TestEntry_Notify(t *testing.T) {
 	entry.SetText("Test")
 
 	assert.True(t, changed)
+
+	changed = false
+	entry.TypedKey(&fyne.KeyEvent{Name: fyne.KeyDelete})
+	assert.True(t, changed)
+
+	changed = false
+	entry.CursorColumn = 1
+	entry.TypedKey(&fyne.KeyEvent{Name: fyne.KeyBackspace})
+	assert.True(t, changed)
 }
 
 func TestEntry_OnCopy(t *testing.T) {
@@ -695,7 +800,7 @@ func TestEntry_OnKeyDown_DeleteNewline(t *testing.T) {
 }
 
 func TestEntry_OnKeyDown_HomeEnd(t *testing.T) {
-	entry := &widget.Entry{}
+	entry := widget.NewEntry()
 	entry.SetText("Hi")
 	assert.Equal(t, 0, entry.CursorRow)
 	assert.Equal(t, 0, entry.CursorColumn)
@@ -709,6 +814,18 @@ func TestEntry_OnKeyDown_HomeEnd(t *testing.T) {
 	entry.TypedKey(home)
 	assert.Equal(t, 0, entry.CursorRow)
 	assert.Equal(t, 0, entry.CursorColumn)
+
+	if runtime.GOOS == "darwin" {
+		endShortcut := &desktop.CustomShortcut{KeyName: fyne.KeyRight, Modifier: fyne.KeyModifierSuper}
+		entry.TypedShortcut(endShortcut)
+		assert.Equal(t, 0, entry.CursorRow)
+		assert.Equal(t, 2, entry.CursorColumn)
+
+		homeShortcut := &desktop.CustomShortcut{KeyName: fyne.KeyLeft, Modifier: fyne.KeyModifierSuper}
+		entry.TypedShortcut(homeShortcut)
+		assert.Equal(t, 0, entry.CursorRow)
+		assert.Equal(t, 0, entry.CursorColumn)
+	}
 }
 
 func TestEntry_OnKeyDown_Insert(t *testing.T) {
@@ -726,7 +843,6 @@ func TestEntry_OnKeyDown_Insert(t *testing.T) {
 
 func TestEntry_OnKeyDown_Newline(t *testing.T) {
 	entry, window := setupImageTest(t, true)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	entry.SetText("Hi")
@@ -833,7 +949,6 @@ func TestEntry_OnPaste(t *testing.T) {
 func TestEntry_PageUpDown(t *testing.T) {
 	t.Run("single line", func(*testing.T) {
 		e, window := setupImageTest(t, false)
-		defer teardownImageTest(window)
 		c := window.Canvas()
 
 		c.Focus(e)
@@ -864,7 +979,6 @@ func TestEntry_PageUpDown(t *testing.T) {
 
 	t.Run("page down single line", func(*testing.T) {
 		e, window := setupImageTest(t, true)
-		defer teardownImageTest(window)
 		c := window.Canvas()
 
 		c.Focus(e)
@@ -926,12 +1040,12 @@ func TestEntry_PasteUnicode(t *testing.T) {
 }
 
 func TestEntry_Placeholder(t *testing.T) {
-	entry := &widget.Entry{}
+	entry := &widget.Entry{Scroll: container.ScrollNone}
 	entry.Text = "Text"
 	entry.PlaceHolder = "Placehold"
 
-	window := test.NewWindow(entry)
-	defer teardownImageTest(window)
+	window := test.NewTempWindow(t, entry)
+	defer test.NewApp()
 	c := window.Canvas()
 
 	assert.Equal(t, "Text", entry.Text)
@@ -1084,7 +1198,6 @@ func TestEntry_Select(t *testing.T) {
 	} {
 		t.Run(name, func(t *testing.T) {
 			entry, window := setupSelection(t, tt.setupReverse)
-			defer teardownImageTest(window)
 			c := window.Canvas()
 
 			if tt.text != "" {
@@ -1101,7 +1214,6 @@ func TestEntry_Select(t *testing.T) {
 
 func TestEntry_SelectAll(t *testing.T) {
 	e, window := setupImageTest(t, true)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	c.Focus(e)
@@ -1124,7 +1236,6 @@ func TestEntry_SelectAll_EmptyEntry(t *testing.T) {
 
 func TestEntry_SelectEndWithoutShift(t *testing.T) {
 	e, window := setupSelection(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	// end after releasing shift
@@ -1135,7 +1246,6 @@ func TestEntry_SelectEndWithoutShift(t *testing.T) {
 
 func TestEntry_SelectHomeEnd(t *testing.T) {
 	e, window := setupSelection(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	// Hold shift to continue selection
@@ -1154,7 +1264,6 @@ func TestEntry_SelectHomeEnd(t *testing.T) {
 
 func TestEntry_SelectHomeWithoutShift(t *testing.T) {
 	e, window := setupSelection(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	// home after releasing shift
@@ -1166,7 +1275,6 @@ func TestEntry_SelectHomeWithoutShift(t *testing.T) {
 func TestEntry_SelectSnapDown(t *testing.T) {
 	// down snaps to end, but it also moves
 	e, window := setupSelection(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	assert.Equal(t, 1, e.CursorRow)
@@ -1181,7 +1289,6 @@ func TestEntry_SelectSnapDown(t *testing.T) {
 
 func TestEntry_SelectSnapLeft(t *testing.T) {
 	e, window := setupSelection(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	assert.Equal(t, 1, e.CursorRow)
@@ -1196,7 +1303,6 @@ func TestEntry_SelectSnapLeft(t *testing.T) {
 
 func TestEntry_SelectSnapRight(t *testing.T) {
 	e, window := setupSelection(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	assert.Equal(t, 1, e.CursorRow)
@@ -1212,7 +1318,6 @@ func TestEntry_SelectSnapRight(t *testing.T) {
 func TestEntry_SelectSnapUp(t *testing.T) {
 	// up snaps to start, but it also moves
 	e, window := setupSelection(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	assert.Equal(t, 1, e.CursorRow)
@@ -1225,9 +1330,19 @@ func TestEntry_SelectSnapUp(t *testing.T) {
 	assert.Equal(t, "", e.SelectedText())
 }
 
+func TestEntry_Select_TripleTap(t *testing.T) {
+	e, _ := setupSelection(t, false)
+	e.MultiLine = true
+	assert.Equal(t, 1, e.CursorRow)
+	assert.Equal(t, "sti", e.SelectedText())
+	test.DoubleTap(e)
+	time.Sleep(50 * time.Millisecond)
+	e.MouseDown(&desktop.MouseEvent{PointEvent: fyne.PointEvent{Position: fyne.NewPos(1, 1)}})
+	assert.Equal(t, "Testing", e.SelectedText())
+}
+
 func TestEntry_SelectedText(t *testing.T) {
 	e, window := setupImageTest(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	c.Focus(e)
@@ -1260,7 +1375,6 @@ func TestEntry_SelectedText(t *testing.T) {
 
 func TestEntry_SelectionHides(t *testing.T) {
 	e, window := setupSelection(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	c.Unfocus()
@@ -1274,7 +1388,6 @@ func TestEntry_SelectionHides(t *testing.T) {
 
 func TestEntry_SetPlaceHolder(t *testing.T) {
 	entry, window := setupImageTest(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	assert.Equal(t, 0, len(entry.Text))
@@ -1290,7 +1403,6 @@ func TestEntry_SetPlaceHolder(t *testing.T) {
 
 func TestEntry_SetPlaceHolder_ByField(t *testing.T) {
 	entry, window := setupImageTest(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	assert.Equal(t, 0, len(entry.Text))
@@ -1320,7 +1432,6 @@ func TestEntry_Disable_KeyDown(t *testing.T) {
 
 func TestEntry_Disable_OnFocus(t *testing.T) {
 	entry, window := setupImageTest(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	entry.Disable()
@@ -1357,7 +1468,6 @@ func TestEntry_SetText_EmptyString(t *testing.T) {
 
 func TestEntry_SetText_Manual(t *testing.T) {
 	entry, window := setupImageTest(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	entry.Text = "Test"
@@ -1423,7 +1533,6 @@ func TestEntry_SetText_Overflow_Multiline(t *testing.T) {
 
 func TestEntry_SetTextStyle(t *testing.T) {
 	entry, window := setupImageTest(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	entry.Text = "Styled Text"
@@ -1440,6 +1549,22 @@ func TestEntry_SetTextStyle(t *testing.T) {
 	test.AssertRendersToMarkup(t, "entry/set_text_style_italic.xml", c)
 }
 
+func TestEntry_Append(t *testing.T) {
+	entry := widget.NewEntry()
+
+	entry.Append("abc")
+	assert.Equal(t, entry.Text, "abc")
+	entry.Append(" def")
+	assert.Equal(t, entry.Text, "abc def")
+
+	entry.SetText("")
+	entry.MultiLine = true
+
+	entry.Append("first line")
+	entry.Append("\nsecond line")
+	assert.Equal(t, entry.Text, "first line\nsecond line")
+}
+
 func TestEntry_Submit(t *testing.T) {
 	t.Run("Callback", func(t *testing.T) {
 		var submission string
@@ -1547,7 +1672,6 @@ func TestTabable(t *testing.T) {
 
 func TestEntry_TappedSecondary(t *testing.T) {
 	entry, window := setupImageTest(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	tapPos := fyne.NewPos(20, 10)
@@ -1577,34 +1701,42 @@ func TestEntry_TappedSecondary(t *testing.T) {
 func TestEntry_TextWrap(t *testing.T) {
 	for name, tt := range map[string]struct {
 		multiLine bool
+		scroll    container.ScrollDirection
 		want      string
 		wrap      fyne.TextWrap
 	}{
 		"single line WrapOff": {
-			want: "entry/wrap_single_line_off.xml",
+			scroll: container.ScrollNone,
+			want:   "entry/wrap_single_line_off.xml",
 		},
 		"single line Truncate": {
-			wrap: fyne.TextTruncate,
+			wrap: fyne.TextWrap(fyne.TextTruncateClip),
 			want: "entry/wrap_single_line_truncate.xml",
 		},
-		// Disallowed - fallback to TextWrapTruncate (horizontal)
+		"single line Scroll": {
+			scroll: container.ScrollHorizontalOnly,
+			wrap:   fyne.TextWrapOff,
+			want:   "entry/wrap_single_line_truncate.xml",
+		},
+		// Disallowed - fallback to Scrollling (horizontal)
 		"single line WrapBreak": {
 			wrap: fyne.TextWrapBreak,
 			want: "entry/wrap_single_line_truncate.xml",
 		},
-		// Disallowed - fallback to TextWrapTruncate (horizontal)
+		// Disallowed - fallback to Scrolling (horizontal)
 		"single line WrapWord": {
 			wrap: fyne.TextWrapWord,
 			want: "entry/wrap_single_line_truncate.xml",
 		},
 		"multi line WrapOff": {
+			scroll:    container.ScrollNone,
 			multiLine: true,
 			want:      "entry/wrap_multi_line_off.xml",
 		},
 		// Disallowed - fallback to TextWrapOff
 		"multi line Truncate": {
 			multiLine: true,
-			wrap:      fyne.TextTruncate,
+			wrap:      fyne.TextWrap(fyne.TextTruncateClip),
 			want:      "entry/wrap_multi_line_truncate.xml",
 		},
 		"multi line WrapBreak": {
@@ -1620,11 +1752,11 @@ func TestEntry_TextWrap(t *testing.T) {
 	} {
 		t.Run(name, func(t *testing.T) {
 			e, window := setupImageTest(t, tt.multiLine)
-			defer teardownImageTest(window)
 			c := window.Canvas()
 
 			c.Focus(e)
 			e.Wrapping = tt.wrap
+			e.Scroll = tt.scroll
 			if tt.multiLine {
 				e.SetText("A long text on short words w/o NLs or LFs.")
 			} else {
@@ -1637,15 +1769,15 @@ func TestEntry_TextWrap(t *testing.T) {
 
 func TestEntry_TextWrap_Changed(t *testing.T) {
 	e, window := setupImageTest(t, false)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	c.Focus(e)
 	e.Wrapping = fyne.TextWrapOff
+	e.Scroll = container.ScrollNone
 	e.SetText("Testing Wrapping")
 	test.AssertRendersToMarkup(t, "entry/wrap_single_line_off.xml", c)
 
-	e.Wrapping = fyne.TextTruncate
+	e.Wrapping = fyne.TextWrap(fyne.TextTruncateClip)
 	e.Refresh()
 	test.AssertRendersToMarkup(t, "entry/wrap_single_line_truncate.xml", c)
 
@@ -1665,6 +1797,7 @@ func TestMultiLineEntry_MinSize(t *testing.T) {
 	assert.True(t, multiMin.Height > singleMin.Height)
 
 	multi.MultiLine = false
+	multi.Refresh()
 	multiMin = multi.MinSize()
 	assert.Equal(t, singleMin.Height, multiMin.Height)
 }
@@ -1689,14 +1822,13 @@ func TestPasswordEntry_ActionItemSizeAndPlacement(t *testing.T) {
 	b := widget.NewButton("", func() {})
 	b.Icon = theme.CancelIcon()
 	e.ActionItem = b
-	test.WidgetRenderer(e).Layout(e.MinSize())
+	test.TempWidgetRenderer(t, e).Layout(e.MinSize())
 	assert.Equal(t, fyne.NewSize(theme.IconInlineSize(), theme.IconInlineSize()), b.Size())
 	assert.Equal(t, fyne.NewPos(e.MinSize().Width-2*theme.Padding()-b.Size().Width, 2*theme.Padding()), b.Position())
 }
 
 func TestPasswordEntry_Disabled(t *testing.T) {
-	entry, window := setupPasswordTest(t)
-	defer teardownImageTest(window)
+	entry, _ := setupPasswordTest(t)
 	entry.Disable()
 
 	test.Tap(entry.ActionItem.(fyne.Tappable))
@@ -1716,7 +1848,6 @@ func TestPasswordEntry_NewlineIgnored(t *testing.T) {
 
 func TestPasswordEntry_Obfuscation(t *testing.T) {
 	entry, window := setupPasswordTest(t)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
 	test.Type(entry, "Hié™שרה")
@@ -1726,9 +1857,10 @@ func TestPasswordEntry_Obfuscation(t *testing.T) {
 
 func TestPasswordEntry_Placeholder(t *testing.T) {
 	entry, window := setupPasswordTest(t)
-	defer teardownImageTest(window)
 	c := window.Canvas()
 
+	test.AssertRendersToMarkup(t, "password_entry/initial.xml", window.Canvas())
+
 	entry.SetPlaceHolder("Password")
 	test.AssertRendersToMarkup(t, "password_entry/placeholder_initial.xml", c)
 
@@ -1738,8 +1870,7 @@ func TestPasswordEntry_Placeholder(t *testing.T) {
 }
 
 func TestPasswordEntry_Reveal(t *testing.T) {
-	test.NewApp()
-	defer test.NewApp()
+	test.NewTempApp(t)
 
 	t.Run("NewPasswordEntry constructor", func(t *testing.T) {
 		entry := widget.NewPasswordEntry()
@@ -1753,34 +1884,34 @@ func TestPasswordEntry_Reveal(t *testing.T) {
 		test.AssertRendersToMarkup(t, "password_entry/initial.xml", c)
 
 		c.Focus(entry)
-		test.Type(entry, "Hié™שרה")
-		assert.Equal(t, "Hié™שרה", entry.Text)
+		test.Type(entry, "Secret")
+		assert.Equal(t, "Secret", entry.Text)
 		test.AssertRendersToMarkup(t, "password_entry/concealed.xml", c)
 
 		// update the Password field
 		entry.Password = false
 		entry.Refresh()
-		assert.Equal(t, "Hié™שרה", entry.Text)
+		assert.Equal(t, "Secret", entry.Text)
 		test.AssertRendersToMarkup(t, "password_entry/revealed.xml", c)
 		assert.Equal(t, entry, c.Focused())
 
 		// update the Password field
 		entry.Password = true
 		entry.Refresh()
-		assert.Equal(t, "Hié™שרה", entry.Text)
+		assert.Equal(t, "Secret", entry.Text)
 		test.AssertRendersToMarkup(t, "password_entry/concealed.xml", c)
 		assert.Equal(t, entry, c.Focused())
 
 		// tap on action icon
 		tapPos := fyne.NewPos(140-theme.InnerPadding()-theme.IconInlineSize()/2, 10+entry.Size().Height/2)
 		test.TapCanvas(c, tapPos)
-		assert.Equal(t, "Hié™שרה", entry.Text)
+		assert.Equal(t, "Secret", entry.Text)
 		test.AssertRendersToMarkup(t, "password_entry/revealed.xml", c)
 		assert.Equal(t, entry, c.Focused())
 
 		// tap on action icon
 		test.TapCanvas(c, tapPos)
-		assert.Equal(t, "Hié™שרה", entry.Text)
+		assert.Equal(t, "Secret", entry.Text)
 		test.AssertRendersToMarkup(t, "password_entry/concealed.xml", c)
 		assert.Equal(t, entry, c.Focused())
 	})
@@ -1801,14 +1932,14 @@ func TestPasswordEntry_Reveal(t *testing.T) {
 		test.AssertRendersToMarkup(t, "password_entry/initial.xml", c)
 
 		c.Focus(entry)
-		test.Type(entry, "Hié™שרה")
-		assert.Equal(t, "Hié™שרה", entry.Text)
+		test.Type(entry, "Secret")
+		assert.Equal(t, "Secret", entry.Text)
 		test.AssertRendersToMarkup(t, "password_entry/concealed.xml", c)
 
 		// update the Password field
 		entry.Password = false
 		entry.Refresh()
-		assert.Equal(t, "Hié™שרה", entry.Text)
+		assert.Equal(t, "Secret", entry.Text)
 		test.AssertRendersToMarkup(t, "password_entry/revealed.xml", c)
 		assert.Equal(t, entry, c.Focused())
 	})
@@ -1821,17 +1952,74 @@ func TestSingleLineEntry_NewlineIgnored(t *testing.T) {
 	checkNewlineIgnored(t, entry)
 }
 
+func TestSingleLineEntry_SelectionSubmitted(t *testing.T) {
+	entry := widget.NewEntry()
+	entry.SetText("abc")
+	assert.Equal(t, "", entry.SelectedText())
+	entry.TypedShortcut(&fyne.ShortcutSelectAll{})
+	assert.Equal(t, "abc", entry.SelectedText())
+	entry.TypedKey(&fyne.KeyEvent{Name: fyne.KeyEnter})
+	assert.Equal(t, entry.Text, "abc")
+}
+
+func TestMultiLineEntry_EnterWithSelection(t *testing.T) {
+	entry := widget.NewMultiLineEntry()
+	entry.SetText("abc")
+	assert.Equal(t, "", entry.SelectedText())
+	entry.TypedShortcut(&fyne.ShortcutSelectAll{})
+	assert.Equal(t, "abc", entry.SelectedText())
+	entry.TypedKey(&fyne.KeyEvent{Name: fyne.KeyEnter})
+	assert.Equal(t, entry.Text, "\n")
+}
+
 func TestEntry_CarriageReturn(t *testing.T) {
 	entry := widget.NewMultiLineEntry()
 	entry.Wrapping = fyne.TextWrapOff
+	entry.Scroll = container.ScrollNone
 	entry.SetText("\r\n\r")
-	w := test.NewWindow(entry)
+	w := test.NewTempWindow(t, entry)
 	w.Resize(fyne.NewSize(64, 64))
 	test.AssertImageMatches(t, "entry/carriage_return_empty.png", w.Canvas().Capture())
 	entry.SetText("\rH\re\rl\rl\ro\r\n\rW\ro\rr\rl\rd\r!\r")
 	test.AssertImageMatches(t, "entry/carriage_return_text.png", w.Canvas().Capture())
 }
 
+func TestEntry_UndoRedo(t *testing.T) {
+	e, window := setupImageTest(t, true)
+	window.Resize(fyne.NewSize(128, 128))
+	c := window.Canvas()
+
+	c.Focus(e)
+	runes := "The undo/\nredo function allows you to efficiently fix"
+	for _, r := range runes {
+		e.TypedRune(r)
+	}
+	test.AssertImageMatches(t, "entry/undo_redo_initial.png", window.Canvas().Capture())
+
+	for _, r := range " mistkaes" {
+		e.TypedRune(r)
+	}
+	test.AssertImageMatches(t, "entry/undo_redo_mistkaes.png", window.Canvas().Capture())
+
+	e.TypedShortcut(&fyne.ShortcutUndo{})
+	test.AssertImageMatches(t, "entry/undo_redo_initial.png", window.Canvas().Capture())
+
+	for _, r := range " mistakes" {
+		e.TypedRune(r)
+	}
+	test.AssertImageMatches(t, "entry/undo_redo_mistake_corrected.png", window.Canvas().Capture())
+
+	for i := 0; i < 5; i++ {
+		e.TypedShortcut(&fyne.ShortcutUndo{})
+	}
+	test.AssertImageMatches(t, "entry/undo_redo_5undo.png", window.Canvas().Capture())
+
+	for i := 0; i < 5; i++ {
+		e.TypedShortcut(&fyne.ShortcutRedo{})
+	}
+	test.AssertImageMatches(t, "entry/undo_redo_mistake_corrected.png", window.Canvas().Capture())
+}
+
 const (
 	entryOffset = 10
 
@@ -1888,14 +2076,15 @@ func checkNewlineIgnored(t *testing.T, entry *widget.Entry) {
 
 func setupImageTest(t *testing.T, multiLine bool) (*widget.Entry, fyne.Window) {
 	test.NewApp()
+	t.Cleanup(func() { test.NewApp() })
 
 	var entry *widget.Entry
 	if multiLine {
-		entry = &widget.Entry{MultiLine: true, Wrapping: fyne.TextWrapWord}
+		entry = &widget.Entry{MultiLine: true, Wrapping: fyne.TextWrapWord, Scroll: container.ScrollNone}
 	} else {
-		entry = &widget.Entry{Wrapping: fyne.TextWrapOff}
+		entry = &widget.Entry{Wrapping: fyne.TextWrapOff, Scroll: container.ScrollNone}
 	}
-	w := test.NewWindow(entry)
+	w := test.NewTempWindow(t, entry)
 	w.Resize(fyne.NewSize(150, 200))
 
 	if multiLine {
@@ -1916,16 +2105,15 @@ func setupImageTest(t *testing.T, multiLine bool) (*widget.Entry, fyne.Window) {
 
 func setupPasswordTest(t *testing.T) (*widget.Entry, fyne.Window) {
 	test.NewApp()
+	t.Cleanup(func() { test.NewApp() })
 
 	entry := widget.NewPasswordEntry()
-	w := test.NewWindow(entry)
+	w := test.NewTempWindow(t, entry)
 	w.Resize(fyne.NewSize(150, 100))
 
 	entry.Resize(entry.MinSize().Max(fyne.NewSize(130, 0)))
 	entry.Move(fyne.NewPos(entryOffset, entryOffset))
 
-	test.AssertRendersToMarkup(t, "password_entry/initial.xml", w.Canvas())
-
 	return entry, w
 }
 
@@ -1955,11 +2143,6 @@ func setupSelection(t *testing.T, reverse bool) (*widget.Entry, fyne.Window) {
 	return e, window
 }
 
-func teardownImageTest(w fyne.Window) {
-	w.Close()
-	test.NewApp()
-}
-
 func waitForBinding() {
 	time.Sleep(time.Millisecond * 100) // data resolves on background thread
 }
@@ -1993,7 +2176,7 @@ func clickPrimary(c fyne.Canvas, obj desktop.Mouseable, ev *fyne.PointEvent) {
 	}
 }
 
-func handleFocusOnTap(c fyne.Canvas, obj interface{}) {
+func handleFocusOnTap(c fyne.Canvas, obj any) {
 	if c == nil {
 		return
 	}
