@@ -11,7 +11,7 @@ import (
 
 func TestNewIcon(t *testing.T) {
 	icon := NewIcon(theme.ConfirmIcon())
-	render := test.WidgetRenderer(icon)
+	render := test.TempWidgetRenderer(t, icon)
 
 	assert.Equal(t, 1, len(render.Objects()))
 	obj := render.Objects()[0]
@@ -24,7 +24,7 @@ func TestNewIcon(t *testing.T) {
 
 func TestIcon_Nil(t *testing.T) {
 	icon := NewIcon(nil)
-	render := test.WidgetRenderer(icon)
+	render := test.TempWidgetRenderer(t, icon)
 
 	assert.Equal(t, 1, len(render.Objects()))
 	assert.Nil(t, render.Objects()[0].(*canvas.Image).Resource)
@@ -40,7 +40,7 @@ func TestIcon_MinSize(t *testing.T) {
 
 func TestIconRenderer_ApplyTheme(t *testing.T) {
 	icon := NewIcon(theme.CancelIcon())
-	render := test.WidgetRenderer(icon).(*iconRenderer)
+	render := test.TempWidgetRenderer(t, icon).(*iconRenderer)
 	visible := render.Objects()[0].Visible()
 
 	render.Refresh()
