@@ -22,7 +22,7 @@ func newExtendedProgressBar() *extendedProgressBar {
 func TestProgressBarRenderer_Extended_Layout(t *testing.T) {
 	bar := newExtendedProgressBar()
 	bar.Resize(fyne.NewSize(100, 100))
-	r := test.WidgetRenderer(bar).(*progressRenderer)
+	r := test.TempWidgetRenderer(t, bar).(*progressRenderer)
 
 	assert.Equal(t, 0.0, bar.Value)
 	assert.Equal(t, float32(0), r.bar.Size().Width)
