@@ -7,9 +7,14 @@ import (
 	"sync"
 	"unicode"
 
+	"github.com/go-text/typesetting/di"
+	"github.com/go-text/typesetting/shaping"
+	"golang.org/x/image/math/fixed"
+
 	"fyne.io/fyne/v2"
 	"fyne.io/fyne/v2/canvas"
 	"fyne.io/fyne/v2/internal/cache"
+	paint "fyne.io/fyne/v2/internal/painter"
 	"fyne.io/fyne/v2/internal/widget"
 	"fyne.io/fyne/v2/layout"
 	"fyne.io/fyne/v2/theme"
@@ -28,6 +33,11 @@ type RichText struct {
 	Wrapping fyne.TextWrap
 	Scroll   widget.ScrollDirection
 
+	// The truncation mode of the text
+	//
+	// Since: 2.4
+	Truncation fyne.TextTruncation
+
 	inset     fyne.Size     // this varies due to how the widget works (entry with scroller vs others with padding)
 	rowBounds []rowBoundary // cache for boundaries
 	scr       *widget.Scroll
@@ -62,9 +72,9 @@ func NewRichTextWithText(text string) *RichText {
 
 // CreateRenderer is a private method to Fyne which links this widget to its renderer
 func (t *RichText) CreateRenderer() fyne.WidgetRenderer {
+	t.prop = canvas.NewRectangle(color.Transparent)
 	if t.scr == nil && t.Scroll != widget.ScrollNone {
-		t.prop = canvas.NewRectangle(color.Transparent)
-		t.scr = widget.NewScroll(&fyne.Container{Layout: layout.NewMaxLayout(), Objects: []fyne.CanvasObject{
+		t.scr = widget.NewScroll(&fyne.Container{Layout: layout.NewStackLayout(), Objects: []fyne.CanvasObject{
 			t.prop, &fyne.Container{}}})
 	}
 
@@ -94,6 +104,12 @@ func (t *RichText) Refresh() {
 	t.minCache = fyne.Size{}
 	t.updateRowBounds()
 
+	for _, s := range t.Segments {
+		if txt, ok := s.(*TextSegment); ok {
+			txt.parent = t
+		}
+	}
+
 	t.BaseWidget.Refresh()
 }
 
@@ -102,17 +118,16 @@ func (t *RichText) Refresh() {
 //
 // Implements: fyne.Widget
 func (t *RichText) Resize(size fyne.Size) {
-	t.propertyLock.RLock()
-	baseSize := t.size
-	segments := t.Segments
-	skipResize := !t.minCache.IsZero() && size.Width >= t.minCache.Width && size.Height >= t.minCache.Height && t.Wrapping == fyne.TextWrapOff
-	t.propertyLock.RUnlock()
-	if baseSize == size {
+	if size == t.size.Load() {
 		return
 	}
-	t.propertyLock.Lock()
-	t.size = size
-	t.propertyLock.Unlock()
+
+	t.size.Store(size)
+
+	t.propertyLock.RLock()
+	segments := t.Segments
+	skipResize := !t.minCache.IsZero() && size.Width >= t.minCache.Width && size.Height >= t.minCache.Height && t.Wrapping == fyne.TextWrapOff && t.Truncation == fyne.TextTruncateOff
+	t.propertyLock.RUnlock()
 
 	if skipResize {
 		if len(segments) < 2 { // we can simplify :)
@@ -134,14 +149,14 @@ func (t *RichText) String() string {
 	return ret.String()
 }
 
-// CharMinSize returns the average char size to use for internal computation
-func (t *RichText) charMinSize(concealed bool, style fyne.TextStyle) fyne.Size {
+// charMinSize returns the average char size to use for internal computation
+func (t *RichText) charMinSize(concealed bool, style fyne.TextStyle, textSize float32) fyne.Size {
 	defaultChar := "M"
 	if concealed {
 		defaultChar = passwordChar
 	}
 
-	return fyne.MeasureText(defaultChar, theme.TextSize(), style)
+	return fyne.MeasureText(defaultChar, textSize, style)
 }
 
 // deleteFromTo removes the text between the specified positions
@@ -210,6 +225,31 @@ func (t *RichText) cachedSegmentVisual(seg RichTextSegment, offset int) fyne.Can
 	return vis
 }
 
+func (t *RichText) cleanVisualCache() {
+	t.cacheLock.Lock()
+	defer t.cacheLock.Unlock()
+	if len(t.visualCache) <= len(t.Segments) {
+		return
+	}
+	var deletingSegs []RichTextSegment
+	for seg1 := range t.visualCache {
+		found := false
+		for _, seg2 := range t.Segments {
+			if seg1 == seg2 {
+				found = true
+				break
+			}
+		}
+		if !found {
+			// cached segment is not currently in t.Segments, clear it
+			deletingSegs = append(deletingSegs, seg1)
+		}
+	}
+	for _, seg := range deletingSegs {
+		delete(t.visualCache, seg)
+	}
+}
+
 // insertAt inserts the text at the specified position
 func (t *RichText) insertAt(pos int, runes string) {
 	index := 0
@@ -233,11 +273,12 @@ func (t *RichText) insertAt(pos int, runes string) {
 		return
 	}
 	r := ([]rune)(into.Text)
+	if pos > len(r) { // safety in case position is out of bounds for the segment
+		pos = len(r)
+	}
 	r2 := append(r[:pos], append([]rune(runes), r[pos:]...)...)
 	into.Text = string(r2)
 	t.Segments[index] = into
-
-	t.Refresh()
 }
 
 // Len returns the text widget buffer length
@@ -250,7 +291,7 @@ func (t *RichText) len() int {
 }
 
 // lineSizeToColumn returns the rendered size for the line specified by row up to the col position
-func (t *RichText) lineSizeToColumn(col, row int) fyne.Size {
+func (t *RichText) lineSizeToColumn(col, row int, textSize, innerPad float32) fyne.Size {
 	if row < 0 {
 		row = 0
 	}
@@ -261,6 +302,9 @@ func (t *RichText) lineSizeToColumn(col, row int) fyne.Size {
 	total := fyne.NewSize(0, 0)
 	counted := 0
 	last := false
+	if bound == nil {
+		return t.charMinSize(false, fyne.TextStyle{}, textSize)
+	}
 	for i, seg := range bound.segments {
 		var size fyne.Size
 		if text, ok := seg.(*TextSegment); ok {
@@ -293,7 +337,7 @@ func (t *RichText) lineSizeToColumn(col, row int) fyne.Size {
 			break
 		}
 	}
-	return total.Add(fyne.NewSize(theme.InnerPadding()-t.inset.Width, 0))
+	return total.Add(fyne.NewSize(innerPad-t.inset.Width, 0))
 }
 
 // Row returns the characters in the row specified.
@@ -346,10 +390,17 @@ func (t *RichText) rows() int {
 // updateRowBounds updates the row bounds used to render properly the text widget.
 // updateRowBounds should be invoked every time a segment Text, widget Wrapping or size changes.
 func (t *RichText) updateRowBounds() {
+	th := t.Theme()
+	innerPadding := th.Size(theme.SizeNameInnerPadding)
+	fitSize := t.size.Load()
+	if t.scr != nil {
+		fitSize = t.scr.Content.MinSize()
+	}
+	fitSize.Height -= (innerPadding + t.inset.Height) * 2
+
 	t.propertyLock.RLock()
 	var bounds []rowBoundary
-	innerPadding := theme.InnerPadding()
-	maxWidth := t.size.Width - 2*innerPadding + 2*t.inset.Width
+	maxWidth := t.size.Load().Width - 2*innerPadding + 2*t.inset.Width
 	wrapWidth := maxWidth
 
 	var currentBound *rowBoundary
@@ -373,11 +424,14 @@ func (t *RichText) updateRowBounds() {
 				} else {
 					bounds[len(bounds)-1].segments = append(bounds[len(bounds)-1].segments, seg)
 				}
+
+				itemMin := t.cachedSegmentVisual(seg, 0).MinSize()
 				if seg.Inline() {
-					wrapWidth -= t.cachedSegmentVisual(seg, 0).MinSize().Width
+					wrapWidth -= itemMin.Width
 				} else {
 					wrapWidth = maxWidth
 					currentBound = nil
+					fitSize.Height -= itemMin.Height + th.Size(theme.SizeNameLineSpacing)
 				}
 				continue
 			}
@@ -389,17 +443,21 @@ func (t *RichText) updateRowBounds() {
 			if textSeg.Style == RichTextStyleBlockquote {
 				leftPad = innerPadding * 2
 			}
-			retBounds := lineBounds(textSeg, t.Wrapping, wrapWidth-leftPad, maxWidth, func(text []rune) float32 {
-				return fyne.MeasureText(string(text), textSize, textStyle).Width
+			retBounds, height := lineBounds(textSeg, t.Wrapping, t.Truncation, wrapWidth-leftPad, fyne.NewSize(maxWidth, fitSize.Height), func(text []rune) fyne.Size {
+				return fyne.MeasureText(string(text), textSize, textStyle)
 			})
 			if currentBound != nil {
 				if len(retBounds) > 0 {
 					bounds[len(bounds)-1].end = retBounds[0].end // invalidate row ending as we have more content
 					bounds[len(bounds)-1].segments = append(bounds[len(bounds)-1].segments, seg)
 					bounds = append(bounds, retBounds[1:]...)
+
+					fitSize.Height -= height
 				}
 			} else {
 				bounds = append(bounds, retBounds...)
+
+				fitSize.Height -= height
 			}
 			currentBound = &bounds[len(bounds)-1]
 			if seg.Inline() {
@@ -418,7 +476,8 @@ func (t *RichText) updateRowBounds() {
 					end = len(runes)
 				}
 				text := string(runes[begin:end])
-				lastWidth := fyne.MeasureText(text, textSeg.size(), textSeg.Style.TextStyle).Width
+				measured := fyne.MeasureText(text, textSeg.size(), textSeg.Style.TextStyle)
+				lastWidth := measured.Width
 				if len(retBounds) == 1 {
 					wrapWidth -= lastWidth
 				} else {
@@ -453,6 +512,7 @@ type textRenderer struct {
 }
 
 func (r *textRenderer) Layout(size fyne.Size) {
+	th := r.obj.Theme()
 	r.obj.propertyLock.RLock()
 	bounds := r.obj.rowBounds
 	objs := r.Objects()
@@ -463,8 +523,8 @@ func (r *textRenderer) Layout(size fyne.Size) {
 	r.obj.propertyLock.RUnlock()
 
 	// Accessing theme here is slow, so we cache the value
-	innerPadding := theme.InnerPadding()
-	lineSpacing := theme.LineSpacing()
+	innerPadding := th.Size(theme.SizeNameInnerPadding)
+	lineSpacing := th.Size(theme.SizeNameLineSpacing)
 
 	xInset := innerPadding - r.obj.inset.Width
 	left := xInset
@@ -525,9 +585,14 @@ func (r *textRenderer) Layout(size fyne.Size) {
 // MinSize calculates the minimum size of a rich text widget.
 // This is based on the contained text with a standard amount of padding added.
 func (r *textRenderer) MinSize() fyne.Size {
+	th := r.obj.Theme()
+	textSize := th.Size(theme.SizeNameText)
 	r.obj.propertyLock.RLock()
+	innerPad := th.Size(theme.SizeNameInnerPadding)
+
 	bounds := r.obj.rowBounds
 	wrap := r.obj.Wrapping
+	trunc := r.obj.Truncation
 	scroll := r.obj.Scroll
 	objs := r.Objects()
 	if r.obj.scr != nil {
@@ -535,13 +600,50 @@ func (r *textRenderer) MinSize() fyne.Size {
 	}
 	r.obj.propertyLock.RUnlock()
 
+	charMinSize := r.obj.charMinSize(false, fyne.TextStyle{}, textSize)
+	min := r.calculateMin(bounds, wrap, objs, charMinSize, th)
+	if r.obj.scr != nil {
+		r.obj.prop.SetMinSize(min)
+	}
+
+	if trunc != fyne.TextTruncateOff && r.obj.Scroll == widget.ScrollNone {
+		minBounds := charMinSize
+		if wrap == fyne.TextWrapOff {
+			minBounds.Height = min.Height
+		} else {
+			minBounds = minBounds.Add(fyne.NewSquareSize(innerPad * 2).Subtract(r.obj.inset).Subtract(r.obj.inset))
+		}
+		if trunc == fyne.TextTruncateClip {
+			return minBounds
+		} else if trunc == fyne.TextTruncateEllipsis {
+			ellipsisSize := fyne.MeasureText("…", th.Size(theme.SizeNameText), fyne.TextStyle{})
+			return minBounds.AddWidthHeight(ellipsisSize.Width, 0)
+		}
+	}
+
+	switch scroll {
+	case widget.ScrollBoth:
+		return fyne.NewSize(32, 32)
+	case widget.ScrollHorizontalOnly:
+		return fyne.NewSize(32, min.Height)
+	case widget.ScrollVerticalOnly:
+		return fyne.NewSize(min.Width, 32)
+	default:
+		return min
+	}
+}
+
+func (r *textRenderer) calculateMin(bounds []rowBoundary, wrap fyne.TextWrap, objs []fyne.CanvasObject,
+	charMinSize fyne.Size, th fyne.Theme) fyne.Size {
 	height := float32(0)
 	width := float32(0)
 	rowHeight := float32(0)
 	rowWidth := float32(0)
+	trunc := r.obj.Truncation
+	innerPad := th.Size(theme.SizeNameInnerPadding)
 
 	// Accessing the theme here is slow, so we cache the value
-	lineSpacing := theme.LineSpacing()
+	lineSpacing := th.Size(theme.SizeNameLineSpacing)
 
 	i := 0
 	for row, bound := range bounds {
@@ -556,6 +658,11 @@ func (r *textRenderer) MinSize() fyne.Size {
 			if img, ok := obj.(*richImage); ok {
 				if !img.MinSize().Subtract(img.oldMin).IsZero() {
 					img.oldMin = img.MinSize()
+
+					min := r.calculateMin(bounds, wrap, objs, charMinSize, th)
+					if r.obj.scr != nil {
+						r.obj.prop.SetMinSize(min)
+					}
 					r.Refresh() // TODO resolve this in a similar way to #2991
 				}
 			}
@@ -563,7 +670,7 @@ func (r *textRenderer) MinSize() fyne.Size {
 			rowWidth += min.Width
 		}
 
-		if wrap == fyne.TextWrapOff {
+		if wrap == fyne.TextWrapOff && trunc == fyne.TextTruncateOff {
 			width = fyne.Max(width, rowWidth)
 		}
 		height += rowHeight
@@ -577,26 +684,10 @@ func (r *textRenderer) MinSize() fyne.Size {
 	}
 
 	if height == 0 {
-		charMinSize := r.obj.charMinSize(false, fyne.TextStyle{})
 		height = charMinSize.Height
 	}
-	min := fyne.NewSize(width, height).
-		Add(fyne.NewSize(theme.InnerPadding()*2, theme.InnerPadding()*2).Subtract(r.obj.inset).Subtract(r.obj.inset))
-
-	if r.obj.scr != nil {
-		r.obj.prop.SetMinSize(min)
-	}
-
-	switch scroll {
-	case widget.ScrollBoth:
-		return fyne.NewSize(32, 32)
-	case widget.ScrollHorizontalOnly:
-		return fyne.NewSize(32, min.Height)
-	case widget.ScrollVerticalOnly:
-		return fyne.NewSize(min.Width, 32)
-	default:
-		return min
-	}
+	return fyne.NewSize(width, height).
+		Add(fyne.NewSquareSize(innerPad * 2).Subtract(r.obj.inset).Subtract(r.obj.inset))
 }
 
 func (r *textRenderer) Refresh() {
@@ -634,6 +725,10 @@ func (r *textRenderer) Refresh() {
 			} else if i == len(bound.segments)-1 && len(bound.segments) > 1 {
 				txt.Text = string(runes[:bound.end])
 			}
+			if bound.ellipsis && i == len(bound.segments)-1 {
+				txt.Text = txt.Text + "…"
+			}
+
 			if concealed(seg) {
 				txt.Text = strings.Repeat(passwordChar, len(runes))
 			}
@@ -644,7 +739,7 @@ func (r *textRenderer) Refresh() {
 
 	r.obj.propertyLock.Lock()
 	if r.obj.scr != nil {
-		r.obj.scr.Content = &fyne.Container{Layout: layout.NewMaxLayout(), Objects: []fyne.CanvasObject{
+		r.obj.scr.Content = &fyne.Container{Layout: layout.NewStackLayout(), Objects: []fyne.CanvasObject{
 			r.obj.prop, &fyne.Container{Objects: objs}}}
 		r.obj.scr.Direction = scroll
 		r.SetObjects([]fyne.CanvasObject{r.obj.scr})
@@ -655,7 +750,9 @@ func (r *textRenderer) Refresh() {
 	r.obj.propertyLock.Unlock()
 
 	r.Layout(r.obj.Size())
-	canvas.Refresh(r.obj)
+	canvas.Refresh(r.obj.super())
+
+	r.obj.cleanVisualCache()
 }
 
 func (r *textRenderer) layoutRow(texts []fyne.CanvasObject, align fyne.TextAlign, xPos, yPos, lineWidth float32) (float32, float32) {
@@ -685,12 +782,12 @@ func (r *textRenderer) layoutRow(texts []fyne.CanvasObject, align fyne.TextAlign
 	baselines := make([]float32, len(texts))
 
 	// Access to theme is slow, so we cache the text size
-	textSize := theme.TextSize()
+	textSize := theme.SizeForWidget(theme.SizeNameText, r.obj)
 
 	for i, text := range texts {
 		var size fyne.Size
 		if txt, ok := text.(*canvas.Text); ok {
-			s, base := fyne.CurrentApp().Driver().RenderedTextSize(txt.Text, txt.TextSize, txt.TextStyle)
+			s, base := fyne.CurrentApp().Driver().RenderedTextSize(txt.Text, txt.TextSize, txt.TextStyle, txt.FontSource)
 			if base > tallestBaseline {
 				if tallestBaseline > 0 {
 					realign = true
@@ -702,7 +799,7 @@ func (r *textRenderer) layoutRow(texts []fyne.CanvasObject, align fyne.TextAlign
 		} else if c, ok := text.(*fyne.Container); ok {
 			wid := c.Objects[0]
 			if link, ok := wid.(*Hyperlink); ok {
-				s, base := fyne.CurrentApp().Driver().RenderedTextSize(link.Text, textSize, link.TextStyle)
+				s, base := fyne.CurrentApp().Driver().RenderedTextSize(link.Text, textSize, link.TextStyle, nil)
 				if base > tallestBaseline {
 					if tallestBaseline > 0 {
 						realign = true
@@ -798,6 +895,27 @@ func concealed(seg RichTextSegment) bool {
 	return false
 }
 
+func ellipsisPriorBound(bounds []rowBoundary, trunc fyne.TextTruncation, width float32, measurer func([]rune) fyne.Size) []rowBoundary {
+	if trunc != fyne.TextTruncateEllipsis || len(bounds) == 0 {
+		return bounds
+	}
+
+	prior := bounds[len(bounds)-1]
+	seg := prior.segments[0].(*TextSegment)
+	ellipsisSize := fyne.MeasureText("…", seg.size(), seg.Style.TextStyle)
+
+	widthChecker := func(low int, high int) bool {
+		return measurer([]rune(seg.Text)[low:high]).Width <= width-ellipsisSize.Width
+	}
+
+	limit := binarySearch(widthChecker, prior.begin, prior.end)
+	prior.end = limit
+
+	prior.ellipsis = true
+	bounds[len(bounds)-1] = prior
+	return bounds
+}
+
 // findSpaceIndex accepts a slice of runes and a fallback index
 // findSpaceIndex returns the index of the last space in the text, or fallback if there are no spaces
 func findSpaceIndex(text []rune, fallback int) int {
@@ -813,21 +931,36 @@ func findSpaceIndex(text []rune, fallback int) int {
 	return curIndex
 }
 
-// lineBounds accepts a slice of Segments, a wrapping mode, a maximum line width and a function to measure line width.
-// lineBounds returns a slice containing the boundary metadata of each line with the given wrapping applied.
-func lineBounds(seg *TextSegment, wrap fyne.TextWrap, firstWidth, maxWidth float32, measurer func([]rune) float32) []rowBoundary {
+func float32ToFixed266(f float32) fixed.Int26_6 {
+	return fixed.Int26_6(float64(f) * (1 << 6))
+}
+
+// lineBounds accepts a slice of Segments, a wrapping mode, a maximum size available to display and a function to
+// measure text size.
+// It will return a slice containing the boundary metadata of each line with the given wrapping applied and the
+// total height required to render the boundaries at the given width/height constraints
+func lineBounds(seg *TextSegment, wrap fyne.TextWrap, trunc fyne.TextTruncation, firstWidth float32, max fyne.Size, measurer func([]rune) fyne.Size) ([]rowBoundary, float32) {
 	lines := splitLines(seg)
-	if maxWidth < 0 || wrap == fyne.TextWrapOff {
-		return lines
+
+	if wrap == fyne.TextWrap(fyne.TextTruncateClip) {
+		if trunc == fyne.TextTruncateOff {
+			trunc = fyne.TextTruncateClip
+		}
+		wrap = fyne.TextWrapOff
 	}
 
-	measureWidth := firstWidth
+	if max.Width < 0 || wrap == fyne.TextWrapOff && trunc == fyne.TextTruncateOff {
+		return lines, 0 // don't bother returning a calculated height, our MinSize is going to cover it
+	}
+
+	measureWidth := float32(math.Min(float64(firstWidth), float64(max.Width)))
 	text := []rune(seg.Text)
-	checker := func(low int, high int) bool {
-		return measurer(text[low:high]) <= measureWidth
+	widthChecker := func(low int, high int) bool {
+		return measurer(text[low:high]).Width <= measureWidth
 	}
 
 	reuse := 0
+	yPos := float32(0)
 	var bounds []rowBoundary
 	for _, l := range lines {
 		low := l.begin
@@ -838,25 +971,31 @@ func lineBounds(seg *TextSegment, wrap fyne.TextWrap, firstWidth, maxWidth float
 			bounds = append(bounds, l)
 			continue
 		}
+
 		switch wrap {
-		case fyne.TextTruncate:
-			high = binarySearch(checker, low, high)
-			bounds = append(bounds, rowBoundary{[]RichTextSegment{seg}, reuse, low, high})
-			reuse++
 		case fyne.TextWrapBreak:
 			for low < high {
-				if measurer(text[low:high]) <= measureWidth {
-					bounds = append(bounds, rowBoundary{[]RichTextSegment{seg}, reuse, low, high})
+				measured := measurer(text[low:high])
+				if yPos+measured.Height > max.Height && trunc != fyne.TextTruncateOff {
+					return ellipsisPriorBound(bounds, trunc, measureWidth, measurer), yPos
+				}
+
+				if measured.Width <= measureWidth {
+					bounds = append(bounds, rowBoundary{[]RichTextSegment{seg}, reuse, low, high, false})
 					reuse++
 					low = high
 					high = l.end
-					measureWidth = maxWidth
+					measureWidth = max.Width
+
+					yPos += measured.Height
 				} else {
-					newHigh := binarySearch(checker, low, high)
+					newHigh := binarySearch(widthChecker, low, high)
 					if newHigh <= low {
-						bounds = append(bounds, rowBoundary{[]RichTextSegment{seg}, reuse, low, low + 1})
+						bounds = append(bounds, rowBoundary{[]RichTextSegment{seg}, reuse, low, low + 1, false})
 						reuse++
 						low++
+
+						yPos += measured.Height
 					} else {
 						high = newHigh
 					}
@@ -865,29 +1004,43 @@ func lineBounds(seg *TextSegment, wrap fyne.TextWrap, firstWidth, maxWidth float
 		case fyne.TextWrapWord:
 			for low < high {
 				sub := text[low:high]
-				subWidth := measurer(sub)
+				measured := measurer(sub)
+				if yPos+measured.Height > max.Height && trunc != fyne.TextTruncateOff {
+					return ellipsisPriorBound(bounds, trunc, measureWidth, measurer), yPos
+				}
+
+				subWidth := measured.Width
 				if subWidth <= measureWidth {
-					bounds = append(bounds, rowBoundary{[]RichTextSegment{seg}, reuse, low, high})
+					bounds = append(bounds, rowBoundary{[]RichTextSegment{seg}, reuse, low, high, false})
 					reuse++
 					low = high
 					high = l.end
 					if low < high && unicode.IsSpace(text[low]) {
 						low++
 					}
-					measureWidth = maxWidth
+					measureWidth = max.Width
+
+					yPos += measured.Height
 				} else {
 					oldHigh := high
 					last := low + len(sub) - 1
-					fallback := binarySearch(checker, low, last) - low
+					fallback := binarySearch(widthChecker, low, last) - low
 
 					if fallback < 1 { // even a character won't fit
-						bounds = append(bounds, rowBoundary{[]RichTextSegment{seg}, reuse, low, low + 1})
+						include := 1
+						ellipsis := false
+						if trunc == fyne.TextTruncateEllipsis {
+							include = 0
+							ellipsis = true
+						}
+						bounds = append(bounds, rowBoundary{[]RichTextSegment{seg}, reuse, low, low + include, ellipsis})
 						low++
 						high = low + 1
 						reuse++
 
+						yPos += measured.Height
 						if high > l.end {
-							return bounds
+							return bounds, yPos
 						}
 					} else {
 						spaceIndex := findSpaceIndex(sub, fallback)
@@ -897,18 +1050,32 @@ func lineBounds(seg *TextSegment, wrap fyne.TextWrap, firstWidth, maxWidth float
 
 						high = low + spaceIndex
 					}
-					if high == fallback && subWidth <= maxWidth { // add a newline as there is more space on next
-						bounds = append(bounds, rowBoundary{[]RichTextSegment{seg}, reuse, low, low})
+					if high == fallback && subWidth <= max.Width { // add a newline as there is more space on next
+						bounds = append(bounds, rowBoundary{[]RichTextSegment{seg}, reuse, low, low, false})
 						reuse++
 						high = oldHigh
-						measureWidth = maxWidth
+						measureWidth = max.Width
+
+						yPos += measured.Height
 						continue
 					}
 				}
 			}
+		default:
+			if trunc == fyne.TextTruncateEllipsis {
+				txt := []rune(seg.Text)[low:high]
+				end, full := truncateLimit(string(txt), seg.Visual().(*canvas.Text), int(measureWidth), []rune{'…'})
+				high = low + end
+				bounds = append(bounds, rowBoundary{[]RichTextSegment{seg}, reuse, low, high, !full})
+				reuse++
+			} else if trunc == fyne.TextTruncateClip {
+				high = binarySearch(widthChecker, low, high)
+				bounds = append(bounds, rowBoundary{[]RichTextSegment{seg}, reuse, low, high, false})
+				reuse++
+			}
 		}
 	}
-	return bounds
+	return bounds, yPos
 }
 
 func setAlign(obj fyne.CanvasObject, align fyne.TextAlign) {
@@ -935,15 +1102,51 @@ func splitLines(seg *TextSegment) []rowBoundary {
 	for i := 0; i < length; i++ {
 		if text[i] == '\n' {
 			high = i
-			lines = append(lines, rowBoundary{[]RichTextSegment{seg}, len(lines), low, high})
+			lines = append(lines, rowBoundary{[]RichTextSegment{seg}, len(lines), low, high, false})
 			low = i + 1
 		}
 	}
-	return append(lines, rowBoundary{[]RichTextSegment{seg}, len(lines), low, length})
+	return append(lines, rowBoundary{[]RichTextSegment{seg}, len(lines), low, length, false})
+}
+
+func truncateLimit(s string, text *canvas.Text, limit int, ellipsis []rune) (int, bool) {
+	face := paint.CachedFontFace(text.TextStyle, text.FontSource, text)
+
+	runes := []rune(s)
+	in := shaping.Input{
+		Text:      ellipsis,
+		RunStart:  0,
+		RunEnd:    len(ellipsis),
+		Direction: di.DirectionLTR,
+		Face:      face.Fonts.ResolveFace('…'),
+		Size:      float32ToFixed266(text.TextSize),
+	}
+	shaper := &shaping.HarfbuzzShaper{}
+
+	conf := shaping.WrapConfig{}
+	conf = conf.WithTruncator(shaper, in)
+	conf.BreakPolicy = shaping.WhenNecessary
+	conf.TruncateAfterLines = 1
+	l := shaping.LineWrapper{}
+
+	in.Text = runes
+	in.RunEnd = len(runes)
+	out := shaper.Shape(in)
+
+	l.Prepare(conf, runes, shaping.NewSliceIterator([]shaping.Output{out}))
+	wrapped, done := l.WrapNextLine(limit)
+
+	count := wrapped.Line[0].Runes.Count
+	full := done && count == len(runes)
+	if !full && len(ellipsis) > 0 {
+		count--
+	}
+	return count, full
 }
 
 type rowBoundary struct {
 	segments          []RichTextSegment
 	firstSegmentReuse int
 	begin, end        int
+	ellipsis          bool
 }
