@@ -7,7 +7,7 @@ import (
 
 	"fyne.io/fyne/v2"
 	"fyne.io/fyne/v2/canvas"
-	"fyne.io/fyne/v2/internal"
+	"fyne.io/fyne/v2/internal/scale"
 	"fyne.io/fyne/v2/theme"
 )
 
@@ -109,6 +109,11 @@ type HyperlinkSegment struct {
 	Alignment fyne.TextAlign
 	Text      string
 	URL       *url.URL
+
+	// OnTapped overrides the default `fyne.OpenURL` call when the link is tapped
+	//
+	// Since: 2.4
+	OnTapped func() `json:"-"`
 }
 
 // Inline returns true as hyperlinks are inside other elements.
@@ -125,7 +130,8 @@ func (h *HyperlinkSegment) Textual() string {
 func (h *HyperlinkSegment) Visual() fyne.CanvasObject {
 	link := NewHyperlink(h.Text, h.URL)
 	link.Alignment = h.Alignment
-	return &fyne.Container{Layout: &unpadTextWidgetLayout{}, Objects: []fyne.CanvasObject{link}}
+	link.OnTapped = h.OnTapped
+	return &fyne.Container{Layout: &unpadTextWidgetLayout{parent: link}, Objects: []fyne.CanvasObject{link}}
 }
 
 // Update applies the current state of this hyperlink segment to an existing visual.
@@ -134,6 +140,7 @@ func (h *HyperlinkSegment) Update(o fyne.CanvasObject) {
 	link.Text = h.Text
 	link.URL = h.URL
 	link.Alignment = h.Alignment
+	link.OnTapped = h.OnTapped
 	link.Refresh()
 }
 
@@ -159,6 +166,10 @@ func (h *HyperlinkSegment) Unselect() {
 type ImageSegment struct {
 	Source fyne.URI
 	Title  string
+
+	// Alignment specifies the horizontal alignment of this image segment
+	// Since: 2.4
+	Alignment fyne.TextAlign
 }
 
 // Inline returns false as images in rich text are blocks.
@@ -173,7 +184,7 @@ func (i *ImageSegment) Textual() string {
 
 // Visual returns the image widget required to render this segment.
 func (i *ImageSegment) Visual() fyne.CanvasObject {
-	return newRichImage(i.Source)
+	return newRichImage(i.Source, i.Alignment)
 }
 
 // Update applies the current state of this image segment to an existing visual.
@@ -184,6 +195,7 @@ func (i *ImageSegment) Update(o fyne.CanvasObject) {
 	// one of the following will be used
 	img.img.File = newer.File
 	img.img.Resource = newer.Resource
+	img.setAlign(i.Alignment)
 
 	img.Refresh()
 }
@@ -310,8 +322,7 @@ func (p *ParagraphSegment) Unselect() {
 //
 // Since: 2.1
 type SeparatorSegment struct {
-	//lint:ignore U1000 This is required due to language design.
-	dummy uint8 // without this a pointer to SeparatorSegment will always be the same
+	_ bool // Without this a pointer to SeparatorSegment will always be the same.
 }
 
 // Inline returns false as a separator should be full width.
@@ -380,6 +391,8 @@ type RichTextSegment interface {
 type TextSegment struct {
 	Style RichTextStyle
 	Text  string
+
+	parent *RichText
 }
 
 // Inline should return true if this text can be included within other elements, or false if it creates a new block.
@@ -429,36 +442,42 @@ func (t *TextSegment) Unselect() {
 
 func (t *TextSegment) color() color.Color {
 	if t.Style.ColorName != "" {
-		return fyne.CurrentApp().Settings().Theme().Color(t.Style.ColorName, fyne.CurrentApp().Settings().ThemeVariant())
+		return theme.ColorForWidget(t.Style.ColorName, t.parent)
 	}
 
-	return theme.ForegroundColor()
+	return theme.ColorForWidget(theme.ColorNameForeground, t.parent)
 }
 
 func (t *TextSegment) size() float32 {
 	if t.Style.SizeName != "" {
-		return fyne.CurrentApp().Settings().Theme().Size(t.Style.SizeName)
+		i := theme.SizeForWidget(t.Style.SizeName, t.parent)
+		return i
 	}
 
-	return theme.TextSize()
+	i := theme.SizeForWidget(theme.SizeNameText, t.parent)
+	return i
 }
 
 type richImage struct {
 	BaseWidget
+	align  fyne.TextAlign
 	img    *canvas.Image
 	oldMin fyne.Size
+	layout *fyne.Container
+	min    fyne.Size
 }
 
-func newRichImage(u fyne.URI) *richImage {
+func newRichImage(u fyne.URI, align fyne.TextAlign) *richImage {
 	img := canvas.NewImageFromURI(u)
 	img.FillMode = canvas.ImageFillOriginal
-	i := &richImage{img: img}
+	i := &richImage{img: img, align: align}
 	i.ExtendBaseWidget(i)
 	return i
 }
 
 func (r *richImage) CreateRenderer() fyne.WidgetRenderer {
-	return NewSimpleRenderer(r.img)
+	r.layout = &fyne.Container{Layout: &richImageLayout{r}, Objects: []fyne.CanvasObject{r.img}}
+	return NewSimpleRenderer(r.layout)
 }
 
 func (r *richImage) MinSize() fyne.Size {
@@ -469,17 +488,49 @@ func (r *richImage) MinSize() fyne.Size {
 	}
 
 	// unscale the image so it is not varying based on canvas
-	w := internal.ScaleInt(c, orig.Width)
-	h := internal.ScaleInt(c, orig.Height)
+	w := scale.ToScreenCoordinate(c, orig.Width)
+	h := scale.ToScreenCoordinate(c, orig.Height)
 	// we return size / 2 as this assumes a HiDPI / 2x image scaling
-	return fyne.NewSize(float32(w)/2, float32(h)/2)
+	r.min = fyne.NewSize(float32(w)/2, float32(h)/2)
+	return r.min
+}
+
+func (r *richImage) setAlign(a fyne.TextAlign) {
+	if r.layout != nil {
+		r.layout.Refresh()
+	}
+	r.align = a
+}
+
+type richImageLayout struct {
+	r *richImage
+}
+
+func (r *richImageLayout) Layout(_ []fyne.CanvasObject, s fyne.Size) {
+	r.r.img.Resize(r.r.min)
+	gap := float32(0)
+
+	switch r.r.align {
+	case fyne.TextAlignCenter:
+		gap = (s.Width - r.r.min.Width) / 2
+	case fyne.TextAlignTrailing:
+		gap = s.Width - r.r.min.Width
+	}
+
+	r.r.img.Move(fyne.NewPos(gap, 0))
+}
+
+func (r *richImageLayout) MinSize(_ []fyne.CanvasObject) fyne.Size {
+	return r.r.min
 }
 
 type unpadTextWidgetLayout struct {
+	parent fyne.Widget
 }
 
 func (u *unpadTextWidgetLayout) Layout(o []fyne.CanvasObject, s fyne.Size) {
-	pad := theme.InnerPadding() * -1
+	innerPad := theme.SizeForWidget(theme.SizeNameInnerPadding, u.parent)
+	pad := innerPad * -1
 	pad2 := pad * -2
 
 	o[0].Move(fyne.NewPos(pad, pad))
@@ -487,6 +538,7 @@ func (u *unpadTextWidgetLayout) Layout(o []fyne.CanvasObject, s fyne.Size) {
 }
 
 func (u *unpadTextWidgetLayout) MinSize(o []fyne.CanvasObject) fyne.Size {
-	pad := theme.InnerPadding() * 2
+	innerPad := theme.SizeForWidget(theme.SizeNameInnerPadding, u.parent)
+	pad := innerPad * 2
 	return o[0].MinSize().Subtract(fyne.NewSize(pad, pad))
 }
