@@ -14,8 +14,8 @@ import (
 	"fyne.io/fyne/v2/theme"
 )
 
-func textRenderTexts(p fyne.Widget) []*canvas.Text {
-	renderer := cache.Renderer(p).(*textRenderer)
+func richTextRenderTexts(rich fyne.Widget) []*canvas.Text {
+	renderer := cache.Renderer(rich).(*textRenderer)
 	texts := make([]*canvas.Text, len(renderer.Objects()))
 	for i, obj := range renderer.Objects() {
 		texts[i] = obj.(*canvas.Text)
@@ -49,7 +49,7 @@ func TestText_Alignment(t *testing.T) {
 	seg := trailingBoldErrorSegment()
 	seg.Text = "Test"
 	text := NewRichText(seg)
-	assert.Equal(t, fyne.TextAlignTrailing, test.WidgetRenderer(text).Objects()[0].(*canvas.Text).Alignment)
+	assert.Equal(t, fyne.TextAlignTrailing, test.TempWidgetRenderer(t, text).Objects()[0].(*canvas.Text).Alignment)
 }
 
 func TestText_Row(t *testing.T) {
@@ -126,6 +126,20 @@ func TestText_Scroll(t *testing.T) {
 	text3.Scroll = widget.ScrollVerticalOnly
 	assert.Equal(t, text3.MinSize().Width, text1.MinSize().Width)
 	assert.Less(t, text3.MinSize().Height, text1.MinSize().Height)
+
+	text4 := NewRichTextWithText("test1\ntest2")
+	text4.Scroll = widget.ScrollVerticalOnly
+	text4.Wrapping = fyne.TextWrapBreak
+
+	text3.Resize(fyne.NewSize(32, 32))
+	text4.Resize(fyne.NewSize(32, 32))
+	assert.Less(t, text4.MinSize().Width, text3.MinSize().Width)
+	assert.Equal(t, text4.MinSize().Height, text3.MinSize().Height)
+
+	content3 := test.TempWidgetRenderer(t, text3).Objects()[0].(*widget.Scroll).Content
+	content4 := test.TempWidgetRenderer(t, text4).Objects()[0].(*widget.Scroll).Content
+	assert.Less(t, content4.MinSize().Width, content3.MinSize().Width)
+	assert.Greater(t, content4.MinSize().Height, content3.MinSize().Height)
 }
 
 func TestText_InsertAt(t *testing.T) {
@@ -319,6 +333,11 @@ func TestText_DeleteFromTo_Segments(t *testing.T) {
 			text := NewRichText(tt.segments...)
 			got := text.deleteFromTo(tt.args.lowBound, tt.args.highBound)
 			assert.Equal(t, tt.want, got)
+			for _, s := range tt.wantSegments {
+				if txt, ok := s.(*TextSegment); ok {
+					txt.parent = text
+				}
+			}
 			assert.Equal(t, tt.wantSegments, text.Segments)
 		})
 	}
@@ -329,7 +348,7 @@ func TestText_Multiline(t *testing.T) {
 		&TextSegment{Text: "line1\nli", Style: RichTextStyleStrong},
 		&TextSegment{Text: "ne2\nline3", Style: RichTextStyleInline})
 
-	w := test.NewWindow(text)
+	w := test.NewTempWindow(t, text)
 	w.Resize(fyne.NewSize(64, 90))
 	test.AssertImageMatches(t, "richtext/richtext_multiline.png", w.Canvas().Capture())
 }
@@ -337,23 +356,23 @@ func TestText_Multiline(t *testing.T) {
 func TestText_Color(t *testing.T) {
 	text := NewRichText(trailingBoldErrorSegment())
 
-	assert.Equal(t, theme.ErrorColor(), textRenderTexts(text)[0].Color)
+	assert.Equal(t, theme.Color(theme.ColorNameError), richTextRenderTexts(text)[0].Color)
 }
 
 func TestTextRenderer_ApplyTheme(t *testing.T) {
 	label := NewLabel("Test\nLine2")
-	render := test.WidgetRenderer(label).(*textRenderer)
+	texts := labelTextRenderTexts(label)
 
-	text1 := render.Objects()[0].(*canvas.Text)
-	text2 := render.Objects()[1].(*canvas.Text)
+	text1 := texts[0]
+	text2 := texts[1]
 	textSize1 := text1.TextSize
 	textSize2 := text2.TextSize
 	customTextSize1 := textSize1
 	customTextSize2 := textSize2
 	test.WithTestTheme(t, func() {
 		label.Refresh()
-		text1 := render.Objects()[0].(*canvas.Text)
-		text2 := render.Objects()[1].(*canvas.Text)
+		text1 := texts[0]
+		text2 := texts[1]
 		customTextSize1 = text1.TextSize
 		customTextSize2 = text2.TextSize
 	})
@@ -367,12 +386,14 @@ func TestTextProvider_LineSizeToColumn(t *testing.T) {
 	label.CreateRenderer() // TODO make this a simple refresh call once it's in
 	provider := label.provider
 
-	fullSize := provider.lineSizeToColumn(4, 0)
-	assert.Equal(t, fullSize, provider.lineSizeToColumn(10, 0))
-	assert.Greater(t, fullSize.Width, provider.lineSizeToColumn(2, 0).Width)
+	inPad := theme.InnerPadding()
+	textSize := theme.TextSize()
+	fullSize := provider.lineSizeToColumn(4, 0, textSize, inPad)
+	assert.Equal(t, fullSize, provider.lineSizeToColumn(10, 0, textSize, inPad))
+	assert.Greater(t, fullSize.Width, provider.lineSizeToColumn(2, 0, textSize, inPad).Width)
 
-	out := provider.lineSizeToColumn(-1, -1)
-	assert.Equal(t, out, provider.lineSizeToColumn(0, 0))
+	out := provider.lineSizeToColumn(-1, -1, textSize, inPad)
+	assert.Equal(t, out, provider.lineSizeToColumn(0, 0, textSize, inPad))
 }
 
 func TestText_splitLines(t *testing.T) {
@@ -425,14 +446,16 @@ func TestText_splitLines(t *testing.T) {
 }
 
 func TestText_lineBounds(t *testing.T) {
-	mockMeasurer := func(text []rune) float32 {
-		return float32(len(text))
+	measurer := func(text []rune) fyne.Size {
+		return fyne.MeasureText(string(text), 14, fyne.TextStyle{})
 	}
 	tests := []struct {
-		name string
-		text string
-		wrap fyne.TextWrap
-		want [][2]int
+		name     string
+		text     string
+		wrap     fyne.TextWrap
+		trunc    fyne.TextTruncation
+		want     [][2]int
+		ellipses int
 	}{
 		{
 			name: "Empty_WrapOff",
@@ -445,7 +468,15 @@ func TestText_lineBounds(t *testing.T) {
 		{
 			name: "Empty_Truncate",
 			text: "",
-			wrap: fyne.TextTruncate,
+			wrap: fyne.TextWrap(fyne.TextTruncateClip),
+			want: [][2]int{
+				{0, 0},
+			},
+		},
+		{
+			name:  "Empty_TruncateClip",
+			text:  "",
+			trunc: fyne.TextTruncateClip,
 			want: [][2]int{
 				{0, 0},
 			},
@@ -477,11 +508,28 @@ func TestText_lineBounds(t *testing.T) {
 		{
 			name: "Single_Short_Truncate",
 			text: "foobar",
-			wrap: fyne.TextTruncate,
+			wrap: fyne.TextWrap(fyne.TextTruncateClip),
 			want: [][2]int{
 				{0, 6},
 			},
 		},
+		{
+			name:  "Single_Short_TruncateClip",
+			text:  "foobar",
+			trunc: fyne.TextTruncateClip,
+			want: [][2]int{
+				{0, 6},
+			},
+		},
+		{
+			name:  "Single_Short_TruncateEllipsis",
+			text:  "foobar",
+			trunc: fyne.TextTruncateEllipsis,
+			want: [][2]int{
+				{0, 6},
+			},
+			ellipses: 0, // too short to cut
+		},
 		{
 			name: "Single_Short_WrapBreak",
 			text: "foobar",
@@ -509,11 +557,28 @@ func TestText_lineBounds(t *testing.T) {
 		{
 			name: "Single_Long_Truncate",
 			text: "foobar foobar",
-			wrap: fyne.TextTruncate,
+			wrap: fyne.TextWrap(fyne.TextTruncateClip),
 			want: [][2]int{
 				{0, 10},
 			},
 		},
+		{
+			name:  "Single_Long_TruncateClip",
+			text:  "foobar foobar",
+			trunc: fyne.TextTruncateClip,
+			want: [][2]int{
+				{0, 10},
+			},
+		},
+		{
+			name:  "Single_Long_TruncateEllipsis",
+			text:  "foobar foobar",
+			trunc: fyne.TextTruncateEllipsis,
+			want: [][2]int{
+				{0, 8},
+			},
+			ellipses: 1,
+		},
 		{
 			name: "Single_Long_WrapBreak",
 			text: "foobar foobar",
@@ -544,12 +609,31 @@ func TestText_lineBounds(t *testing.T) {
 		{
 			name: "Multiple_Short_Truncate",
 			text: "foo\nbar",
-			wrap: fyne.TextTruncate,
+			wrap: fyne.TextWrap(fyne.TextTruncateClip),
 			want: [][2]int{
 				{0, 3},
 				{4, 7},
 			},
 		},
+		{
+			name:  "Multiple_Short_TruncateClip",
+			text:  "foo\nbar",
+			trunc: fyne.TextTruncateClip,
+			want: [][2]int{
+				{0, 3},
+				{4, 7},
+			},
+		},
+		{
+			name:  "Multiple_Short_TruncateEllipsis",
+			text:  "foo\nbar",
+			trunc: fyne.TextTruncateEllipsis,
+			want: [][2]int{
+				{0, 3},
+				{4, 7},
+			},
+			ellipses: 0, // too wide
+		},
 		{
 			name: "Multiple_Short_WrapBreak",
 			text: "foo\nbar",
@@ -581,13 +665,34 @@ func TestText_lineBounds(t *testing.T) {
 		{
 			name: "Multiple_Long_Truncate",
 			text: "foobar\nfoobar foobar foobar\nfoobar foobar",
-			wrap: fyne.TextTruncate,
+			wrap: fyne.TextWrap(fyne.TextTruncateClip),
 			want: [][2]int{
 				{0, 6},
 				{7, 17},
 				{28, 38},
 			},
 		},
+		{
+			name:  "Multiple_Long_TruncateClip",
+			text:  "foobar\nfoobar foobar foobar\nfoobar foobar",
+			trunc: fyne.TextTruncateClip,
+			want: [][2]int{
+				{0, 6},
+				{7, 17},
+				{28, 38},
+			},
+		},
+		{
+			name:  "Multiple_Long_TruncateEllipsis",
+			text:  "foobar\nfoobar foobar foobar\nfoobar foobar",
+			trunc: fyne.TextTruncateEllipsis,
+			want: [][2]int{
+				{0, 6},
+				{7, 15},
+				{28, 36},
+			},
+			ellipses: 2,
+		},
 		{
 			name: "Multiple_Long_WrapBreak",
 			text: "foobar\nfoobar foobar foobar\nfoobar foobar",
@@ -627,7 +732,7 @@ func TestText_lineBounds(t *testing.T) {
 		{
 			name: "Multiple_Contiguous_Long_Truncate",
 			text: "foobar\nfoobarfoobarfoobar\nfoobarfoobar\n",
-			wrap: fyne.TextTruncate,
+			wrap: fyne.TextWrap(fyne.TextTruncateClip),
 			want: [][2]int{
 				{0, 6},
 				{7, 17},
@@ -635,6 +740,29 @@ func TestText_lineBounds(t *testing.T) {
 				{39, 39},
 			},
 		},
+		{
+			name:  "Multiple_Contiguous_Long_TruncateClip",
+			text:  "foobar\nfoobarfoobarfoobar\nfoobarfoobar\n",
+			trunc: fyne.TextTruncateClip,
+			want: [][2]int{
+				{0, 6},
+				{7, 17},
+				{26, 36},
+				{39, 39},
+			},
+		},
+		{
+			name:  "Multiple_Contiguous_Long_TruncateEllipsis",
+			text:  "foobar\nfoobarfoobarfoobar\nfoobarfoobar\n",
+			trunc: fyne.TextTruncateEllipsis,
+			want: [][2]int{
+				{0, 6},
+				{7, 14},
+				{26, 33},
+				{39, 39},
+			},
+			ellipses: 2,
+		},
 		{
 			name: "Multiple_Contiguous_Long_WrapBreak",
 			text: "foobar\nfoobarfoobarfoobar\nfoobarfoobar\n",
@@ -674,7 +802,27 @@ func TestText_lineBounds(t *testing.T) {
 		{
 			name: "Multiple_Trailing_Short_Truncate",
 			text: "foo\nbar\n",
-			wrap: fyne.TextTruncate,
+			wrap: fyne.TextWrap(fyne.TextTruncateClip),
+			want: [][2]int{
+				{0, 3},
+				{4, 7},
+				{8, 8},
+			},
+		},
+		{
+			name:  "Multiple_Trailing_Short_TruncateClip",
+			text:  "foo\nbar\n",
+			trunc: fyne.TextTruncateClip,
+			want: [][2]int{
+				{0, 3},
+				{4, 7},
+				{8, 8},
+			},
+		},
+		{
+			name:  "Multiple_Trailing_Short_TruncateEllipsis",
+			text:  "foo\nbar\n",
+			trunc: fyne.TextTruncateEllipsis,
 			want: [][2]int{
 				{0, 3},
 				{4, 7},
@@ -701,6 +849,27 @@ func TestText_lineBounds(t *testing.T) {
 				{8, 8},
 			},
 		},
+		{
+			name:  "Multiple_Trailing_Short_WrapTruncateClip",
+			text:  "foo\nbar\nbaz",
+			wrap:  fyne.TextWrapBreak,
+			trunc: fyne.TextTruncateClip,
+			want: [][2]int{
+				{0, 3},
+				{4, 7},
+			},
+		},
+		{
+			name:  "Multiple_Trailing_Short_WrapTruncateEllipsis",
+			text:  "foo\nbar\nbaz",
+			wrap:  fyne.TextWrapBreak,
+			trunc: fyne.TextTruncateEllipsis,
+			want: [][2]int{
+				{0, 3},
+				{4, 7},
+			},
+			ellipses: 0,
+		},
 		{
 			name: "Multiple_Trailing_Long_WrapOff",
 			text: "foobar\nfoobar foobar foobar\nfoobar foobar\n",
@@ -715,7 +884,7 @@ func TestText_lineBounds(t *testing.T) {
 		{
 			name: "Multiple_Trailing_Long_Truncate",
 			text: "foobar\nfoobar foobar foobar\nfoobar foobar\n",
-			wrap: fyne.TextTruncate,
+			wrap: fyne.TextWrap(fyne.TextTruncateClip),
 			want: [][2]int{
 				{0, 6},
 				{7, 17},
@@ -723,6 +892,29 @@ func TestText_lineBounds(t *testing.T) {
 				{42, 42},
 			},
 		},
+		{
+			name:  "Multiple_Trailing_Long_TruncateClip",
+			text:  "foobar\nfoobar foobar foobar\nfoobar foobar\n",
+			trunc: fyne.TextTruncateClip,
+			want: [][2]int{
+				{0, 6},
+				{7, 17},
+				{28, 38},
+				{42, 42},
+			},
+		},
+		{
+			name:  "Multiple_Trailing_Long_TruncateEllipsis",
+			text:  "foobar\nfoobar foobar foobar\nfoobar foobar\n",
+			trunc: fyne.TextTruncateEllipsis,
+			want: [][2]int{
+				{0, 6},
+				{7, 15},
+				{28, 36},
+				{42, 42},
+			},
+			ellipses: 2,
+		},
 		{
 			name: "Multiple_Trailing_Long_WrapBreak",
 			text: "foobar\nfoobar foobar foobar\nfoobar foobar\n",
@@ -750,24 +942,75 @@ func TestText_lineBounds(t *testing.T) {
 				{42, 42},
 			},
 		},
+		{
+			name:  "Multiple_Trailing_Long_WrapTruncateClip",
+			text:  "foo\nfoobar foobar foobar\nbaz",
+			wrap:  fyne.TextWrapBreak,
+			trunc: fyne.TextTruncateClip,
+			want: [][2]int{
+				{0, 3},
+				{4, 14},
+			},
+		},
+		{
+			name:  "Multiple_Trailing_Long_WrapTruncateEllipsis",
+			text:  "foo\nfoobar foobar foobar\nbazzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz",
+			wrap:  fyne.TextWrapBreak,
+			trunc: fyne.TextTruncateEllipsis,
+			want: [][2]int{
+				{0, 3},
+				{4, 14},
+				{14, 23},
+			},
+			ellipses: 1,
+		},
+		{
+			name:  "Multiple_Trailing_Long_WrapWordTruncateEllipsis",
+			text:  "foo\nfoobar foobar foobar\nbaz",
+			wrap:  fyne.TextWrapWord,
+			trunc: fyne.TextTruncateEllipsis,
+			want: [][2]int{
+				{0, 3},
+				{4, 10},
+				{11, 17},
+			},
+			ellipses: 1,
+		},
+		{
+			name:  "Multi_byte_ellipsis_not_truncated",
+			text:  "🪃 234",
+			trunc: fyne.TextTruncateEllipsis,
+			wrap:  fyne.TextWrapOff,
+			want: [][2]int{
+				{0, 5},
+			},
+			ellipses: 0,
+		},
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			got := lineBounds(&TextSegment{Text: tt.text}, tt.wrap, 10, 10, mockMeasurer)
+			ellipses := 0
+			got, _ := lineBounds(&TextSegment{Text: tt.text}, tt.wrap, tt.trunc, 76, fyne.NewSize(76, 64), measurer)
 			for i, wantRow := range tt.want {
 				assert.Equal(t, wantRow[0], got[i].begin)
 				assert.Equal(t, wantRow[1], got[i].end)
+
+				if got[i].ellipsis {
+					ellipses++
+				}
 			}
+			assert.Equal(t, tt.ellipses, ellipses)
 		})
 	}
 }
 
 func TestText_lineBounds_variable_char_width(t *testing.T) {
 	tests := []struct {
-		name string
-		text string
-		wrap fyne.TextWrap
-		want [][2]int
+		name  string
+		text  string
+		wrap  fyne.TextWrap
+		trunc fyne.TextTruncation
+		want  [][2]int
 	}{
 		{
 			name: "IM_WrapOff",
@@ -780,11 +1023,27 @@ func TestText_lineBounds_variable_char_width(t *testing.T) {
 		{
 			name: "IM_Truncate",
 			text: "iiiiiiiiiimmmmmmmmmm",
-			wrap: fyne.TextTruncate,
+			wrap: fyne.TextWrap(fyne.TextTruncateClip),
 			want: [][2]int{
 				{0, 12},
 			},
 		},
+		{
+			name:  "IM_TruncateClip",
+			text:  "iiiiiiiiiimmmmmmmmmm",
+			trunc: fyne.TextTruncateClip,
+			want: [][2]int{
+				{0, 12},
+			},
+		},
+		{
+			name:  "IM_TruncateEllipsis",
+			text:  "iiiiiiiiiimmmmmmmmmm",
+			trunc: fyne.TextTruncateEllipsis,
+			want: [][2]int{
+				{0, 8},
+			},
+		},
 		{
 			name: "IM_WrapBreak",
 			text: "iiiiiiiiiimmmmmmmmmm",
@@ -808,12 +1067,12 @@ func TestText_lineBounds_variable_char_width(t *testing.T) {
 	}
 	textSize := float32(10)
 	textStyle := fyne.TextStyle{}
-	measurer := func(text []rune) float32 {
-		return fyne.MeasureText(string(text), textSize, textStyle).Width
+	measurer := func(text []rune) fyne.Size {
+		return fyne.MeasureText(string(text), textSize, textStyle)
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			got := lineBounds(&TextSegment{Text: tt.text}, tt.wrap, 46, 46, measurer)
+			got, _ := lineBounds(&TextSegment{Text: tt.text}, tt.wrap, tt.trunc, 46, fyne.NewSize(46, 184), measurer)
 			for i, wantRow := range tt.want {
 				assert.Equal(t, wantRow[0], got[i].begin)
 				assert.Equal(t, wantRow[1], got[i].end)
