@@ -1,6 +1,8 @@
 package widget
 
 import (
+	"image/color"
+
 	"fyne.io/fyne/v2"
 	"fyne.io/fyne/v2/canvas"
 	"fyne.io/fyne/v2/theme"
@@ -13,6 +15,8 @@ var _ fyne.Widget = (*Separator)(nil)
 // Since: 1.4
 type Separator struct {
 	BaseWidget
+
+	invert bool
 }
 
 // NewSeparator creates a new separator.
@@ -29,7 +33,16 @@ func NewSeparator() *Separator {
 // Implements: fyne.Widget
 func (s *Separator) CreateRenderer() fyne.WidgetRenderer {
 	s.ExtendBaseWidget(s)
-	bar := canvas.NewRectangle(theme.SeparatorColor())
+	th := s.Theme()
+	v := fyne.CurrentApp().Settings().ThemeVariant()
+	var col color.Color
+	if s.invert {
+		col = th.Color(theme.ColorNameForeground, v)
+	} else {
+		col = th.Color(theme.ColorNameSeparator, v)
+	}
+	bar := canvas.NewRectangle(col)
+
 	return &separatorRenderer{
 		WidgetRenderer: NewSimpleRenderer(bar),
 		bar:            bar,
@@ -42,8 +55,7 @@ func (s *Separator) CreateRenderer() fyne.WidgetRenderer {
 // Implements: fyne.Widget
 func (s *Separator) MinSize() fyne.Size {
 	s.ExtendBaseWidget(s)
-	t := theme.SeparatorThicknessSize()
-	return fyne.NewSize(t, t)
+	return s.BaseWidget.MinSize()
 }
 
 var _ fyne.WidgetRenderer = (*separatorRenderer)(nil)
@@ -55,11 +67,17 @@ type separatorRenderer struct {
 }
 
 func (r *separatorRenderer) MinSize() fyne.Size {
-	t := theme.SeparatorThicknessSize()
-	return fyne.NewSize(t, t)
+	return fyne.NewSquareSize(r.d.Theme().Size(theme.SizeNameSeparatorThickness))
 }
 
 func (r *separatorRenderer) Refresh() {
-	r.bar.FillColor = theme.SeparatorColor()
+	th := r.d.Theme()
+	v := fyne.CurrentApp().Settings().ThemeVariant()
+
+	if r.d.invert {
+		r.bar.FillColor = th.Color(theme.ColorNameForeground, v)
+	} else {
+		r.bar.FillColor = th.Color(theme.ColorNameSeparator, v)
+	}
 	canvas.Refresh(r.d)
 }
