@@ -1,17 +1,25 @@
 package widget
 
 import (
+	"fmt"
+
 	"fyne.io/fyne/v2"
 	"fyne.io/fyne/v2/canvas"
+	"fyne.io/fyne/v2/data/binding"
 	"fyne.io/fyne/v2/driver/desktop"
 	"fyne.io/fyne/v2/internal/cache"
 	"fyne.io/fyne/v2/internal/widget"
 	"fyne.io/fyne/v2/theme"
 )
 
+// allTreeNodesID represents all tree nodes when refreshing requested nodes
+const allTreeNodesID = "_ALLNODES"
+
 // TreeNodeID represents the unique id of a tree node.
 type TreeNodeID = string
 
+// Declare conformity with interfaces
+var _ fyne.Focusable = (*Tree)(nil)
 var _ fyne.Widget = (*Tree)(nil)
 
 // Tree widget displays hierarchical data.
@@ -22,6 +30,11 @@ type Tree struct {
 	BaseWidget
 	Root TreeNodeID
 
+	// HideSeparators hides the separators between tree nodes
+	//
+	// Since: 2.5
+	HideSeparators bool
+
 	ChildUIDs      func(uid TreeNodeID) (c []TreeNodeID)                     `json:"-"` // Return a sorted slice of Children TreeNodeIDs for the given Node TreeNodeID
 	CreateNode     func(branch bool) (o fyne.CanvasObject)                   `json:"-"` // Return a CanvasObject that can represent a Branch (if branch is true), or a Leaf (if branch is false)
 	IsBranch       func(uid TreeNodeID) (ok bool)                            `json:"-"` // Return true if the given TreeNodeID represents a Branch
@@ -32,6 +45,8 @@ type Tree struct {
 	UpdateNode     func(uid TreeNodeID, branch bool, node fyne.CanvasObject) `json:"-"` // Called to update the given CanvasObject to represent the data at the given TreeNodeID
 
 	branchMinSize fyne.Size
+	currentFocus  TreeNodeID
+	focused       bool
 	leafMinSize   fyne.Size
 	offset        fyne.Position
 	open          map[TreeNodeID]bool
@@ -52,6 +67,30 @@ func NewTree(childUIDs func(TreeNodeID) []TreeNodeID, isBranch func(TreeNodeID)
 	return t
 }
 
+// NewTreeWithData creates a new tree widget that will display the contents of the provided data.
+//
+// Since: 2.4
+func NewTreeWithData(data binding.DataTree, createItem func(bool) fyne.CanvasObject, updateItem func(binding.DataItem, bool, fyne.CanvasObject)) *Tree {
+	t := NewTree(
+		data.ChildIDs,
+		func(id TreeNodeID) bool {
+			children := data.ChildIDs(id)
+			return len(children) > 0
+		},
+		createItem,
+		func(i TreeNodeID, branch bool, o fyne.CanvasObject) {
+			item, err := data.GetItem(i)
+			if err != nil {
+				fyne.LogError(fmt.Sprintf("Error getting data item %s", i), err)
+				return
+			}
+			updateItem(item, branch, o)
+		})
+
+	data.AddListener(binding.NewDataListener(t.Refresh))
+	return t
+}
+
 // NewTreeWithStrings creates a new tree with the given string map.
 // Data must contain a mapping for the root, which defaults to empty string ("").
 //
@@ -126,16 +165,56 @@ func (t *Tree) IsBranchOpen(uid TreeNodeID) bool {
 	return t.open[uid]
 }
 
+// FocusGained is called after this Tree has gained focus.
+//
+// Implements: fyne.Focusable
+func (t *Tree) FocusGained() {
+	if t.currentFocus == "" {
+		if childUIDs := t.ChildUIDs; childUIDs != nil {
+			if ids := childUIDs(""); len(ids) > 0 {
+				t.currentFocus = ids[0]
+			}
+		}
+	}
+
+	t.focused = true
+	t.ScrollTo(t.currentFocus)
+	t.RefreshItem(t.currentFocus)
+}
+
+// FocusLost is called after this Tree has lost focus.
+//
+// Implements: fyne.Focusable
+func (t *Tree) FocusLost() {
+	t.focused = false
+	t.Refresh() // Item(t.currentFocus)
+}
+
 // MinSize returns the size that this widget should not shrink below.
 func (t *Tree) MinSize() fyne.Size {
 	t.ExtendBaseWidget(t)
 	return t.BaseWidget.MinSize()
 }
 
+// RefreshItem refreshes a single item, specified by the item ID passed in.
+//
+// Since: 2.4
+func (t *Tree) RefreshItem(id TreeNodeID) {
+	if t.scroller == nil {
+		return
+	}
+	r := cache.Renderer(t.scroller.Content.(*treeContent))
+	if r == nil {
+		return
+	}
+
+	r.(*treeContentRenderer).refreshForID(id)
+}
+
 // OpenAllBranches opens all branches in the tree.
 func (t *Tree) OpenAllBranches() {
 	t.ensureOpenMap()
-	t.walkAll(func(uid string, branch bool, depth int) {
+	t.walkAll(func(uid, parent TreeNodeID, branch bool, depth int) {
 		if branch {
 			t.propertyLock.Lock()
 			t.open[uid] = true
@@ -159,17 +238,11 @@ func (t *Tree) OpenBranch(uid TreeNodeID) {
 
 // Resize sets a new size for a widget.
 func (t *Tree) Resize(size fyne.Size) {
-	t.propertyLock.RLock()
-	s := t.size
-	t.propertyLock.RUnlock()
-
-	if s == size {
+	if size == t.size.Load() {
 		return
 	}
 
-	t.propertyLock.Lock()
-	t.size = size
-	t.propertyLock.Unlock()
+	t.size.Store(size)
 
 	t.Refresh() // trigger a redraw
 }
@@ -252,6 +325,81 @@ func (t *Tree) ToggleBranch(uid string) {
 	}
 }
 
+// TypedKey is called if a key event happens while this Tree is focused.
+//
+// Implements: fyne.Focusable
+func (t *Tree) TypedKey(event *fyne.KeyEvent) {
+	switch event.Name {
+	case fyne.KeySpace:
+		t.Select(t.currentFocus)
+	case fyne.KeyDown:
+		t.RefreshItem(t.currentFocus)
+		next := false
+		t.walk(t.Root, "", 0, func(id, p TreeNodeID, _ bool, _ int) {
+			if next {
+				t.currentFocus = id
+				next = false
+			} else if id == t.currentFocus {
+				next = true
+			}
+		})
+
+		t.ScrollTo(t.currentFocus)
+		t.RefreshItem(t.currentFocus)
+	case fyne.KeyLeft:
+		// If the current focus is on a branch which is open, just close it
+		if t.IsBranch(t.currentFocus) && t.IsBranchOpen(t.currentFocus) {
+			t.CloseBranch(t.currentFocus)
+		} else {
+			// Every other case should move the focus to the current parent node
+			t.walk(t.Root, "", 0, func(id, p TreeNodeID, _ bool, _ int) {
+				if id == t.currentFocus && p != "" {
+					t.currentFocus = p
+				}
+			})
+		}
+
+		t.RefreshItem(t.currentFocus)
+		t.ScrollTo(t.currentFocus)
+		t.RefreshItem(t.currentFocus)
+	case fyne.KeyRight:
+		if t.IsBranch(t.currentFocus) {
+			t.OpenBranch(t.currentFocus)
+		}
+		children := []TreeNodeID{}
+		if childUIDs := t.ChildUIDs; childUIDs != nil {
+			children = childUIDs(t.currentFocus)
+		}
+
+		if len(children) > 0 {
+			t.currentFocus = children[0]
+		}
+
+		t.RefreshItem(t.currentFocus)
+		t.ScrollTo(t.currentFocus)
+		t.RefreshItem(t.currentFocus)
+	case fyne.KeyUp:
+		t.RefreshItem(t.currentFocus)
+		previous := ""
+		t.walk(t.Root, "", 0, func(id, p TreeNodeID, _ bool, _ int) {
+			if id == t.currentFocus && previous != "" {
+				t.currentFocus = previous
+			}
+			previous = id
+		})
+
+		t.ScrollTo(t.currentFocus)
+		t.RefreshItem(t.currentFocus)
+	}
+}
+
+// TypedRune is called if a text event happens while this Tree is focused.
+//
+// Implements: fyne.Focusable
+func (t *Tree) TypedRune(_ rune) {
+	// intentionally left blank
+}
+
 // Unselect marks the specified node to be not selected.
 func (t *Tree) Unselect(uid TreeNodeID) {
 	if len(t.selected) == 0 || t.selected[0] != uid {
@@ -292,8 +440,8 @@ func (t *Tree) ensureOpenMap() {
 }
 
 func (t *Tree) findBottom() (y float32, size fyne.Size) {
-	sep := theme.Padding()
-	t.walkAll(func(id TreeNodeID, branch bool, _ int) {
+	sep := t.Theme().Size(theme.SizeNamePadding)
+	t.walkAll(func(id, _ TreeNodeID, branch bool, _ int) {
 		size = t.leafMinSize
 		if branch {
 			size = t.branchMinSize
@@ -319,7 +467,9 @@ func (t *Tree) findBottom() (y float32, size fyne.Size) {
 }
 
 func (t *Tree) offsetAndSize(uid TreeNodeID) (y float32, size fyne.Size, found bool) {
-	t.walkAll(func(id TreeNodeID, branch bool, _ int) {
+	pad := t.Theme().Size(theme.SizeNamePadding)
+
+	t.walkAll(func(id, _ TreeNodeID, branch bool, _ int) {
 		m := t.leafMinSize
 		if branch {
 			m = t.branchMinSize
@@ -335,7 +485,7 @@ func (t *Tree) offsetAndSize(uid TreeNodeID) (y float32, size fyne.Size, found b
 			}
 			// If this is not the first item, add a separator
 			if y > 0 {
-				y += theme.Padding()
+				y += pad
 			}
 
 			y += m.Height
@@ -352,26 +502,26 @@ func (t *Tree) offsetUpdated(pos fyne.Position) {
 	t.scroller.Content.Refresh()
 }
 
-func (t *Tree) walk(uid string, depth int, onNode func(string, bool, int)) {
+func (t *Tree) walk(uid, parent TreeNodeID, depth int, onNode func(TreeNodeID, TreeNodeID, bool, int)) {
 	if isBranch := t.IsBranch; isBranch != nil {
 		if isBranch(uid) {
-			onNode(uid, true, depth)
+			onNode(uid, parent, true, depth)
 			if t.IsBranchOpen(uid) {
 				if childUIDs := t.ChildUIDs; childUIDs != nil {
 					for _, c := range childUIDs(uid) {
-						t.walk(c, depth+1, onNode)
+						t.walk(c, uid, depth+1, onNode)
 					}
 				}
 			}
 		} else {
-			onNode(uid, false, depth)
+			onNode(uid, parent, false, depth)
 		}
 	}
 }
 
 // walkAll visits every open node of the tree and calls the given callback with TreeNodeID, whether node is branch, and the depth of node.
-func (t *Tree) walkAll(onNode func(TreeNodeID, bool, int)) {
-	t.walk(t.Root, 0, onNode)
+func (t *Tree) walkAll(onNode func(TreeNodeID, TreeNodeID, bool, int)) {
+	t.walk(t.Root, "", 0, onNode)
 }
 
 var _ fyne.WidgetRenderer = (*treeRenderer)(nil)
@@ -410,8 +560,11 @@ func (r *treeRenderer) Refresh() {
 
 func (r *treeRenderer) updateMinSizes() {
 	if f := r.tree.CreateNode; f != nil {
-		r.tree.branchMinSize = newBranch(r.tree, f(true)).MinSize()
-		r.tree.leafMinSize = newLeaf(r.tree, f(false)).MinSize()
+		branch := createItemAndApplyThemeScope(func() fyne.CanvasObject { return f(true) }, r.tree)
+		r.tree.branchMinSize = newBranch(r.tree, branch).MinSize()
+
+		leaf := createItemAndApplyThemeScope(func() fyne.CanvasObject { return f(false) }, r.tree)
+		r.tree.leafMinSize = newLeaf(r.tree, leaf).MinSize()
 	}
 }
 
@@ -443,17 +596,11 @@ func (c *treeContent) CreateRenderer() fyne.WidgetRenderer {
 }
 
 func (c *treeContent) Resize(size fyne.Size) {
-	c.propertyLock.RLock()
-	s := c.size
-	c.propertyLock.RUnlock()
-
-	if s == size {
+	if size == c.size.Load() {
 		return
 	}
 
-	c.propertyLock.Lock()
-	c.size = size
-	c.propertyLock.Unlock()
+	c.size.Store(size)
 
 	c.Refresh() // trigger a redraw
 }
@@ -472,6 +619,7 @@ type treeContentRenderer struct {
 }
 
 func (r *treeContentRenderer) Layout(size fyne.Size) {
+	th := r.treeContent.Theme()
 	r.treeContent.propertyLock.Lock()
 	defer r.treeContent.propertyLock.Unlock()
 
@@ -479,16 +627,18 @@ func (r *treeContentRenderer) Layout(size fyne.Size) {
 	branches := make(map[string]*branch)
 	leaves := make(map[string]*leaf)
 
+	pad := th.Size(theme.SizeNamePadding)
 	offsetY := r.treeContent.tree.offset.Y
 	viewport := r.treeContent.viewport
 	width := fyne.Max(size.Width, viewport.Width)
 	separatorCount := 0
-	separatorThickness := theme.SeparatorThicknessSize()
+	separatorThickness := th.Size(theme.SizeNameSeparatorThickness)
 	separatorSize := fyne.NewSize(width, separatorThickness)
-	separatorOff := (theme.Padding() - separatorThickness) / 2
+	separatorOff := (pad + separatorThickness) / 2
+	hideSeparators := r.treeContent.tree.HideSeparators
 	y := float32(0)
 	// walkAll open branches and obtain nodes to render in scroller's viewport
-	r.treeContent.tree.walkAll(func(uid string, isBranch bool, depth int) {
+	r.treeContent.tree.walkAll(func(uid, _ string, isBranch bool, depth int) {
 		// Root node is not rendered unless it has been customized
 		if r.treeContent.tree.Root == "" {
 			depth = depth - 1
@@ -499,19 +649,9 @@ func (r *treeContentRenderer) Layout(size fyne.Size) {
 		}
 
 		// If this is not the first item, add a separator
-		if y > 0 {
-			var separator fyne.CanvasObject
-			if separatorCount < len(r.separators) {
-				separator = r.separators[separatorCount]
-			} else {
-				separator = NewSeparator()
-				r.separators = append(r.separators, separator)
-			}
-			separator.Move(fyne.NewPos(0, y+separatorOff))
-			separator.Resize(separatorSize)
-			separator.Show()
-			r.objects = append(r.objects, separator)
-			y += theme.Padding()
+		addSeparator := y > 0
+		if addSeparator {
+			y += pad
 			separatorCount++
 		}
 
@@ -525,6 +665,22 @@ func (r *treeContentRenderer) Layout(size fyne.Size) {
 			// Node is below viewport and not visible
 		} else {
 			// Node is in viewport
+
+			if addSeparator && !hideSeparators {
+				var separator fyne.CanvasObject
+				if separatorCount < len(r.separators) {
+					separator = r.separators[separatorCount]
+					separator.Show() // it may previously have been hidden
+				} else {
+					separator = NewSeparator()
+					r.separators = append(r.separators, separator)
+				}
+				separator.Move(fyne.NewPos(0, y-separatorOff))
+				separator.Resize(separatorSize)
+				r.objects = append(r.objects, separator)
+				separatorCount++
+			}
+
 			var n fyne.CanvasObject
 			if isBranch {
 				b, ok := r.branches[uid]
@@ -559,6 +715,10 @@ func (r *treeContentRenderer) Layout(size fyne.Size) {
 		y += m.Height
 	})
 
+	if hideSeparators {
+		// start below iteration from 0 to hide all separators
+		separatorCount = 0
+	}
 	// Hide any separators that haven't been reused
 	for ; separatorCount < len(r.separators); separatorCount++ {
 		r.separators[separatorCount].Hide()
@@ -567,13 +727,11 @@ func (r *treeContentRenderer) Layout(size fyne.Size) {
 	// Release any nodes that haven't been reused
 	for uid, b := range r.branches {
 		if _, ok := branches[uid]; !ok {
-			b.Hide()
 			r.branchPool.Release(b)
 		}
 	}
 	for uid, l := range r.leaves {
 		if _, ok := leaves[uid]; !ok {
-			l.Hide()
 			r.leafPool.Release(l)
 		}
 	}
@@ -583,10 +741,13 @@ func (r *treeContentRenderer) Layout(size fyne.Size) {
 }
 
 func (r *treeContentRenderer) MinSize() (min fyne.Size) {
+	th := r.treeContent.Theme()
 	r.treeContent.propertyLock.Lock()
 	defer r.treeContent.propertyLock.Unlock()
+	pad := th.Size(theme.SizeNamePadding)
+	iconSize := th.Size(theme.SizeNameInlineIcon)
 
-	r.treeContent.tree.walkAll(func(uid string, isBranch bool, depth int) {
+	r.treeContent.tree.walkAll(func(uid, _ string, isBranch bool, depth int) {
 		// Root node is not rendered unless it has been customized
 		if r.treeContent.tree.Root == "" {
 			depth = depth - 1
@@ -598,14 +759,14 @@ func (r *treeContentRenderer) MinSize() (min fyne.Size) {
 
 		// If this is not the first item, add a separator
 		if min.Height > 0 {
-			min.Height += theme.Padding()
+			min.Height += pad
 		}
 
 		m := r.treeContent.tree.leafMinSize
 		if isBranch {
 			m = r.treeContent.tree.branchMinSize
 		}
-		m.Width += float32(depth) * (theme.IconInlineSize() + theme.Padding())
+		m.Width += float32(depth) * (iconSize + pad)
 		min.Width = fyne.Max(min.Width, m.Width)
 		min.Height += m.Height
 	})
@@ -617,6 +778,10 @@ func (r *treeContentRenderer) Objects() []fyne.CanvasObject {
 }
 
 func (r *treeContentRenderer) Refresh() {
+	r.refreshForID(allTreeNodesID)
+}
+
+func (r *treeContentRenderer) refreshForID(toDraw TreeNodeID) {
 	s := r.treeContent.Size()
 	if s.IsZero() {
 		r.treeContent.Resize(r.treeContent.MinSize().Max(r.treeContent.tree.Size()))
@@ -624,10 +789,18 @@ func (r *treeContentRenderer) Refresh() {
 		r.Layout(s)
 	}
 	r.treeContent.propertyLock.RLock()
-	for _, b := range r.branches {
+	for id, b := range r.branches {
+		if toDraw != allTreeNodesID && id != toDraw {
+			continue
+		}
+
 		b.Refresh()
 	}
-	for _, l := range r.leaves {
+	for id, l := range r.leaves {
+		if toDraw != allTreeNodesID && id != toDraw {
+			continue
+		}
+
 		l.Refresh()
 	}
 	r.treeContent.propertyLock.RUnlock()
@@ -641,7 +814,7 @@ func (r *treeContentRenderer) getBranch() (b *branch) {
 	} else {
 		var content fyne.CanvasObject
 		if f := r.treeContent.tree.CreateNode; f != nil {
-			content = f(true)
+			content = createItemAndApplyThemeScope(func() fyne.CanvasObject { return f(true) }, r.treeContent.tree)
 		}
 		b = newBranch(r.treeContent.tree, content)
 	}
@@ -655,7 +828,7 @@ func (r *treeContentRenderer) getLeaf() (l *leaf) {
 	} else {
 		var content fyne.CanvasObject
 		if f := r.treeContent.tree.CreateNode; f != nil {
-			content = f(false)
+			content = createItemAndApplyThemeScope(func() fyne.CanvasObject { return f(false) }, r.treeContent.tree)
 		}
 		l = newLeaf(r.treeContent.tree, content)
 	}
@@ -682,7 +855,11 @@ func (n *treeNode) Content() fyne.CanvasObject {
 }
 
 func (n *treeNode) CreateRenderer() fyne.WidgetRenderer {
-	background := canvas.NewRectangle(theme.HoverColor())
+	th := n.Theme()
+	v := fyne.CurrentApp().Settings().ThemeVariant()
+
+	background := canvas.NewRectangle(th.Color(theme.ColorNameHover, v))
+	background.CornerRadius = th.Size(theme.SizeNameSelectionRadius)
 	background.Hide()
 	return &treeNodeRenderer{
 		BaseRenderer: widget.BaseRenderer{},
@@ -692,7 +869,8 @@ func (n *treeNode) CreateRenderer() fyne.WidgetRenderer {
 }
 
 func (n *treeNode) Indent() float32 {
-	return float32(n.depth) * (theme.IconInlineSize() + theme.Padding())
+	th := n.Theme()
+	return float32(n.depth) * (th.Size(theme.SizeNameInlineIcon) + th.Size(theme.SizeNamePadding))
 }
 
 // MouseIn is called when a desktop pointer enters the widget
@@ -712,7 +890,19 @@ func (n *treeNode) MouseOut() {
 }
 
 func (n *treeNode) Tapped(*fyne.PointEvent) {
+	if n.tree.currentFocus != "" {
+		n.tree.RefreshItem(n.tree.currentFocus)
+	}
+
 	n.tree.Select(n.uid)
+	if !fyne.CurrentDevice().IsMobile() {
+		canvas := fyne.CurrentApp().Driver().CanvasForObject(n.tree)
+		if canvas != nil {
+			canvas.Focus(n.tree)
+		}
+		n.tree.currentFocus = n.uid
+		n.Refresh()
+	}
 }
 
 func (n *treeNode) partialRefresh() {
@@ -739,15 +929,18 @@ type treeNodeRenderer struct {
 }
 
 func (r *treeNodeRenderer) Layout(size fyne.Size) {
-	x := theme.Padding() + r.treeNode.Indent()
+	th := r.treeNode.Theme()
+	pad := th.Size(theme.SizeNamePadding)
+	iconSize := th.Size(theme.SizeNameInlineIcon)
+	x := pad + r.treeNode.Indent()
 	y := float32(0)
 	r.background.Resize(size)
 	if r.treeNode.icon != nil {
 		r.treeNode.icon.Move(fyne.NewPos(x, y))
-		r.treeNode.icon.Resize(fyne.NewSize(theme.IconInlineSize(), size.Height))
+		r.treeNode.icon.Resize(fyne.NewSize(iconSize, size.Height))
 	}
-	x += theme.IconInlineSize()
-	x += theme.Padding()
+	x += iconSize
+	x += pad
 	if r.treeNode.content != nil {
 		r.treeNode.content.Move(fyne.NewPos(x, y))
 		r.treeNode.content.Resize(fyne.NewSize(size.Width-x, size.Height))
@@ -758,8 +951,11 @@ func (r *treeNodeRenderer) MinSize() (min fyne.Size) {
 	if r.treeNode.content != nil {
 		min = r.treeNode.content.MinSize()
 	}
-	min.Width += theme.InnerPadding() + r.treeNode.Indent() + theme.IconInlineSize()
-	min.Height = fyne.Max(min.Height, theme.IconInlineSize())
+	th := r.treeNode.Theme()
+	iconSize := th.Size(theme.SizeNameInlineIcon)
+
+	min.Width += th.Size(theme.SizeNameInnerPadding) + r.treeNode.Indent() + iconSize
+	min.Height = fyne.Max(min.Height, iconSize)
 	return
 }
 
@@ -784,20 +980,24 @@ func (r *treeNodeRenderer) Refresh() {
 }
 
 func (r *treeNodeRenderer) partialRefresh() {
+	th := r.treeNode.Theme()
+	v := fyne.CurrentApp().Settings().ThemeVariant()
+
 	if r.treeNode.icon != nil {
 		r.treeNode.icon.Refresh()
 	}
+	r.background.CornerRadius = th.Size(theme.SizeNameSelectionRadius)
 	if len(r.treeNode.tree.selected) > 0 && r.treeNode.uid == r.treeNode.tree.selected[0] {
-		r.background.FillColor = theme.SelectionColor()
+		r.background.FillColor = th.Color(theme.ColorNameSelection, v)
 		r.background.Show()
-	} else if r.treeNode.hovered {
-		r.background.FillColor = theme.HoverColor()
+	} else if r.treeNode.hovered || (r.treeNode.tree.focused && r.treeNode.tree.currentFocus == r.treeNode.uid) {
+		r.background.FillColor = th.Color(theme.ColorNameHover, v)
 		r.background.Show()
 	} else {
 		r.background.Hide()
 	}
 	r.background.Refresh()
-	r.Layout(r.treeNode.size)
+	r.Layout(r.treeNode.Size())
 	canvas.Refresh(r.treeNode.super())
 }
 
@@ -817,12 +1017,16 @@ func newBranch(tree *Tree, content fyne.CanvasObject) (b *branch) {
 		},
 	}
 	b.ExtendBaseWidget(b)
+
+	if cache.OverrideThemeMatchingScope(b, tree) {
+		b.Refresh()
+	}
 	return
 }
 
 func (b *branch) update(uid string, depth int) {
 	b.treeNode.update(uid, depth)
-	b.icon.(*branchIcon).update(uid, depth)
+	b.icon.(*branchIcon).update(uid)
 }
 
 var _ fyne.Tappable = (*branchIcon)(nil)
@@ -854,7 +1058,7 @@ func (i *branchIcon) Tapped(*fyne.PointEvent) {
 	i.tree.ToggleBranch(i.uid)
 }
 
-func (i *branchIcon) update(uid string, depth int) {
+func (i *branchIcon) update(uid string) {
 	i.uid = uid
 	i.Refresh()
 }
@@ -874,5 +1078,9 @@ func newLeaf(tree *Tree, content fyne.CanvasObject) (l *leaf) {
 		},
 	}
 	l.ExtendBaseWidget(l)
+
+	if cache.OverrideThemeMatchingScope(l, tree) {
+		l.Refresh()
+	}
 	return
 }
